/*
 *
 *    ANY.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/ANY.c,v 
 *    $Revision: 1.15 
 *    $Date: 1997/06/15 22:37:59 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */


#include <alloc_ctx.h>
#include <define.h>
#include <io_ctx.h>  
#include <io_ctx_memio.h>


#include "asn.h"
#include "asn_error.h"
#include "asn_types.h"
#include "a2c.h"
#include "berencode.h"
#include "ber_length.h"
#include "ber_tag.h"
#include "params.h"

#include "ANY.h"


#ifndef LINT
static char rcsid[] = "$Id: ANY.c,v 1.15 1997/06/15 22:37:59 ekr Exp ";
#endif




static char type_name[] = "ANY";


static A2C_DECODE_FUNCTION_PROTOTYPE(decode_ANY);
A2C_ENCODE_FUNCTION_PROTOTYPE(encode_ANY);
static A2C_VERIFY_FUNCTION_PROTOTYPE(verify_ANY);



static int
decode_ANY(instream, ctx, module, selector, index, data, error)
    io_ctx                    instream;
    struct asn_decode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status     = NO_ERROR;
    int                       nread      = 0;
    int                       tl_length  = 0;
    tag_type                  tag        = 0;
    length_type               length     = 0;
    UCHAR                    *value      = 0;
UNUSED_ARG(error);


    /* for ANY, any tag will be accepted, so check_tag is always NO_ERROR */
    status = check_tag(ctx, module, selector, index);
    switch (status) {
    case NO_ERROR:              /* do nothing */            break;
    case A2C_WARN_WRONG_TAG:    return A2C_WARN_WRONG_TAG;  break;
    default:                    ERETURN(status);            break;
    }

    tag = get_current_tag(ctx);
    length = get_current_length(ctx);
    tl_length = sizeof_encoded_tag(tag) + sizeof_encoded_length(length);

    value = (UCHAR*)AC_malloc(ctx->data_heap, tl_length+length+1);
    if (value == 0) ERETURN(A2C_ERR_NO_MEMORY);

{  /* temporary hack */
    io_ctx  tmp = 0;

    status = create_bound_static_memio_ctx(&tmp, value, tl_length);
    if (status != NO_ERROR) ERETURN(status);

    status = put_tag(tmp, tag);
    if (status != NO_ERROR) ERETURN(status);
    status = put_length(tmp, length);
    if (status != NO_ERROR) ERETURN(status);

    (VOID)IO_delete(&tmp);
}

    while (nread < length) {
        status = IO_fread(((char*)value)+tl_length+nread, 1, length-nread, instream);
        /* not the best way to handle non-blocking IO */
        if (status == 0) ERETURN(A2C_ERR_EOF);

        nread += status;
    }

    ((asn__any*)data)->length  = tl_length+length;
    ((asn__any*)data)->object  = (UCHAR*)value;

    ctx->value.nread = nread;

    return NO_ERROR;
}



int
encode_ANY(outstream, ctx, module, selector, index, data, error)
    io_ctx                    outstream;
    struct asn_encode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status     = NO_ERROR;
    tag_type                  tag        = 0;
    length_type               length     = 0;
    int                       tl_length  = 0;
    io_ctx                    tmp        = 0;
    struct asn__any_         *any        = 0;
    int                       nread      = 0;
    UCHAR                     buffer[16];
    int			      indefinite = 0;
    
    (VOID)query_a2c_ctx(ctx->app_ctx, A2C_CTX_QUERY_INDEFINITE, &indefinite);
    if(indefinite)
      return(ber_encode_ANY(outstream,ctx,module,selector,index,data,error));


    any = (struct asn__any_*)data;

    if (any->length == 0 || any->object == 0) ERETURN(A2C_ERR_VALUE_MISSING);

    status = create_bound_static_memio_ctx(&tmp, any->object, any->length);
    if (status != NO_ERROR) ERETURN(status);

    nread = 0;
    status = get_tag(tmp, buffer, &nread, sizeof(buffer), &tag);
    if (status != NO_ERROR) ERETURN(status);
    tl_length = nread;

    nread = 0;
    status = get_length(tmp, buffer, &nread, sizeof(buffer), &length);
    if (status != NO_ERROR) ERETURN(status);
    tl_length += nread;

    (VOID)IO_delete(&tmp);

    status = IO_fwrite(((char*)(any->object))+tl_length, 1,
                         any->length-tl_length, outstream);
    if (status != (int)any->length-tl_length) ERETURN(A2C_ERR_OUTSTREAM_LEN);

    if (ctx->list_tail->tag == (CLASS_SPECIAL|TAG_ANY)) {
        ctx->list_tail->tag = tag;
        ctx->list_tail->length = length;
    }
    else {
        if (IS_CONSTRUCTED(tag)) {
            ctx->list_tail->tag = SET_CONSTRUCTED(ctx->list_tail->tag);
        } 
    }

    return NO_ERROR;
}





static int
verify_ANY(null, ctx, module, selector, index, data, error)
    io_ctx                    null;
    struct asn_verify_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    struct asn_template_     *template       = 0;
UNUSED_ARG(null);
UNUSED_ARG(ctx);
UNUSED_ARG(data);
UNUSED_ARG(error);


    template = module->types[selector].template;

    if (template[*index].flags & FLAG_BOUNDED) {
ASSERT(!"not implemented");
    }

    return NO_ERROR;
}






PRIVATE_DEF struct vtbl_ a2c_vtbl__ANY =
    { type_name, decode_ANY, encode_ANY, verify_ANY };


