/*
 *
 *    BIT_STRING.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/BIT_STRING.c,v 
 *    $Revision: 1.12 
 *    $Date: 1997/06/06 00:30:24 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */

#include <alloc_ctx.h>
#include <define.h>
#include <io_ctx.h>  


#include "asn.h"
#include "asn_types.h"
#include "BIT_STRING.h"
#include "asn_error.h"


#ifndef LINT
static char rcsid[] = "$Id: BIT_STRING.c,v 1.12 1997/06/06 00:30:24 briank Exp ";
#endif



static char type_name[] = "BIT_STRING";


INLINE static UCHAR    used_bits
  PROTO_LIST((
    UINT4  i,
    int    unused
  ));




static A2C_DECODE_FUNCTION_PROTOTYPE(decode_BIT_STRING); 
static A2C_ENCODE_FUNCTION_PROTOTYPE(encode_BIT_STRING); 
static A2C_VERIFY_FUNCTION_PROTOTYPE(verify_BIT_STRING); 


INLINE static UCHAR 
used_bits(i, unused)
    UINT4  i;
    int    unused;
{
   /* assumes unused between 0 and 7 */

/* TODO: DER constraint */

/* note, if we're using DER, then the unused bits should be zero 
 * already, and thus if we're pedantic we'd cause an error if any
 * are non-zero
 */

    switch (unused) {
    case 0:
        i &= 0xff;
        break;
    case 1:
        i &= 0xfe;
        break;
    case 2:
        i &= 0xfc;
        break;
    case 3:
        i &= 0xf8;
        break;
    case 4:
        i &= 0xf0;
        break;
    case 5:
        i &= 0xe0;
        break;
    case 6:
        i &= 0xc0;
        break;
    case 7:
        i &= 0x80;
        break;
    }
    return i;
}




static int
decode_BIT_STRING(instream, ctx, module, selector, index, data, error)
    io_ctx                    instream;
    struct asn_decode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status         = NO_ERROR;
    struct asn_template_     *template       = 0;
    UCHAR                    *value          = 0;
    int                       nbits          = 0;
    int                       nread          = 0;
    int                       length         = 0;
    UCHAR                     unused         = 0; 
UNUSED_ARG(error);


    status = check_tag(ctx, module, selector, index);
    switch (status) {
    case NO_ERROR:              /* do nothing */            break;
    case A2C_WARN_WRONG_TAG:    return A2C_WARN_WRONG_TAG;  break;
    default:                    ERETURN(status);            break;
    }

    /* DER only */
    if (IS_CONSTRUCTED(get_current_tag(ctx))) ERETURN(A2C_ERR_CONSTRUCTED);

    template = module->types[selector].template;
    length = get_current_length(ctx);

    if (length < 0) ERETURN(SE_ERR_INTERNAL);

    if (length > 0) {

        status = IO_fread((char*)&unused, 1, 1, instream);
        /* not the best way to handle non-blocking IO */
        if (status == 0) ERETURN(A2C_ERR_EOF);
        ctx->value.nread = 1;

        length -= 1;   /* remove the "unused bits" byte */

        if (template[*index].flags & FLAG_BOUNDED) {
            nbits = length * 8 - unused;
            if (nbits > template[*index].max_size) {
                ERETURN(A2C_ERR_MAX_SIZE_STRING);
            }
            if (nbits < template[*index].min_size) {
                ERETURN(A2C_ERR_MIN_SIZE_STRING);
            }
            value = (UCHAR*)data;
        }
        else {
            value = (UCHAR*)AC_calloc(ctx->data_heap, 1, length);
        } 

        while (nread < length) {
            status = IO_fread((char*)(value+nread), 1, length-nread, instream);
            /* not the best way to handle non-blocking IO */
            if (status == 0) ERETURN(A2C_ERR_EOF);
            nread += status;
        }
        ctx->value.nread += nread;
    
        /* clear the unused bits */
        value[nread-1] = used_bits(value[nread-1], unused);
    }

    if (! (template[*index].flags & FLAG_BOUNDED)) {
#if 0
        if (length == 0) {
            /* this data won't be used, but a null pointer isn't allowed */
            value = (UCHAR*)AC_calloc(ctx->data_heap, 1, 1);
        }
#endif
        ((struct asn__bit_string_*)data)->value = value;
        ((struct asn__bit_string_*)data)->length =
                (nread == 0) ? 0 : (nread-1)*8 + (8-(int)unused);
    }

    return NO_ERROR;
}



static int
encode_BIT_STRING(outstream, ctx, module, selector, index, data, error)
    io_ctx                    outstream;
    struct asn_encode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status                = NO_ERROR;
    int                       last_significant_bit  = 0;
    int                       i                     = 0;
    struct asn_template_     *template              = 0;
    UCHAR                    *value                 = 0;
#if 1
    int                       bits                  = 0;
#endif
    int                       bytes                 = 0;
    int                       unused                = 0;
UNUSED_ARG(ctx);
UNUSED_ARG(error);


    template = module->types[selector].template;

    if (template[*index].flags & FLAG_BOUNDED) {
        /* find the last (furthest to the right) non-zero bit, since
         * DER encoding prohibits trailing zeros in named bit lists
         */
        value = (UCHAR*)data;
#if 1
        bits = template[*index].max_size;
        last_significant_bit = bits;
#else
        bytes = template[*index].max_size;
        last_significant_bit = (bytes * 8) - 1;
#endif
        for (i = last_significant_bit; i >= 0; --i) {
            if (BIT_IS_SET(value, i)) break;
        }
        last_significant_bit = i;
    }
    else {
        if (((struct asn__bit_string_*)data)->length != 0) {
            value = ((struct asn__bit_string_*)data)->value;
            if (value == 0) ERETURN(A2C_ERR_VALUE_MISSING);
        }
    
        /*  DER encoding permits trailing zeros in unbounded bit lists 
         *  and the encoder/decoder is not allowed to add or remove 
         *  trailing 0 bits
         */
        last_significant_bit = (((struct asn__bit_string_*)data)->length)-1;
    } 

    if (last_significant_bit < 0) {
        /* a zero length bit string */
        /* do nothing */
    }
    else {
        bytes = (last_significant_bit / 8) + 1;
        unused = 7 - (last_significant_bit % 8);

        status = IO_fputc(unused, outstream);
        if (status == EOF) ERETURN(A2C_ERR_OUTSTREAM_LEN);

        /* dump all bits except the last byte */
        for (i = 0; i < bytes-1; ++i) {
            status = IO_fputc(value[i], outstream);
            if (status == EOF) ERETURN(A2C_ERR_OUTSTREAM_LEN);
        }

        /* clear the unused bits and dump the last byte */
        i = used_bits(value[bytes-1], unused);
        status = IO_fputc(i, outstream);
        if (status == EOF) ERETURN(A2C_ERR_OUTSTREAM_LEN);
    }

    return NO_ERROR;
}




static int
verify_BIT_STRING(null, ctx, module, selector, index, data, error)
    io_ctx                    null;
    struct asn_verify_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
UNUSED_ARG(null);
UNUSED_ARG(ctx);
UNUSED_ARG(module);
UNUSED_ARG(selector);
UNUSED_ARG(index);
UNUSED_ARG(data);
UNUSED_ARG(error);    



    return NO_ERROR;
}



PRIVATE_DEF struct vtbl_ a2c_vtbl__BIT_STRING =
    { type_name, decode_BIT_STRING, encode_BIT_STRING, verify_BIT_STRING };


