/*
 *
 *    BMPString.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/BMPString.c,v 
 *    $Revision: 1.16 
 *    $Date: 1997/06/06 00:30:26 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */

#include <limits.h>

#include <define.h>
#include <io_ctx.h>  

#include "asn.h"
#include "asn_error.h"
#include "anystring.h"
#include "asciichar.h"

#include "BMPString.h"


#ifndef LINT
static char rcsid[] = "$Id: BMPString.c,v 1.16 1997/06/06 00:30:26 briank Exp ";
#endif



static char type_name[] = "BMPString";


static A2C_DECODE_FUNCTION_PROTOTYPE(decode_BMPString); 
static A2C_ENCODE_FUNCTION_PROTOTYPE(encode_BMPString); 
static A2C_VERIFY_FUNCTION_PROTOTYPE(verify_BMPString); 
 



/* run-time check for little endian processor */
static unsigned int   endian_check = 0x1;
#define  LITTLE_ENDIAN_PRX  (((UCHAR*)&endian_check)[0] != 0)


static int endian_mangle_string
  PROTO_LIST((
    asn__bmp_string           bmpstr
  ));

static int
endian_mangle_string(bmpstr)
    asn__bmp_string           bmpstr;
{
    UCHAR                    *str;
    unsigned int              hold;

    for (; *bmpstr != 0; bmpstr++) {
        str = (UCHAR*)bmpstr;
        hold = str[0];
        str[0] = str[1];
        str[1] = hold;
    }
    return NO_ERROR;
}


PRIVATE asn__bmp_string
bmpchr(bmpstr, c)
    asn__bmp_string           bmpstr;
    int                       c;
{
    for (; *bmpstr != 0; bmpstr++) {
        if (*bmpstr == (unsigned int)c) return bmpstr;
    }
    return 0; /* no match found */
}


PRIVATE int
bmpcmp(bmpstr1, bmpstr2)
    asn__bmp_string           bmpstr1;
    asn__bmp_string           bmpstr2;
{
    for (; *bmpstr1 == *bmpstr2; bmpstr1++, bmpstr2++) {
        if (*bmpstr1 == 0 && *bmpstr2 == 0) return 0;
    }
    return (*bmpstr1 - *bmpstr2);
}


PRIVATE asn__bmp_string
bmpdup(src)
    asn__bmp_string           src;
{
    asn__bmp_string           dest;
    int                       status;

    status = COPY_BMP_STRING(standard_heap, &dest, &src);
    if (status != NO_ERROR) ERETURN_NULL(status);

    return dest;
}


PRIVATE int
bmplen(bmpstr)
    asn__bmp_string           bmpstr;
{
    asn__bmp_string           start  = bmpstr;

    while (*bmpstr != 0) {
        bmpstr++;
    }
    return (bmpstr - start);
}


PRIVATE int
bmpstrcmp(bmpstr, str)
    asn__bmp_string           bmpstr;
    char                     *str;
{
    for (; *bmpstr == *(UCHAR*)str; bmpstr++, str++) {
        if (*bmpstr == 0 && *str == 0) return 0;
    }
    return (*bmpstr - *(UCHAR*)str);
}







static int
decode_BMPString(instream, ctx, module, selector, index, data, error)
    io_ctx                    instream;
    struct asn_decode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status          = NO_ERROR;
    struct asn_template_     *template        = 0;
    UCHAR                    *value           = 0;
    int                       length          = 0;
    int                       nread           = 0;
UNUSED_ARG(error);


    status = check_tag(ctx, module, selector, index);
    switch (status) {
    case NO_ERROR:              /* do nothing */            break;
    case A2C_WARN_WRONG_TAG:    return A2C_WARN_WRONG_TAG;  break;
    default:                    ERETURN(status);            break;
    }

    /* DER only */
    if (IS_CONSTRUCTED(get_current_tag(ctx))) ERETURN(A2C_ERR_CONSTRUCTED);

    template = module->types[selector].template;

    length = get_current_length(ctx);
    if (length < 0) ERETURN(SE_ERR_INTERNAL);
    value = (UCHAR*)AC_malloc(ctx->data_heap, length+2);

    nread = 0;
    while (nread < length) {
        status = IO_fread(((char*)value)+nread, 1, length-nread, instream);
        /* not the best way to handle non-blocking IO */
        if (status == 0) ERETURN(A2C_ERR_EOF);

        nread += status;
    }
    ctx->value.nread = nread;

    value[nread] = '\0';
    value[nread+1] = '\0';

    if (LITTLE_ENDIAN_PRX) (VOID)endian_mangle_string((VOID*)value);

    *(UCHAR**)data = value;

    return NO_ERROR;
}



static int
encode_BMPString(outstream, ctx, module, selector, index, data, error)
    io_ctx                    outstream;
    struct asn_encode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status          = NO_ERROR;
    struct asn_template_     *template        = 0;
    asn__bmp_string           value           = 0;
    int                       length          = 0;
UNUSED_ARG(ctx);
UNUSED_ARG(index);
UNUSED_ARG(error);


    template = module->types[selector].template;

    value = *(asn__bmp_string*)data;

    if (value == 0) ERETURN(A2C_ERR_VALUE_MISSING);

    length = bmplen(value)*2;  /* 2 bytes per character */

    if (LITTLE_ENDIAN_PRX) {
        value = bmpdup(value);
        (VOID)endian_mangle_string(value);
    }

    status = IO_fwrite((char*)value, 1, length, outstream);
    if (LITTLE_ENDIAN_PRX) (VOID)FREE(value);
    if (status != length) ERETURN(A2C_ERR_OUTSTREAM_LEN);

    return NO_ERROR;
}




static int
verify_BMPString(null, ctx, module, selector, index, data, error)
    io_ctx                    null;
    struct asn_verify_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    struct asn_template_     *template  = 0;
    asn__bmp_string           value     = 0;
    int                       length;
UNUSED_ARG(null);
UNUSED_ARG(ctx);
UNUSED_ARG(error);


    /* BMPString can contain any character */

    template = module->types[selector].template;

    value = *(asn__bmp_string*)data;
    if (value == 0) ERETURN(A2C_ERR_VALUE_MISSING);

    if (template[*index].flags & FLAG_BOUNDED) {
        length = bmplen(value);
        if (length > template[*index].max_size) {
            ERETURN(A2C_ERR_MAX_SIZE_STRING);
        }
        if (length < template[*index].min_size) {
            ERETURN(A2C_ERR_MIN_SIZE_STRING);
        }
    }

    return NO_ERROR;
}



PRIVATE_DEF struct vtbl_ a2c_vtbl__BMPString =
    { type_name, decode_BMPString, encode_BMPString, verify_BMPString };


