/*
 *
 *    CHOICE.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/CHOICE.c,v 
 *    $Revision: 1.14 
 *    $Date: 1997/06/06 00:30:31 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */

#include <define.h>
#include <io_ctx.h>

#include "asn_error.h"
#include "asn_types.h"
#include "CHOICE.h"


#ifndef LINT
static char rcsid[] = "$Id: CHOICE.c,v 1.14 1997/06/06 00:30:31 briank Exp ";
#endif


static char type_name[] = "CHOICE";

struct choice_ {
    unsigned int  _choice; 
    union { void *data; } u; 
}; 

static A2C_DECODE_FUNCTION_PROTOTYPE(decode_CHOICE);
static A2C_ENCODE_FUNCTION_PROTOTYPE(encode_CHOICE);
static A2C_VERIFY_FUNCTION_PROTOTYPE(verify_CHOICE);



static int
decode_CHOICE(instream, ctx, module, selector, index, data, error)
    io_ctx                    instream;
    struct asn_decode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status         = NO_ERROR;
    int                       indent         = 0;
    struct asn_template_     *template       = 0;
    int                       choice_number  = 0;
    int                       hold_index     = 0;



    template = module->types[selector].template;
    indent = template[*index].indent;

    hold_index = *index;

    do {

        choice_number = template[*index + 1].me;

        ++*index;


        status = __decode(instream, ctx, module, selector, index, data, error);
        if (status != NO_ERROR) {
            if (status == A2C_ERR_WRONG_TAG) {
                status = A2C_WARN_WRONG_TAG;
            }
            else ERETURN(status);
        } 

        if (status != A2C_WARN_WRONG_TAG) {

            /* remember which one was chosen */
            ((struct choice_*)data)->_choice = choice_number;

            /* fast forward to end of choice */
            while (indent < template[*index].indent) ++*index;

            /* RETURN */
            return NO_ERROR;
        }
        else {
            /* get to the end of this choice element */
            status = fast_forward(module, selector, index);
            if (status != NO_ERROR) ERETURN(status);
        }

    /*  don't go past the end of the choices */
    } while (indent < template[*index+1].indent);

    /* if we got to here, none of the choices was found */

    /* go back to the start of the CHOICE object */
    *index = hold_index;

    /* none was chosen */
    ((struct choice_*)data)->_choice = 0;

    return A2C_WARN_WRONG_TAG;
}



static int
encode_CHOICE(outstream, ctx, module, selector, index, data, error)
    io_ctx                    outstream;
    struct asn_encode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status         = NO_ERROR;
    int                       indent         = 0;
    UINT4                     chosen         = 0;
    struct asn_template_     *template       = 0;



    template = module->types[selector].template;
    indent = template[*index].indent;

    chosen = ((struct choice_*)data)->_choice;
    if (chosen == 0) ERETURN(A2C_ERR_NOTHING_CHOSEN);

    /* go to the first of the choices */
    ++*index;

    while (1) {
        if (indent >= template[*index].indent) ERETURN(A2C_ERR_BAD_CHOSEN);

        if (chosen == template[*index].me) break;   /* BREAK */

        /* get to the end of this choice */
        status = fast_forward(module, selector, index);
        if (status != NO_ERROR) ERETURN(status);

        /* go to the start of the next choice */
        ++*index;
    } 

    status = __encode(outstream, ctx, module, selector, index, data, error);
    if (status != NO_ERROR) ERETURN(status);

    /* fast forward to end of choice */
    while (indent < template[*index].indent) ++*index;

    return NO_ERROR;
}





static int
verify_CHOICE(null, ctx, module, selector, index, data, error)
    io_ctx                    null;
    struct asn_verify_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
UNUSED_ARG(null);
UNUSED_ARG(ctx);
UNUSED_ARG(module);
UNUSED_ARG(selector);
UNUSED_ARG(index);
UNUSED_ARG(data);
UNUSED_ARG(error);


    return NO_ERROR;
}




PRIVATE_DEF struct vtbl_ a2c_vtbl__CHOICE =
    { type_name, decode_CHOICE, encode_CHOICE, verify_CHOICE };


