/*
 *
 *    EXPLICIT_TAG.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/EXPLICIT_TAG.c,v 
 *    $Revision: 1.12 
 *    $Date: 1997/06/06 00:30:35 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */

#include <define.h>
#include <io_ctx.h>  

#include "asn_error.h"

#include "EXPLICIT_TAG.h"


#ifndef LINT
static char rcsid[] = "$Id: EXPLICIT_TAG.c,v 1.12 1997/06/06 00:30:35 briank Exp ";
#endif


static char type_name[] = "EXPLICIT_TAG";

static A2C_DECODE_FUNCTION_PROTOTYPE(decode_EXPLICIT_TAG);
static A2C_ENCODE_FUNCTION_PROTOTYPE(encode_EXPLICIT_TAG);
static A2C_VERIFY_FUNCTION_PROTOTYPE(verify_EXPLICIT_TAG);


static int
decode_EXPLICIT_TAG(instream, ctx, module, selector, index, data, error)
    io_ctx                    instream;
    struct asn_decode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status  = NO_ERROR;


    status = check_tag(ctx, module, selector, index);
    switch (status) {
    case NO_ERROR:              /* do nothing */            break;
    case A2C_WARN_WRONG_TAG:
if (module->types[selector].template[*index].flags & FLAG_DEFAULT) {
  UINT4  hold_tag = get_current_tag(ctx);
  while (module->types[selector].template[*index].tag_tagging != TAGGING_BASE) {
      ++*index;
  }
  set_current_tag(ctx,0); /* always invalid with explicit tagging */
  status = GET_VTBL(module->types[selector].template[*index].vtbl)->
           decode(instream, ctx, module, selector, index, data, error);
  set_current_tag(ctx,hold_tag);
  ++*index; /* get past EOC */
  switch (status) {
  case A2C_WARN_DEFAULT:
      /* this is what we're expecting */
      break;
  case NO_ERROR:
      ERETURN(A2C_ERR_INTERNAL);
      break;
  default:
      ERETURN(status);
      break;
  }
  return A2C_WARN_DEFAULT;
}
else
    return A2C_WARN_WRONG_TAG;  break;
    default:                    ERETURN(status);            break;
    }

++*index;
status = __decode(instream, ctx, module, selector, index, data, error);
if (status != NO_ERROR) ERETURN(status);

    /* decode EOC */
++*index;


    status = __decode(instream, ctx, module, selector, index, data, error);
    if (status != NO_ERROR) ERETURN(status);

    return NO_ERROR;
}



static int
encode_EXPLICIT_TAG(outstream, ctx, module, selector, index, data, error)
    io_ctx                    outstream;
    struct asn_encode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status  = NO_ERROR;


++*index;
status = __encode(outstream, ctx, module, selector, index, data, error);
if (status == A2C_WARN_DEFAULT) {
  /* do nothing */
}
else
    if (status != NO_ERROR) ERETURN(status);

    /* encode EOC */
++*index;
status = __encode(outstream, ctx, module, selector, index, data, error);
/* ignore status */

    return NO_ERROR;
}






static int
verify_EXPLICIT_TAG(null, ctx, module, selector, index, data, error)
    io_ctx                    null;
    struct asn_verify_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
UNUSED_ARG(null);
UNUSED_ARG(module);
UNUSED_ARG(selector);
UNUSED_ARG(index);
UNUSED_ARG(ctx);
UNUSED_ARG(data);
UNUSED_ARG(error);


    return NO_ERROR;
}




PRIVATE_DEF struct vtbl_ a2c_vtbl__EXPLICIT_TAG =
    { type_name, decode_EXPLICIT_TAG, encode_EXPLICIT_TAG, verify_EXPLICIT_TAG };


