/*
 *
 *    INTEGER.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/INTEGER.c,v 
 *    $Revision: 1.14 
 *    $Date: 1997/06/25 03:04:22 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */

#include <define.h>
#include <io_ctx.h>

#include "asn.h"
#include "asn_types.h"
#include "asn_error.h"

#include "INTEGER.h"


#ifndef LINT
static char rcsid[] = "$Id: INTEGER.c,v 1.14 1997/06/25 03:04:22 briank Exp ";
#endif



static char type_name[] = "INTEGER";

PRIVATE A2C_DECODE_FUNCTION_PROTOTYPE(decode_INTEGER);
PRIVATE A2C_ENCODE_FUNCTION_PROTOTYPE(encode_INTEGER);
static A2C_VERIFY_FUNCTION_PROTOTYPE(verify_INTEGER);





PRIVATE int
decode_INTEGER(instream, ctx, module, selector, index, data, error)
    io_ctx                    instream;
    struct asn_decode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status  = NO_ERROR;
    length_type               length  = 0;
    int                       c       = 0;
    int                       integer = 0;
UNUSED_ARG(error);

/* assumes we're on a 2's complement machine */


    status = check_tag(ctx, module, selector, index);
    switch (status) {
    case NO_ERROR:              /* do nothing */            break;
    case A2C_WARN_WRONG_TAG: 
if (module->types[selector].template[*index].defaults) {
    *(asn__integer*)data = *(asn__integer*)(module->types[selector].template[*index].defaults);
   return A2C_WARN_DEFAULT;
}
   return A2C_WARN_WRONG_TAG;  break;
    default:                    ERETURN(status);            break;
    }

    /* DER only */
    if (IS_CONSTRUCTED(get_current_tag(ctx))) ERETURN(A2C_ERR_CONSTRUCTED);

    length = get_current_length(ctx);
    if (length > (length_type)module->types[selector].template[*index].sizeoftype) {
        ERETURN(A2C_ERR_OBJECT_SIZE);
    }

    ctx->value.nread = 0;
    integer = 0;
    if (length > 0) {

        c = IO_fgetc(instream);
        if (c == EOF) ERETURN(A2C_ERR_EOF);
        ++ctx->value.nread;
        if (c & 0x80) {
            /* negative number */
            integer = ((char)0 - ((char)-c));
        }
        else {
            integer = c;
        }

        while (--length > 0) {
           c = IO_fgetc(instream);
           if (c == EOF) ERETURN(A2C_ERR_EOF);
           ++ctx->value.nread;
           integer <<= 8;
           integer |= (c & 0xff);
        }
    }

    *(asn__integer*)data = integer;

    return NO_ERROR;
}



PRIVATE int
encode_INTEGER(outstream, ctx, module, selector, index, data, error)
    io_ctx                    outstream;
    struct asn_encode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status   = NO_ERROR;
    int                       i        = 0;
    int                       negative = 0;
    asn__integer              integer  = *(asn__integer*)data;
    int                       last_non_padding_byte = 0;
UNUSED_ARG(error);


if (module->types[selector].template[*index].defaults) {
   if (*(asn__integer*)data == *(asn__integer*)(module->types[selector].template[*index].defaults)) {
       ctx->list_tail->tag = TAG_EOC;  /* force skip */
if (CLASS(ctx->STACK[ctx->stack_index-1]->tag) != CLASS_UNIVERSAL) {
    ctx->STACK[ctx->stack_index-1]->tag = 0; /* force skip */
}
       return NO_ERROR;
   }
}

/* assumes we're on a 2's complement machine */

    negative = integer < 0;
    if (negative) {
        for (i = sizeof(integer)-1; i >= 0; --i) {
            if ((integer >> (8*i) & 0xFF) != 0xFF) break;
        }
    }
    else {
        for (i = sizeof(integer)-1; i >= 0; --i) {
            if ((integer >> (8*i) & 0xFF) != 0x00) break;
        }
    }
    
    if (i < 0) i = 0;
    last_non_padding_byte = i;
    
    if (negative) {
        if (((integer>>(8*last_non_padding_byte))&0x80) == 0) {
            status = IO_fputc(0xFF, outstream);
            if (status == EOF) ERETURN(A2C_ERR_OUTSTREAM_LEN);
        }
    }
    else {
        if (((integer>>(8*last_non_padding_byte))&0x80) != 0) {
            status = IO_fputc(0x00, outstream);
            if (status == EOF) ERETURN(A2C_ERR_OUTSTREAM_LEN);
        }
    }

    for (i = last_non_padding_byte; i >= 0; --i) {
        status = IO_fputc((integer>>(8*i))&0xFF, outstream);
        if (status == EOF) ERETURN(A2C_ERR_OUTSTREAM_LEN);
    } 
    
    return NO_ERROR;
}




static int
verify_INTEGER(null, ctx, module, selector, index, data, error)
    io_ctx                    null;
    struct asn_verify_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    struct asn_template_     *template       = 0;
    asn__integer              integer;
UNUSED_ARG(null);
UNUSED_ARG(ctx);
UNUSED_ARG(error);


    template = module->types[selector].template;

    if (template[*index].flags & FLAG_BOUNDED) {
        integer = *(asn__integer*)data;
        if (integer > template[*index].max_size) {
            ERETURN(A2C_ERR_MAX_VALUE);
        }
        if (integer < template[*index].min_size) {
            ERETURN(A2C_ERR_MIN_VALUE);
        }
    }

    return NO_ERROR;
}



PRIVATE_DEF struct vtbl_ a2c_vtbl__INTEGER =
    { type_name, decode_INTEGER, encode_INTEGER, verify_INTEGER };



