/*
 *
 *    REAL.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/REAL.c,v 
 *    $Revision: 1.14 
 *    $Date: 1997/06/06 00:30:48 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */

#include <limits.h>
#include <math.h>

#include <define.h>
#include <io_ctx.h>  

#include "asn.h"
#include "asn_error.h"
#include "asn_types.h"

#include "REAL.h"


#ifndef LINT
static char rcsid[] = "$Id: REAL.c,v 1.14 1997/06/06 00:30:48 briank Exp ";
#endif



static char type_name[] = "REAL";

static A2C_DECODE_FUNCTION_PROTOTYPE(decode_REAL);
static A2C_ENCODE_FUNCTION_PROTOTYPE(encode_REAL);
static A2C_VERIFY_FUNCTION_PROTOTYPE(verify_REAL);





static int
decode_REAL(instream, ctx, module, selector, index, data, error)
    io_ctx                    instream;
    struct asn_decode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status  = NO_ERROR;
    int                       bytes_remaining;
    int                       c;
    double                    real = 0.0;
    double                    mantissa;
    long                      sign, base, factor;
    long                      exponent, e_type;
UNUSED_ARG(error);


    status = check_tag(ctx, module, selector, index);
    switch (status) {
    case NO_ERROR:              /* do nothing */            break;
    case A2C_WARN_WRONG_TAG:    return A2C_WARN_WRONG_TAG;  break;
    default:                    ERETURN(status);            break;
    }

    /* DER only */
    if (IS_CONSTRUCTED(get_current_tag(ctx))) ERETURN(A2C_ERR_CONSTRUCTED);

    bytes_remaining = get_current_length(ctx);

    /* if butes_remaining == 0, then the encoding is of 0.0 */
    if (bytes_remaining > 0) {

        c = IO_fgetc(instream);
        if (c == EOF) ERETURN(A2C_ERR_EOF);
        ctx->value.nread++;
        --bytes_remaining;


/* DER requires binary (base 2) encoding */
        if ((c & 0x80) != 0x80) ERETURN(A2C_ERR_BAD_VALUE);
ASSERT((c & 0x80) == 0x80);

        sign = (c & 0x40) ? -1 : 1;

/* DER only supports base 2 */
        if ((c & 0x30) != 0x00) ERETURN(A2C_ERR_BAD_VALUE);
        ASSERT((c & 0x30) == 0x00);
    
        switch (c & 0x30) {
        case 0x00:
            base = 2;
            break;
        case 0x10:
            base = 8;
            break;
        case 0x20:
            base = 16;
            break;
        default:
            ERETURN(A2C_ERR_BAD_VALUE);
        }
    
        factor = (c & 0x0c) >> 2;
    
        e_type = (c & 0x3);

        if (bytes_remaining < 1) ERETURN(A2C_ERR_BAD_VALUE);
        c = IO_fgetc(instream);
        if (c == EOF) ERETURN(A2C_ERR_EOF);
        ctx->value.nread++;
        --bytes_remaining;
    
        exponent = (c & 0x80) ? -1 : 0;
        ASSERT(e_type != 0x3);
        switch (e_type) {
        case 0x3:
            /* unsupported encoding */
            ERETURN(A2C_ERR_BAD_VALUE);
            break;
        case 0x2:
            exponent = (c & 0xff);
            if (bytes_remaining < 1) ERETURN(A2C_ERR_BAD_VALUE);
            c = IO_fgetc(instream);
            if (c == EOF) ERETURN(A2C_ERR_EOF);
            ctx->value.nread++;
            --bytes_remaining;
        /* FALL THROUGH */
        case 0x1:
            exponent = (exponent << 8) | (c & 0xff);
            if (bytes_remaining < 1) ERETURN(A2C_ERR_BAD_VALUE);
            c = IO_fgetc(instream);
            if (c == EOF) ERETURN(A2C_ERR_EOF);
            ctx->value.nread++;
            --bytes_remaining;
        /* FALL THROUGH */
        case 0x0:
            exponent = (exponent << 8) | (c & 0xff);
            break;
        default:
            ERETURN(A2C_ERR_BAD_VALUE);
            break;
        }

        mantissa = 0.0; 
        while (bytes_remaining-- > 0) {
            c = IO_fgetc(instream);
            if (c == EOF) ERETURN(A2C_ERR_EOF);
            ctx->value.nread++;
            mantissa *= 0x100;
            mantissa += (unsigned int)(c & 0xff);
        }
    
        mantissa = sign * mantissa * (1 << factor);
        real = mantissa * pow((double)base, (double)(exponent));
    }

    *(asn__real*)data = real;

    return NO_ERROR;
}



static int
encode_REAL(outstream, ctx, module, selector, index, data, error)
    io_ctx                    outstream;
    struct asn_encode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status  = NO_ERROR;
    double                    real;
    int                       sign, exponent;
    double                    mantissa;
    unsigned char             m_encoded[sizeof(asn__real)*2];
    double                    fract, integral;
    unsigned long             result;
    int                       i;
    long                      e_len = 0, e_shift;
    unsigned long             e_mask;
    int                       m_len = 0, m_shift;
    long                      left_shift;
    unsigned long             left_mask;
UNUSED_ARG(ctx);
UNUSED_ARG(module);
UNUSED_ARG(selector);
UNUSED_ARG(index);
UNUSED_ARG(error);


    real = *(asn__real*)data;

    if (real == 0.0) return NO_ERROR;


    sign = (real < 0.0) ? 0x40 : 0x00;

    if (real < 0.0) real = -real;
    mantissa = frexp(real, &exponent);
    ASSERT(mantissa != 0);

    fract = mantissa;
    m_len = 0;
    for (i = 0; i < (int)sizeof(asn__real); ++i) {
        fract = modf(fract*(double)0x100, &integral);
        m_encoded[i] = (unsigned char)(int)integral;
        if (m_encoded[i] != 0) m_len = i+1;
    }
 
    /* for DER encoding, M must be odd */
    i = m_encoded[m_len-1];
    ASSERT((i & 0xff) != 0);
    m_shift = 0;
    if ((i & 0x7f) == 0) m_shift = 7;
    else if ((i & 0x3f) == 0) m_shift = 6;
    else if ((i & 0x1f) == 0) m_shift = 5;
    else if ((i & 0x0f) == 0) m_shift = 4;
    else if ((i & 0x07) == 0) m_shift = 3;
    else if ((i & 0x03) == 0) m_shift = 2;
    else if ((i & 0x01) == 0) m_shift = 1;

    if (m_shift != 0) {
        left_mask = ~(0xffff << m_shift);
        left_shift = (CHAR_BIT-m_shift);
        for (i = m_len-1; i > 0; --i) {
            m_encoded[i] >>= m_shift;
            m_encoded[i] |= (left_mask & m_encoded[i-1]) << left_shift;
        }
        m_encoded[0] >>= m_shift;
    }

    exponent -= (8 * m_len) - m_shift;
    if (exponent >= 0) {
        e_mask = 0x00;
    }
    else {
        e_mask = 0xFF;
    }

    if ((exponent == 0) || (exponent == ~0)) {
        e_len = 1;
        e_shift = 0;
    }
    else {
        e_shift = CHAR_BIT*(sizeof(exponent));
        for (e_len = sizeof(exponent); e_len > 0; --e_len) {
            e_shift -= 8;
            result = (exponent >> e_shift) & 0xFF;
            if (result != e_mask) break;  /* BREAK */
        }
        if ((result & 0x80) != (e_mask & 0x80)) {
            ++e_len;
            e_shift += 8;
        }
    }

    i = sign
      | 0x80                 /* binary encoding */
      | ((e_len>3)?0x11:(e_len-1));
    status = IO_fputc(i, outstream);
    if (status == EOF) ERETURN(A2C_ERR_OUTSTREAM_LEN);

    if (e_len > 3) {
        status = IO_fputc(e_len, outstream);
        if (status == EOF) ERETURN(A2C_ERR_OUTSTREAM_LEN);
    }

    while (e_len-- > 0) {
        result = (exponent >> e_shift) & 0xFF;
        status = IO_fputc(result, outstream);
        if (status == EOF) ERETURN(A2C_ERR_OUTSTREAM_LEN);
        e_shift -= 8;
    }

    status = IO_fwrite((char*)m_encoded, 1, m_len, outstream);
    if (status != m_len) ERETURN(A2C_ERR_OUTSTREAM_LEN);

    return NO_ERROR;
}





static int
verify_REAL(null, ctx, module, selector, index, data, error)
    io_ctx                    null;
    struct asn_verify_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
UNUSED_ARG(null);
UNUSED_ARG(ctx);
UNUSED_ARG(module);
UNUSED_ARG(selector);
UNUSED_ARG(index);
UNUSED_ARG(data);
UNUSED_ARG(error);


    return NO_ERROR;
}



PRIVATE_DEF struct vtbl_ a2c_vtbl__REAL =
    { type_name, decode_REAL, encode_REAL, verify_REAL };


