/*
 *
 *    UTCTime.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/UTCTime.c,v 
 *    $Revision: 1.13 
 *    $Date: 1997/06/06 00:30:53 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */

#include <time.h>

#include <define.h>
#include <io_ctx.h>  

#include "asn.h"
#include "asn_error.h"
#include "asn_types.h"
#include "utcfunc.h"

#include "UTCTime.h"


#ifndef LINT
static char rcsid[] = "$Id: UTCTime.c,v 1.13 1997/06/06 00:30:53 briank Exp ";
#endif


/* any year less than YEAR_ROLLOVER is after 2000 */
#define YEAR_ROLLOVER  50

static char type_name[] = "UTCTime";

static A2C_DECODE_FUNCTION_PROTOTYPE(decode_UTCTime);
static A2C_ENCODE_FUNCTION_PROTOTYPE(encode_UTCTime);
static A2C_VERIFY_FUNCTION_PROTOTYPE(verify_UTCTime);





static int
decode_UTCTime(instream, ctx, module, selector, index, data, error)
    io_ctx                    instream;
    struct asn_decode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status      = NO_ERROR;
    length_type               length      = 0;
    int                       nread       = 0;
    struct tm                 tm_struct;
    UCHAR                     buffer[32];
UNUSED_ARG(error);


    MEMZERO(tm_struct);

    status = check_tag(ctx, module, selector, index);
    switch (status) {
    case NO_ERROR:              /* do nothing */            break;
    case A2C_WARN_WRONG_TAG:    return A2C_WARN_WRONG_TAG;  break;
    default:                    ERETURN(status);            break;
    }

    /* DER only */
    if (IS_CONSTRUCTED(get_current_tag(ctx))) ERETURN(A2C_ERR_CONSTRUCTED);

    length = get_current_length(ctx);
    /* permit only DER */
    if (length != 13) ERETURN(A2C_ERR_OBJECT_SIZE);

    while (nread < length) {
        status = IO_fread((char*)buffer+nread, 1, length-nread, instream);
        /* not the best way to handle non-blocking IO */
        if (status == 0) ERETURN(A2C_ERR_EOF);

        nread += status;
    }
    ctx->value.nread = nread;

#define TO_NUMBER(n) ((n)-'0')

    tm_struct.tm_year   = TO_NUMBER(buffer[0])*10;
    tm_struct.tm_year  += TO_NUMBER(buffer[1]);
    if (tm_struct.tm_year < YEAR_ROLLOVER) tm_struct.tm_year += 100;
    tm_struct.tm_mon    = TO_NUMBER(buffer[2])*10;
    tm_struct.tm_mon   += TO_NUMBER(buffer[3]);
    tm_struct.tm_mon--; 
    tm_struct.tm_mday   = TO_NUMBER(buffer[4])*10;
    tm_struct.tm_mday  += TO_NUMBER(buffer[5]);
    tm_struct.tm_hour   = TO_NUMBER(buffer[6])*10;
    tm_struct.tm_hour  += TO_NUMBER(buffer[7]);
    tm_struct.tm_min    = TO_NUMBER(buffer[8])*10;
    tm_struct.tm_min   += TO_NUMBER(buffer[9]);
        tm_struct.tm_sec    = TO_NUMBER(buffer[10])*10;
        tm_struct.tm_sec   += TO_NUMBER(buffer[11]);
        if (buffer[12] != 'Z') ERETURN(A2C_ERR_UTC_BAD_CHAR);
    
    *(asn__utc_time*)data = timegm(&tm_struct);

    return NO_ERROR;
}



static int
encode_UTCTime(outstream, ctx, module, selector, index, data, error)
    io_ctx                    outstream;
    struct asn_encode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status      = NO_ERROR;
    struct tm                *tm_struct   = 0;
    UCHAR                     buffer[32];
UNUSED_ARG(ctx);
UNUSED_ARG(module);
UNUSED_ARG(selector);
UNUSED_ARG(index);
UNUSED_ARG(error);


#define FROM_NUMBER(n) ((n)+'0')

    tm_struct = gmtime((asn__utc_time*)data);

    buffer[0] = FROM_NUMBER( tm_struct->tm_year / 10 );
    buffer[1] = FROM_NUMBER( tm_struct->tm_year % 10 );
    tm_struct->tm_mon++;
    buffer[2] = FROM_NUMBER( tm_struct->tm_mon  / 10 );
    buffer[3] = FROM_NUMBER( tm_struct->tm_mon  % 10 );
    buffer[4] = FROM_NUMBER( tm_struct->tm_mday / 10 );
    buffer[5] = FROM_NUMBER( tm_struct->tm_mday % 10 );
    buffer[6] = FROM_NUMBER( tm_struct->tm_hour / 10 );
    buffer[7] = FROM_NUMBER( tm_struct->tm_hour % 10 );
    buffer[8] = FROM_NUMBER( tm_struct->tm_min  / 10 );
    buffer[9] = FROM_NUMBER( tm_struct->tm_min  % 10 );
        buffer[10] = FROM_NUMBER( tm_struct->tm_sec  / 10 );
        buffer[11] = FROM_NUMBER( tm_struct->tm_sec  % 10 );
        buffer[12] = 'Z';
        status = IO_fwrite((char*)buffer, 1, 13, outstream);
        if (status != 13) ERETURN(A2C_ERR_OUTSTREAM_LEN);

    return NO_ERROR;
}




static int
verify_UTCTime(null, ctx, module, selector, index, data, error)
    io_ctx                    null;
    struct asn_verify_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
UNUSED_ARG(null);
UNUSED_ARG(ctx);
UNUSED_ARG(module);
UNUSED_ARG(selector);
UNUSED_ARG(index);
UNUSED_ARG(data);
UNUSED_ARG(error);


    return NO_ERROR;
}



PRIVATE_DEF struct vtbl_ a2c_vtbl__UTCTime =
    { type_name, decode_UTCTime, encode_UTCTime, verify_UTCTime };


