/*
 *
 *    a2c.h
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/a2c.h,v 
 *    $Revision: 1.16 
 *    $Date: 1997/06/25 03:06:46 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */


#ifndef __A2C_H__
#define __A2C_H__


#include <alloc_ctx.h>
#include <define.h>
#include <io_ctx.h>

#include "asn_types.h"


#ifdef __cplusplus
extern "C" {
#endif


typedef VOID*   a2c_ctx;




PRIVATE int    create_a2c_ctx
  PROTO_LIST((
    a2c_ctx           *codec                          /* OUT */
  ));

PRIVATE int    delete_a2c_ctx
  PROTO_LIST((
    a2c_ctx           *codec                          /* IN */
  ));

PRIVATE int    modify_a2c_ctx
  PROTO_LIST((
    a2c_ctx            codec,                         /* IN */
    int                modify,                        /* IN */
    ...                                               /* IN */
  ));
#define A2C_CTX_MODIFY_VERIFY             1   /*  int bool -- default 0  */
#define A2C_CTX_MODIFY_OPAQUE_CONTEXT     2   /*  VOID*  -- default 0  */
#define A2C_CTX_MODIFY_INDEFINITE	  3   /*  int bool -- default 0  */

/*  Note, query_a2c_ctx does not allocate any data.  Any pointers  
    that are returned point to internal data structures and should
    not be deallocated by the caller */
PRIVATE int    query_a2c_ctx
  PROTO_LIST((
    a2c_ctx            codec,                         /* IN */
    int                query,                         /* IN */
    ...                                               /* IN/OUT */
  ));
#define A2C_CTX_QUERY_VERIFY              1   /*  int *bool -- default 0  */
#define A2C_CTX_QUERY_OPAQUE_CONTEXT      2   /*  VOID** -- default 0 */
#define A2C_CTX_QUERY_INDEFINITE          3   /*  int *bool -- default 0  */

/* instream must support ftell() */
PRIVATE int    decode
  PROTO_LIST((
    a2c_ctx            app_ctx,                       /* IN */
    io_ctx             instream,                      /* IN */
    a2c_module         module,                        /* IN */
    UINT4              selector,                      /* IN */
    alloc_ctx          heap,                          /* IN */   /* may be 0 */ 
    VOID              *data,                          /* OUT */
    char               error[ERROR_MESSAGE_LENGTH]    /* OUT */  /* may be 0 */
  ));

PRIVATE int    encode
  PROTO_LIST((
    a2c_ctx            app_ctx,                       /* IN */
    io_ctx             outstream,                     /* OUT */
    a2c_module         module,                        /* IN */
    UINT4              selector,                      /* IN */
    VOID              *data,                          /* IN */
    char               error[ERROR_MESSAGE_LENGTH]    /* OUT */  /* may be 0 */
  ));



/* use asn__any instead of io_ctx */
PRIVATE int    decode_using_any
  PROTO_LIST((
    a2c_ctx            app_ctx,                       /* IN */
    asn__any          *instream,                      /* IN */
    a2c_module         module,                        /* IN */
    UINT4              selector,                      /* IN */
    alloc_ctx          heap,                          /* IN */   /* may be 0 */ 
    VOID              *data,                          /* OUT */
    char               error[ERROR_MESSAGE_LENGTH]    /* OUT */  /* may be 0 */
  ));

/* use asn__any instead of io_ctx */
PRIVATE int    encode_using_any
  PROTO_LIST((
    a2c_ctx            app_ctx,                       /* IN */
    asn__any          *outstream,                     /* OUT */
    a2c_module         module,                        /* IN */
    UINT4              selector,                      /* IN */
    alloc_ctx          heap,                          /* IN */   /* may be 0 */ 
    VOID              *data,                          /* IN */
    char               error[ERROR_MESSAGE_LENGTH]    /* OUT */  /* may be 0 */
  ));


/*
 *    ERRORS
 */

#define  A2C_MODULE_NO_               2

#define  A2C_ERR_NO_MEMORY            SE_ERR_NO_MEMORY
#define  A2C_ERR_INTERNAL             SE_ERR_INTERNAL
#define  A2C_ERR_USER_DATA            ERROR_MAKE(A2C_MODULE_NO_, 2)
#define  A2C_ERR_IO                   ERROR_MAKE(A2C_MODULE_NO_, 20)
#define  A2C_ERR_ENCODING             ERROR_MAKE(A2C_MODULE_NO_, 50)

#define  A2C_ERR_BAD_PARAMETER        ERROR_MAKE(A2C_MODULE_NO_, 3)
#define  A2C_ERR_VALUE_MISSING        ERROR_MAKE(A2C_MODULE_NO_, 4)
#define  A2C_ERR_NOTHING_CHOSEN       ERROR_MAKE(A2C_MODULE_NO_, 5)
#define  A2C_ERR_BAD_CHOSEN           ERROR_MAKE(A2C_MODULE_NO_, 6)
#define  A2C_ERR_UNKNOWN_MODIFY       ERROR_MAKE(A2C_MODULE_NO_, 7)
#define  A2C_ERR_UNKNOWN_QUERY        ERROR_MAKE(A2C_MODULE_NO_, 8)
#define  A2C_ERR_BAD_PARAMETERS       ERROR_MAKE(A2C_MODULE_NO_, 9)
#define  A2C_ERR_BAD_SELECTOR         ERROR_MAKE(A2C_MODULE_NO_, 10)
#define  A2C_ERR_BAD_INDEX            ERROR_MAKE(A2C_MODULE_NO_, 11)



#ifdef __cplusplus
}
#endif
#endif

