/*
 *
 *    a2c_ctx.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/a2c_ctx.c,v 
 *    $Revision: 1.11 
 *    $Date: 1997/06/25 03:04:50 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */


#include <alloc_ctx.h>

#include "a2c.h"
#include "a2c_ctx.h"
#include "asn_error.h"


#ifndef LINT
static char rcsid[] = "$Id: a2c_ctx.c,v 1.11 1997/06/25 03:04:50 briank Exp ";
#endif






PRIVATE int
create_a2c_ctx(ctx_opaque)
    a2c_ctx    *ctx_opaque;
{
    struct a2c_ctx_   *ctx = 0;

    if (ctx_opaque == 0) ERETURN(A2C_ERR_BAD_PARAMETER);

    ctx = (struct a2c_ctx_*)CALLOC(1,sizeof(struct a2c_ctx_));
    if (ctx == 0) ERETURN(A2C_ERR_NO_MEMORY);

    *ctx_opaque = ctx;

    return NO_ERROR;
}

PRIVATE int
delete_a2c_ctx(ctx_opaque)
    a2c_ctx    *ctx_opaque;
{
    if (ctx_opaque == 0) ERETURN(A2C_ERR_BAD_PARAMETER);

    if (*ctx_opaque != 0) {
        MEMZERO(**(struct a2c_ctx_**)ctx_opaque);
        FREE(*ctx_opaque);
    }
    *ctx_opaque = 0;

    return NO_ERROR;
}


PRIVATE int
#ifdef USE_ANSI_STDARG
modify_a2c_ctx(a2c_ctx ctx_opaque, int modify, ...)
#else
modify_a2c_ctx(ctx_opaque, modify, va_alist)
    a2c_ctx         ctx_opaque;
    int             modify;
    va_dcl
#endif
{
    va_list            ap;
    struct a2c_ctx_   *ctx = (struct a2c_ctx_*)ctx_opaque;
    int                i;

    VA_START(ap, modify);

    switch (modify) {
    case A2C_CTX_MODIFY_VERIFY:  /* int bool -- default 0 */
        i = va_arg(ap, int);
        if (i) BIT_SET(ctx->flag, A2C_CTX_FLAG_VERIFY);
        else   BIT_UNSET(ctx->flag, A2C_CTX_FLAG_VERIFY);
        break;
    case A2C_CTX_MODIFY_INDEFINITE:  /* int bool -- default 0 */
        i = va_arg(ap, int);
        if (i) BIT_SET(ctx->flag, A2C_CTX_FLAG_INDEFINITE);
        else   BIT_UNSET(ctx->flag, A2C_CTX_FLAG_INDEFINITE);
        break;
    case A2C_CTX_MODIFY_OPAQUE_CONTEXT:  /* VOID* -- default 0 */
        ctx->opaque_context = va_arg(ap, VOID*);
        break;
    default:
        va_end(ap);
        ERETURN(A2C_ERR_UNKNOWN_MODIFY);
        break;
    }

    va_end(ap);

    return NO_ERROR;
}

PRIVATE int
#ifdef USE_ANSI_STDARG
query_a2c_ctx(a2c_ctx ctx_opaque, int query, ...)
#else
query_a2c_ctx(ctx_opaque, query, va_alist)
    a2c_ctx         ctx_opaque;
    int             query;
    va_dcl
#endif
{
    va_list            ap;
    struct a2c_ctx_   *ctx = (struct a2c_ctx_*)ctx_opaque;
    int                *ip;
    VOID              **vpp;

    VA_START(ap, query);

    switch (query) {
    case A2C_CTX_QUERY_VERIFY:  /* int *bool -- default 0 */
        ip = va_arg(ap, int*);
        *ip = BIT_IS_SET(ctx->flag, A2C_CTX_FLAG_VERIFY);
        break;
    case A2C_CTX_QUERY_INDEFINITE:  /* int *bool -- default 0 */
        ip = va_arg(ap, int*);
        *ip = BIT_IS_SET(ctx->flag, A2C_CTX_FLAG_INDEFINITE);
        break;
    case A2C_CTX_QUERY_OPAQUE_CONTEXT:  /*  VOID**  */
        vpp = va_arg(ap, VOID**);
        *vpp = ctx->opaque_context;
        break;
    default:
        va_end(ap);
        ERETURN(A2C_ERR_UNKNOWN_QUERY);
        break;
    }

    va_end(ap);

    return NO_ERROR;
}


