/*
 *
 *    a2c_decode.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/a2c_decode.c,v 
 *    $Revision: 1.1 
 *    $Date: 1997/06/25 03:01:16 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */

#include <alloc_ctx_pool.h>


#include "asn.h"
#include "asn_decode_ctx.h"
#include "asn_error.h"
#include "a2c.h"
#include "a2c_ctx.h"
#include "a2c_dot_c_file.h"
#include "ber_length.h"
#include "ber_tag.h"
#include "params.h"
#include "runtime.h"
#include "a2c_vtbl.h"


#ifndef LINT
static char rcsid[] = "$Id: a2c_decode.c,v 1.1 1997/06/25 03:01:16 briank Exp ";
#endif





static int    decode_tag_and_length
  PROTO_LIST((
    io_ctx                    instream,
    struct asn_decode_ctx_   *ctx,
    struct a2c_module_       *module,
    UINT4                     selector,
    UINT4                    *index,
    VOID                     *data,
    char                      error[ERROR_MESSAGE_LENGTH] 
  ));

static int    get_tag_and_length
  PROTO_LIST((
    io_ctx                    instream,
    struct asn_decode_ctx_   *ctx
  ));




static int
get_tag_and_length(instream, ctx)
    io_ctx                    instream;
    struct asn_decode_ctx_   *ctx;
{
    int                        status = NO_ERROR;

    /* if we've already got the tag, then we should skip getting it */
    if ( ! ctx->have_tag) { 

        /* create a symbol on stack, new tag implies new object */
        status = asn_decode_ctx_push_object(ctx);
        if (status != NO_ERROR) ERETURN(status);

        status = initiate_get_tag(ctx, instream);
        if (status != NO_ERROR) ERETURN(status);
        status = continue_get_tag(ctx, instream);
        if (status != NO_ERROR) ERETURN(status);
        status = finalize_get_tag(ctx, instream);
        if (status != NO_ERROR) ERETURN(status);
    }

    /* if we've already got the length, then we should skip getting it! */
    if ( ! ctx->have_length) { 
        status = initiate_get_length(ctx, instream);
        if (status != NO_ERROR) ERETURN(status);
        status = continue_get_length(ctx, instream);
        if (status != NO_ERROR) ERETURN(status);
        status = finalize_get_length(ctx, instream);
        if (status != NO_ERROR) ERETURN(status);
    }

    return NO_ERROR;
}




static int
decode_tag_and_length(instream, ctx, module, selector, index, data, error)
    io_ctx                    instream;
    struct asn_decode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status       = NO_ERROR;
    int                       result       = NO_ERROR;
    struct asn_template_     *current      = 0;
UNUSED_ARG(data);
UNUSED_ARG(error);

    current = &(module->types[selector].template[*index]);

    if (! ctx->skip_next_tag) {

        status = get_tag_and_length(instream, ctx);
        if (status != NO_ERROR) ERETURN(status);


    }

    return result;
}



PRIVATE int
__decode(instream, ctx, module, selector, index, data, error)
    io_ctx                    instream;
    struct asn_decode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status           = NO_ERROR;
    UINT4                     entry            = *index;
    struct asn_type_info_    *type             = 0;
    struct asn_template_     *current          = 0;
    struct vtbl_             *vtbl             = 0;
    VOID                     *el_data          = 0;
    int                       optional         = 0;
    int                       verify           = 0;

    type = &(module->types[selector]);

    if (*index >= type->size) ERETURN(A2C_ERR_STRUCT);


#if 1
/* initiate_get_value = initiate_decode */
    /* reset the # of bytes read! */
    ctx->value.nread = 0;
#endif

    status = decode_tag_and_length(
                instream, ctx, module, selector, index, data, error);  
    if (status != NO_ERROR) ERETURN(status);

    current = &(type->template[*index]);
    optional = current->flags & FLAG_OPTIONAL;

    el_data = ((char*)data) + current->offset;
    vtbl = GET_VTBL(current->vtbl);

    status = vtbl->decode(
                    instream, ctx, module, selector, index, el_data, error);
    switch (status) {
    case NO_ERROR:

#if 1
/* finalize_get_value */
        status = 1;  /* assume an object was decoded */
        if (current->tag_tagging == TAGGING_IMPLICIT)  status = 0;
        else if ((current->tag_class == CLASS_SPECIAL)
              && (current->tag_number == TAG_CHOICE)) status = 0;
        if (status) {
            if (IS_PRIMITIVE(get_current_tag(ctx))) {
                if (get_current_length(ctx) != ctx->value.nread) {
                    ERETURN(A2C_ERR_FINALIZE);
                }
            }
            status = asn_decode_ctx_pop_object(ctx);       
            if (status != NO_ERROR) ERETURN(status);
        }
#endif
        (VOID)query_a2c_ctx(ctx->app_ctx, A2C_CTX_QUERY_VERIFY, &verify);
        if (verify) {
            status = vtbl->verify(0, 0, module, selector, &entry, el_data, error);
            if (status != NO_ERROR) ERETURN(status);
        }

        if (optional) {
            /* mark this as present */
            /* _present[] is the first thing in the struct, so a
             * pointer to the struct (that is, data) is the same
             * as the _present[] array pointer
             */
            BIT_SET(data, current->number);
        }
        break;

    case A2C_WARN_DEFAULT:
        /* do nothing */
        break;

    case A2C_WARN_WRONG_TAG:

        if (optional) {
            status = fast_forward(module, selector, index);
            if (status != NO_ERROR) ERETURN(status);
        }
        else ERETURN(A2C_ERR_WRONG_TAG);
        break;

    default:
        ERETURN(status);
        break;
    }

    return NO_ERROR;
}




PRIVATE int
decode(app_ctx, instream, module_opaque, selector, heap, data, error)
    a2c_ctx            app_ctx;
    io_ctx             instream;
    a2c_module         module_opaque;
    UINT4              selector;
    alloc_ctx          heap;
    VOID              *data;
    char               error[ERROR_MESSAGE_LENGTH];
{
    int                       status    = NO_ERROR;
    struct a2c_module_       *module    = (struct a2c_module_*)module_opaque;
    UINT4                     index     = 0;
    struct asn_decode_ctx_    ctx__, *ctx = &ctx__;



    status = check_parameters(CHECK_DECODE_PARAMETERS,
                           instream, module, selector, &index, data, error);
    if (status != NO_ERROR) ERETURN(a2c_extern_error(A2C_ERR_BAD_PARAMETER));

MEMZERO(ctx__);
ctx->app_ctx = app_ctx;
ctx->data_heap = heap;
ctx->data = data;
status = create_pool_ctx(&ctx->tmp_heap);
if (status != NO_ERROR) ERETURN(a2c_extern_error(A2C_ERR_NO_MEMORY));

    if (error != 0) error[0] = '\0';

    status = __decode(instream, ctx, module, selector, &index, data, error);
    (VOID)AC_delete(&ctx->tmp_heap);
    if (status != NO_ERROR) ERETURN(a2c_extern_error(status));

    if (ctx->stack_index != 0) ERETURN(a2c_extern_error(A2C_ERR_INTERNAL));

    return NO_ERROR;
}


