/*
 *
 *    a2c_dot_c_file.h
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/a2c_dot_c_file.h,v 
 *    $Revision: 1.14 
 *    $Date: 1997/06/25 03:06:23 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */


#ifndef __A2C_DOT_C_FILE_H__
#define __A2C_DOT_C_FILE_H__

#include <limits.h>

#include <define.h>

#include "a2c.h"
#include "asn_types.h"


#ifdef __cplusplus
extern "C" {
#endif


typedef UINT4  tag_type;
typedef INT4   length_type;

#define __N_TAG_BITS                   (CHAR_BIT*sizeof(tag_type))
#define __T_CLASS_OFFSET               (__N_TAG_BITS-2)
#define __T_METHOD_OFFSET              (__N_TAG_BITS-3)
#define __T_CLASS                      (((tag_type)0x3)<<__T_CLASS_OFFSET)
#define __T_METHOD                     (((tag_type)0x1)<<__T_METHOD_OFFSET)
#define __T_NUMBER                     (~(__T_CLASS|__T_METHOD))

#define CLASS_UNIVERSAL                (0x0)
#define CLASS_APPLICATION              (((tag_type)0x1)<<__T_CLASS_OFFSET)
#define CLASS_CONTEXT                  (((tag_type)0x2)<<__T_CLASS_OFFSET)
#define CLASS_PRIVATE                  (((tag_type)0x3)<<__T_CLASS_OFFSET)

#define TAG_EOC                        (0)
#define TAG_BOOLEAN                    (1)
#define TAG_INTEGER                    (2)
#define TAG_BIT_STRING                 (3)
#define TAG_OCTET_STRING               (4)
#define TAG_NULL                       (5)
#define TAG_OBJECT_IDENTIFIER          (6)
#define TAG_OBJECT_DESCRIPTOR          (7)
#define TAG_EXTERNAL                   (8)
#define TAG_REAL                       (9)
#define TAG_ENUMERATED                 (10)
#define TAG_SEQUENCE                   (16)
#define TAG_SEQUENCE_OF                TAG_SEQUENCE
#define TAG_SET                        (17)
#define TAG_SET_OF                     TAG_SET
#define TAG_NumericString              (18)
#define TAG_PrintableString            (19)
#define TAG_TeletexString              (20)
#define TAG_VideotexString             (21)
#define TAG_IA5String                  (22)
#define TAG_UTCTime                    (23)
#define TAG_GeneralizedTime            (24)
#define TAG_GRAPHICSTRING              (25)
#define TAG_VisibleString              (26)
#define TAG_GENERALSTRING              (27)
#define TAG_BMPString                  (30)

#define CLASS_SPECIAL                  (((tag_type)0x1)<<(__T_CLASS_OFFSET-1))
#define TAG_ANY                        (253)
#define TAG_CHOICE                     (254)
#define TAG_CHOICE_END                 (255)

#define TAGGING_BASE                   (0)
#define TAGGING_EXPLICIT               (1)
#define TAGGING_IMPLICIT               (2)

#define FLAG_OPTIONAL                  (((unsigned int)0x1)<<0)
#define FLAG_ARRAY                     (((unsigned int)0x1)<<1)
#define FLAG_END_OF_ARRAY              (((unsigned int)0x1)<<2)
#define FLAG_CHOICE                    (((unsigned int)0x1)<<3)
#define FLAG_BOUNDED                   (((unsigned int)0x1)<<4)
#define FLAG_DEFAULT                   (((unsigned int)0x1)<<5)

PRIVATE struct vtbl_   a2c_vtbl__EOC;
PRIVATE struct vtbl_   a2c_vtbl__BOOLEAN;
PRIVATE struct vtbl_   a2c_vtbl__INTEGER;
PRIVATE struct vtbl_   a2c_vtbl__BIT_STRING;
PRIVATE struct vtbl_   a2c_vtbl__OCTET_STRING;
PRIVATE struct vtbl_   a2c_vtbl__NULL;
PRIVATE struct vtbl_   a2c_vtbl__OBJECT_IDENTIFIER;
PRIVATE struct vtbl_   a2c_vtbl__OBJECT_DESCRIPTOR;
PRIVATE struct vtbl_   a2c_vtbl__EXTERNAL;
PRIVATE struct vtbl_   a2c_vtbl__REAL;
PRIVATE struct vtbl_   a2c_vtbl__SEQUENCE;
PRIVATE struct vtbl_   a2c_vtbl__SET;
PRIVATE struct vtbl_   a2c_vtbl__NumericString;
PRIVATE struct vtbl_   a2c_vtbl__PrintableString;
PRIVATE struct vtbl_   a2c_vtbl__TeletexString;
PRIVATE struct vtbl_   a2c_vtbl__VideotexString;
PRIVATE struct vtbl_   a2c_vtbl__VisibleString;
PRIVATE struct vtbl_   a2c_vtbl__IA5String;
PRIVATE struct vtbl_   a2c_vtbl__UTCTime;
PRIVATE struct vtbl_   a2c_vtbl__GeneralizedTime;
PRIVATE struct vtbl_   a2c_vtbl__GRAPHICSTRING;
PRIVATE struct vtbl_   a2c_vtbl__VISIBLESTRING;
PRIVATE struct vtbl_   a2c_vtbl__GENERALSTRING;
PRIVATE struct vtbl_   a2c_vtbl__BMPString;

PRIVATE struct vtbl_   a2c_vtbl__SEQUENCE_OF;
PRIVATE struct vtbl_   a2c_vtbl__SET_OF;
PRIVATE struct vtbl_   a2c_vtbl__ANY;
PRIVATE struct vtbl_   a2c_vtbl__EXPLICIT_TAG;
PRIVATE struct vtbl_   a2c_vtbl__IMPLICIT_TAG;
PRIVATE struct vtbl_   a2c_vtbl__CHOICE;
PRIVATE struct vtbl_   a2c_vtbl__ENUMERATED;

#define VTBL_EOC                       (&(a2c_vtbl__EOC))
#define VTBL_BOOLEAN                   (&(a2c_vtbl__BOOLEAN))
#define VTBL_INTEGER                   (&(a2c_vtbl__INTEGER))
#define VTBL_BIT_STRING                (&(a2c_vtbl__BIT_STRING))
#define VTBL_OCTET_STRING              (&(a2c_vtbl__OCTET_STRING))
#define VTBL_NULL                      (&(a2c_vtbl__NULL))
#define VTBL_OBJECT_IDENTIFIER         (&(a2c_vtbl__OBJECT_IDENTIFIER))
#define VTBL_OBJECT_DESCRIPTOR         (&(a2c_vtbl__OBJECT_DESCRIPTOR))
#define VTBL_EXTERNAL                  (&(a2c_vtbl__EXTERNAL))
#define VTBL_REAL                      (&(a2c_vtbl__REAL))
#define VTBL_SEQUENCE                  (&(a2c_vtbl__SEQUENCE))
#define VTBL_SET                       (&(a2c_vtbl__SET))
#define VTBL_NumericString             (&(a2c_vtbl__NumericString))
#define VTBL_PrintableString           (&(a2c_vtbl__PrintableString))
#define VTBL_TeletexString             (&(a2c_vtbl__TeletexString))
#define VTBL_VideotexString            (&(a2c_vtbl__VideotexString))
#define VTBL_IA5String                 (&(a2c_vtbl__IA5String))
#define VTBL_UTCTime                   (&(a2c_vtbl__UTCTime))
#define VTBL_GeneralizedTime           (&(a2c_vtbl__GeneralizedTime))
#define VTBL_GRAPHICSTRING             (&(a2c_vtbl__GRAPHICSTRING))
#define VTBL_VisibleString             (&(a2c_vtbl__VisibleString))
#define VTBL_GENERALSTRING             (&(a2c_vtbl__GENERALSTRING))
#define VTBL_BMPString                 (&(a2c_vtbl__BMPString))

#define VTBL_SEQUENCE_OF               (&(a2c_vtbl__SEQUENCE_OF))
#define VTBL_SET_OF                    (&(a2c_vtbl__SET_OF))
#define VTBL_ANY                       (&(a2c_vtbl__ANY))
#define VTBL_EXPLICIT_TAG              (&(a2c_vtbl__EXPLICIT_TAG))
#define VTBL_IMPLICIT_TAG              (&(a2c_vtbl__IMPLICIT_TAG))
#define VTBL_CHOICE                    (&(a2c_vtbl__CHOICE))
#define VTBL_ENUMERATED                (&(a2c_vtbl__ENUMERATED))

#define OFFSET(parent,element)         ((UINT4)&(((parent*)0)->element))
#define AOFFSET(parent,element)        ((UINT4)&(((parent*)0)->element))
#define COFFSET(parent,element)        ((UINT4)&(((parent*)0)->u.element))

#define TEMPLATE_SIZE(t)               LENGTH_OF_ARRAY((t))

#define NO_ELIPSIS                     (0)
#define ELIPSIS                        (1)

typedef struct {
    int                elipsis;
    int                length;
    asn__enumerated   *value;
} ENUMERATION;


struct asn_template_ {
    int            indent;
    UINT4          parent;
    UINT4          me;
    VOID          *loc; 
    VOID          *obj; 
    UINT4          tag_class;
    UINT4          tag_number;
    UINT4          tag_tagging;
    UINT4          sizeoftype;
    UINT4          offset;
    INT4           min_size;
    INT4           max_size;
    VOID          *defaults;
    ENUMERATION   *enumeration;
    VOID          *vtbl;
    UINT4          flags;
    INT4           number;
};

struct asn_type_info_ {
    struct asn_template_  *template;
    char                  *name;
    UINT4                  size;
};

struct a2c_module_ {
    UINT4                   size;
    struct asn_type_info_  *types;
    char                   *name;
};



#ifdef __cplusplus
}
#endif
#endif

