/*
 *
 *    a2c_encode.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/a2c_encode.c,v 
 *    $Revision: 1.1 
 *    $Date: 1997/06/25 03:01:12 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */


#include <io_ctx_memio.h>
#include <alloc_ctx_pool.h>

#include "asn.h"
#include "asn_encode_ctx.h"
#include "asn_error.h"
#include "a2c.h"
#include "a2c_ctx.h"
#include "a2c_dot_c_file.h"
#include "berencode.h"
#include "ber_length.h"
#include "ber_tag.h"
#include "params.h"
#include "runtime.h"
#include "a2c_vtbl.h"


#ifndef LINT
static char rcsid[] = "$Id: a2c_encode.c,v 1.1 1997/06/25 03:01:12 briank Exp ";
#endif



static int    encode_tag
  PROTO_LIST((
    io_ctx                   outstream,
    struct asn_encode_ctx_  *ctx,
    struct a2c_module_      *module,
    UINT4                    selector,
    UINT4                   *index,
    VOID                    *data,
    char                     error[ERROR_MESSAGE_LENGTH]
  ));


static int    encode_tlv_list
  PROTO_LIST((
    io_ctx                   outstream,
    struct asn_encode_ctx_  *ctx,
    struct a2c_module_      *module,
    UINT4                    selector,
    VOID                    *data,
    char                     error[ERROR_MESSAGE_LENGTH]
  ));



static int
encode_tag(outstream, ctx, module, selector, index, data, error)
    io_ctx                   outstream;
    struct asn_encode_ctx_  *ctx;
    struct a2c_module_      *module;
    UINT4                    selector;
    UINT4                   *index;
    VOID                    *data;
    char                     error[ERROR_MESSAGE_LENGTH];
{
    int                       status       = NO_ERROR;
    int                       result       = NO_ERROR;
    struct asn_template_     *current      = 0;
    tag_type                  tag          = 0;
UNUSED_ARG(outstream);
UNUSED_ARG(data);
UNUSED_ARG(error);

    current = &(module->types[selector].template[*index]);

    if (! ctx->skip_next_tag) {
struct asn_tlv_           *tlv = 0;

        tag = current->tag_class | current->tag_number;

        status = create_asn_tlv(ctx->tmp_heap, (void**)&tlv);
        if (status != NO_ERROR) ERETURN(status);

        tlv->tag = tag;

        status = insert_asn_tlv(ctx, tlv);
        if (status != NO_ERROR) ERETURN(status);

        status = push_asn_tlv(ctx, tlv);
        if (status != NO_ERROR) ERETURN(status);

    }

    switch (current->tag_tagging) {
    case TAGGING_BASE:
        if (current->tag_class == CLASS_UNIVERSAL) {
            switch (current->tag_number) {
            case TAG_SEQUENCE:   /* same as SEQUENCE OF */
            case TAG_SET:        /* same as SET OF */
                tag = ctx->STACK[ctx->stack_index]->tag;
                ctx->STACK[ctx->stack_index]->tag = SET_CONSTRUCTED(tag);
            }
        }
        ctx->skip_next_tag = 0;
        break;

    case TAGGING_EXPLICIT:
        ctx->skip_next_tag = 0;
        tag = ctx->STACK[ctx->stack_index]->tag;
        ctx->STACK[ctx->stack_index]->tag = SET_CONSTRUCTED(tag);
        break;

    case TAGGING_IMPLICIT:
        ctx->skip_next_tag = 1;
        break;

    default:
        ASSERT(!"bad tag_tagging");
    }


    return result;
}


static int
encode_tlv_list(outstream, ctx, module, selector, data, error)
    io_ctx                   outstream;
    struct asn_encode_ctx_  *ctx;
    struct a2c_module_      *module;
    UINT4                    selector;
    VOID                    *data;
    char                     error[ERROR_MESSAGE_LENGTH];
{
    int                status = NO_ERROR;
    struct asn_tlv_   *tlv = 0;
    int                nwritten = 0;
UNUSED_ARG(module);
UNUSED_ARG(selector);
UNUSED_ARG(data);
UNUSED_ARG(error);

    for (tlv = ctx->list_head; tlv != 0; tlv = tlv->next) {
        if (tlv->tag == TAG_EOC) continue;  /* skip end-of-contents */

        status = put_tag(outstream, tlv->tag);
        if (status != 0) ERETURN(status);
        status = put_length(outstream, tlv->length);
        if (status != 0) ERETURN(status);
        if (tlv->value) {
          nwritten = IO_fwrite((char*)tlv->value, 1, tlv->length, outstream);
          if (nwritten != tlv->length) ERETURN(A2C_ERR_OUTSTREAM_LEN);
        }
    }

    return NO_ERROR;
}





PRIVATE int
__encode(outstream, ctx, module, selector, index, data, error)
    io_ctx                    outstream;
    struct asn_encode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status          = NO_ERROR;
    struct asn_type_info_    *type            = 0;
    struct asn_template_     *current         = 0;
    struct vtbl_             *vtbl            = 0;
    void                     *el_data         = 0;
    int                       optional        = 0;
    int                       verify          = 0;
    tag_type                  tag             = 0;
    int                       create_buffer   = 0;
    int                       len;
    int                       nread;
    int			      indefinite      = 0;
    
io_ctx  YYY       = 0;

    (VOID)query_a2c_ctx(ctx->app_ctx, A2C_CTX_QUERY_INDEFINITE, &indefinite);

    if(indefinite)
      return(__ber_encode(outstream,ctx,module,selector,index,data,error));
	
    type = &(module->types[selector]);

    if (*index >= type->size) ERETURN(A2C_ERR_STRUCT);

    current = &(type->template[*index]);

    optional = current->flags & FLAG_OPTIONAL;


    /* if this element doesn't exist, don't output it */
    if (optional && ! BIT_IS_SET(data, current->number)) {
        status = fast_forward(module, selector, index);
        if (status != NO_ERROR) ERETURN(status);
        return NO_ERROR;
    }


    /* ANY must have the tag 'encoded' in order to deal with IMPLICIT */
    /* tagging--logic in encode_ANY handles setting the tag correctly */
    if (! (current->tag_class == CLASS_SPECIAL
      && current->tag_number == TAG_CHOICE))
    {
        status = encode_tag(
                outstream, ctx, module, selector, index, data, error);
        if (status != NO_ERROR) ERETURN(status);
    }

    if (current->tag_class == CLASS_SPECIAL) {
        switch (current->tag_number) {
        case TAG_CHOICE:
            create_buffer = 0;
            break;
        case TAG_ANY:
            create_buffer = 1;
            break;
        default:
            ERETURN(A2C_ERR_INTERNAL);
            break;
        }
    }
    else {

        create_buffer = 0;


        tag = ctx->list_tail->tag;
        if (IS_PRIMITIVE(tag) && current->tag_tagging == TAGGING_BASE) {
            if (tag != TAG_EOC) create_buffer = 1;
        }  
    }

    el_data = ((char*)data) + current->offset;


    vtbl = GET_VTBL(current->vtbl);

    (VOID)query_a2c_ctx(ctx->app_ctx, A2C_CTX_QUERY_VERIFY, &verify);
    if (verify) {
        status = vtbl->verify(0, 0, module, selector, index, el_data, error);
        if (status != NO_ERROR) ERETURN(status);
    }

    if (create_buffer) {
        /* create a buffer in which to place the encoded object */
        status = create_memio_ctx(&YYY, 0);
        if (status != NO_ERROR) ERETURN(status);
    }

    status = vtbl->encode(YYY, ctx, module, selector, index, el_data, error);
    if (status != NO_ERROR) {
        if (create_buffer) (VOID)IO_delete(&YYY);
        ERETURN(status);
    }

    if (create_buffer) {
        /* make a copy of the io_ctx (buffer) and delete the io_ctx */
        status = IO_fseek(YYY, 0, FSEEK_EOF);
        if (status == -1) ERETURN(A2C_ERR_INTERNAL);
        len = IO_ftell(YYY);
        if (len == -1) ERETURN(A2C_ERR_INTERNAL);
        ctx->list_tail->length = len;
        ctx->list_tail->value = AC_malloc(ctx->tmp_heap, len);
        if (ctx->list_tail->value == 0) {
            (VOID)IO_delete(&YYY);
            ERETURN(A2C_ERR_NO_MEMORY);
        }
        (VOID)IO_fseek(YYY, 0, FSEEK_BOF);
        if (status == -1) ERETURN(A2C_ERR_INTERNAL);
        nread = IO_fread((char*)ctx->list_tail->value, 1, len, YYY);
        (VOID)IO_delete(&YYY);
        if (nread != len) ERETURN(A2C_ERR_INTERNAL);
        ctx->list_tail->is_encoded = 1;
status = pop_asn_tlv(ctx);
if (status != NO_ERROR) ERETURN(status);
    }

    if (current->tag_class == CLASS_UNIVERSAL
     && current->tag_number == TAG_EOC)
    {
        status = pop_asn_tlv(ctx);
        if (status != NO_ERROR) ERETURN(status);
        status = pop_asn_tlv(ctx);
        if (status != NO_ERROR) ERETURN(status);
    }

    return NO_ERROR;
}



PRIVATE int
encode(app_ctx, outstream, module_opaque, selector, data, error)
    a2c_ctx            app_ctx;
    io_ctx             outstream;
    a2c_module         module_opaque;
    UINT4              selector;
    void              *data;
    char               error[ERROR_MESSAGE_LENGTH];
{
    int                       status  = NO_ERROR;
    struct a2c_module_       *module  = (struct a2c_module_*)module_opaque;
    UINT4                     index   = 0;
    struct asn_encode_ctx_    ctx__, *ctx = &ctx__;


    status = check_parameters(CHECK_ENCODE_PARAMETERS,
                          outstream, module, selector, &index, data, error);
    if (status != NO_ERROR) ERETURN(A2C_ERR_BAD_PARAMETER);

    if (error != 0) error[0] = '\0';


MEMZERO(ctx__);
ctx->app_ctx = app_ctx;
ctx->data = data;
status = create_pool_ctx(&ctx->tmp_heap);
if (status != NO_ERROR) ERETURN(a2c_extern_error(A2C_ERR_NO_MEMORY));
ctx->STACK[0] = &ctx->wrapper;


    status = __encode(outstream, ctx, module, selector, &index, data, error);
    if (status != NO_ERROR) {
        (VOID)AC_delete(&ctx->tmp_heap);
        ERETURN(a2c_extern_error(status));
    }

    if (ctx->stack_index != 0) {
        (VOID)AC_delete(&ctx->tmp_heap);
        ERETURN(A2C_ERR_INTERNAL);
    }

    /* we're done with the object */

    status = encode_tlv_list(outstream, ctx, module, selector, data, error);
    (VOID)AC_delete(&ctx->tmp_heap);
    if (status != NO_ERROR) ERETURN(a2c_extern_error(status));

    return NO_ERROR;
}



