/*
 *
 *    a2c_util.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/a2c_util.c,v 
 *    $Revision: 1.8 
 *    $Date: 1997/06/06 00:30:57 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */



#include "asn_error.h"
#include "a2c.h"


#ifndef LINT
static char rcsid[] = "$Id: a2c_util.c,v 1.8 1997/06/06 00:30:57 briank Exp ";
#endif



PRIVATE int
COPY_ANY(heap, dest, src)
    alloc_ctx    heap;
    asn__any    *dest;
    asn__any    *src;
{
    /* asn__any and asn__octet_string have the same internal structure */
    return COPY_OCTET_STRING(heap, (asn__octet_string*)dest,
                                   (asn__octet_string*)src);
}


PRIVATE int
COPY_BMP_STRING(heap, dest, src)
    alloc_ctx          heap;
    asn__bmp_string   *dest;
    asn__bmp_string   *src;
{
    int      len = 2*(bmplen(*src)+1);

    *dest = (asn__bmp_string)AC_malloc(heap, len);
    if (*dest == 0) ERETURN(SE_ERR_NO_MEMORY);  /* memory failure */
    (VOID)memcpy(*dest, *src, len);

    return NO_ERROR;
}


PRIVATE int
COPY_GENERALIZED_TIME(heap, dest, src)
    alloc_ctx                    heap;
    asn__generalized_time       *dest;
    asn__generalized_time       *src;
{
UNUSED_ARG(heap);
    *dest = *src;
    return NO_ERROR;
}


PRIVATE int
COPY_OBJECT_IDENTIFIER(heap, dest, src)
    alloc_ctx                heap;
    asn__object_identifier  *dest;
    asn__object_identifier  *src;
{
    /* asn__object_identifier and asn__octet_string have the same internal structure */
    return COPY_OCTET_STRING(heap, (asn__octet_string*)dest,
                                   (asn__octet_string*)src);
}


PRIVATE int
COPY_OCTET_STRING(heap, dest, src)
    alloc_ctx            heap;
    asn__octet_string   *dest;
    asn__octet_string   *src;
{
    VOID   *tmpv = src->value;
    int     tmpl = src->length;

    dest->value = (UCHAR*)AC_malloc(heap, tmpl);
    if (dest->value == 0) ERETURN(SE_ERR_NO_MEMORY);   /* memory failure */
    (VOID)memcpy(dest->value, tmpv, tmpl);
    dest->length = tmpl;

    return NO_ERROR;
}


PRIVATE int
COPY_PRINTABLE_STRING(heap, dest, src)
    alloc_ctx                heap;
    asn__printable_string   *dest;
    asn__printable_string   *src;
{
    int      len = strlen(*src)+1;

    *dest = (asn__printable_string)AC_malloc(heap, len);
    if (*dest == 0) ERETURN(SE_ERR_NO_MEMORY);  /* memory failure */
    (VOID)memcpy(*dest, *src, len);

    return NO_ERROR;
}


PRIVATE int
COPY_UTC_TIME(heap, dest, src)
    alloc_ctx            heap;
    asn__utc_time       *dest;
    asn__utc_time       *src;
{
UNUSED_ARG(heap);
    *dest = *src;
    return NO_ERROR;
}




/* utility macro used by the a2c BORROW functions */
#define A2C_BORROW__(dest,src,type)   ((VOID)memcpy((dest),(src),sizeof(type)))


PRIVATE int
BORROW_ANY(dest, src)
    asn__any    *dest;
    asn__any    *src;
{
    A2C_BORROW__((dest),(src),asn__any);
    return NO_ERROR;
}


PRIVATE int
BORROW_BMP_STRING(dest, src)
    asn__bmp_string   *dest;
    asn__bmp_string   *src;
{
    (*(dest)=*(src));
    return NO_ERROR;
}


PRIVATE int
BORROW_GENERALIZED_TIME(dest, src)
    asn__generalized_time       *dest;
    asn__generalized_time       *src;
{
    (*(dest)=*(src));
    return NO_ERROR;
}


PRIVATE int
BORROW_OBJECT_IDENTIFIER(dest, src)
    asn__object_identifier  *dest;
    asn__object_identifier  *src;
{
    A2C_BORROW__((dest),(src),asn__object_identifier);
    return NO_ERROR;
}


PRIVATE int
BORROW_OCTET_STRING(dest, src)
    asn__octet_string   *dest;
    asn__octet_string   *src;
{
    A2C_BORROW__((dest),(src),asn__octet_string);
    return NO_ERROR;
}


PRIVATE int
BORROW_PRINTABLE_STRING(dest, src)
    asn__printable_string   *dest;
    asn__printable_string   *src;
{
    (*(dest)=*(src));
    return NO_ERROR;
}


PRIVATE int
BORROW_UTC_TIME(dest, src)
    asn__utc_time       *dest;
    asn__utc_time       *src;
{
    (*(dest)=*(src));
    return NO_ERROR;
}


