/*
 *
 *    asn_encode_ctx.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/asn_encode_ctx.c,v 
 *    $Revision: 1.9 
 *    $Date: 1997/06/06 00:31:03 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */


#include <define.h>
#include <io_ctx.h>

#include "asn.h"
#include "asn_encode_ctx.h"
#include "asn_error.h"
#include "asn_tlv.h"
#include "ber_length.h"
#include "ber_tag.h"
#include "runtime.h"


#ifndef LINT
static char rcsid[] = "$Id: asn_encode_ctx.c,v 1.9 1997/06/06 00:31:03 briank Exp ";
#endif










PRIVATE int
push_asn_tlv(ctx, tlv)
    struct asn_encode_ctx_   *ctx;
    struct asn_tlv_          *tlv;
{
    if (ctx == 0 || tlv == 0) ERETURN(A2C_ERR_BAD_PARAMETER);

    ++(ctx->stack_index);
    ctx->STACK[(ctx->stack_index)] = tlv;


    return NO_ERROR;
}

PRIVATE int
pop_asn_tlv(ctx)
    struct asn_encode_ctx_   *ctx;
{
    int    length = 0;
    struct asn_tlv_  *tlv = 0;

    if (ctx == 0) ERETURN(A2C_ERR_BAD_PARAMETER);

    if (ctx->stack_index < 1) ERETURN(A2C_ERR_STRUCT);

    tlv = ctx->STACK[ctx->stack_index];

    if (tlv->tag != TAG_EOC) {

        /* first, get the length of the value */
        length = tlv->length;
        length += sizeof_encoded_length(length);

        length += sizeof_encoded_tag(tlv->tag);
    }

    ctx->STACK[ctx->stack_index] = 0;

    --(ctx->stack_index);

    /* add the total length to the parent's length */

    ctx->STACK[ctx->stack_index]->length += length;


    return NO_ERROR;
}


PRIVATE int
insert_asn_tlv(ctx, tlv)
    struct asn_encode_ctx_ *ctx;
    struct asn_tlv_ *tlv;
{
    if (ctx == 0 || tlv == 0) ERETURN(A2C_ERR_BAD_PARAMETER);

    if (ctx->list_head == 0) {
        ctx->list_head = tlv;
        ctx->list_tail = tlv;
    }
    else {
        ctx->list_tail->next = tlv;
        ctx->list_tail = tlv;
    }

    return NO_ERROR;
}


