/*
 *
 *    asn_types.h
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/asn_types.h,v 
 *    $Revision: 1.12 
 *    $Date: 1997/06/25 03:04:01 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */

#ifndef __ASN_TYPES_H__
#define __ASN_TYPES_H__


#include <time.h>

#include <alloc_ctx.h>
#include <define.h>


#ifdef __cplusplus
extern "C" {
#endif


/* length must be at least 2 because of the tag and length octets */
typedef struct asn__any_ {
    UINT4   length;
    UCHAR  *object;   /* contains the tag and length octets */
} asn__any;

/* if length is 0, then value may be 0 */
typedef struct asn__bit_string_
{
    UINT4   length;   /* number of bits */
    UCHAR  *value;
} asn__bit_string;

/* if length is 0, then value may be 0 */
typedef struct asn__octet_string_ {
    UINT4   length;
    UCHAR  *value;
} asn__octet_string;

/* object identifiers cannot have a length of 0 */
typedef struct asn__object_identifier_ {
    UINT4   length;
    UCHAR  *value;
} asn__object_identifier;

typedef UCHAR      asn__boolean;
typedef long       asn__enumerated;
typedef time_t     asn__generalized_time;
typedef long       asn__integer;
typedef VOID       asn__null;
typedef double     asn__real;
typedef time_t     asn__utc_time;

/* never specify a 0 pointer for the following string types, 
 * use a zero-length string ("\0") instead */
typedef char      *asn__ia5_string;
typedef char      *asn__numeric_string;
typedef char      *asn__printable_string;
typedef char      *asn__visible_string;

/* never specify a 0 pointer for the following string type, 
 * use a zero-length string ((asn__bmp_string)"\0\0") instead */
typedef UINT2     *asn__bmp_string;




#define __BYTE__(obj,bit)        (((UCHAR*)obj)[((UINT2)bit)/8])
#define __BIT__(bit)             (((UINT2)0x80)>>(((UINT2)bit)%8))

#define SET_PRESENT(obj,bit)     (BIT_SET((obj)._present,bit))
#define IS_PRESENT(obj,bit)      (BIT_IS_SET((obj)._present,bit))
#define SET_ABSENT(obj,bit)      (BIT_UNSET((obj)._present,bit))
#define IS_ABSENT(obj,bit)       (!IS_PRESENT(obj,bit))

#define BIT_SET(obj,bit)         ((VOID)(__BYTE__(obj,bit) |= __BIT__(bit)))
#define BIT_UNSET(obj,bit)       ((VOID)(__BYTE__(obj,bit) &= ~(__BIT__(bit))))
#define BIT_IS_SET(obj,bit)      (!!(__BYTE__(obj,bit) & __BIT__(bit)))

#define __SAME_LEN__(obj1,obj2)  ((obj1).length==(obj2.length))
#define __SAME_VAL__(obj1,obj2)  (!memcmp(obj1.value,obj2.value,obj1.length))
#define __SAME_OBJ__(obj1,obj2)  (!memcmp(obj1.object,obj2.object,obj1.length))

#define SAME_ANY(any1,any2)                  ((__SAME_LEN__((any1),(any2)) \
                                             &&__SAME_OBJ__((any1),(any2))))
#define SAME_BMP_STRING(str1,str2)           (bmpcmp((str1),(str2)))
#define SAME_OCTET_STRING(oct1,oct2)         ((__SAME_LEN__((oct1),(oct2)) \
                                             &&__SAME_VAL__((oct1),(oct2))))
#define SAME_OBJECT_IDENTIFIER(oid1,oid2)    ((__SAME_LEN__((oid1),(oid2)) \
                                             &&__SAME_VAL__((oid1),(oid2))))


/*
 *    CONVENIENCE FUNCTIONS
 */
PUBLIC asn__bmp_string    bmpchr
  PROTO_LIST((
    asn__bmp_string           str,
    int                       c
  ));

PUBLIC int                bmpcmp
  PROTO_LIST((
    asn__bmp_string           str1,
    asn__bmp_string           str2
  ));

PUBLIC asn__bmp_string    bmpdup
  PROTO_LIST((
    asn__bmp_string           str
  ));

PUBLIC int                bmplen
  PROTO_LIST((
    asn__bmp_string           str
  ));

PUBLIC int                bmpstrcmp
  PROTO_LIST((
    asn__bmp_string           bmpstr,
    char                     *str 
  ));





/* "COPY" functions allocate memory for "dest" (if relevant),
 * performing a deep copy */

PUBLIC int    COPY_ANY
  PROTO_LIST((
    alloc_ctx    heap,
    asn__any    *dest,
    asn__any    *src
  ));

PUBLIC int    COPY_BMP_STRING
  PROTO_LIST((
    alloc_ctx          heap,
    asn__bmp_string   *dest,
    asn__bmp_string   *src
  ));

PUBLIC int    COPY_GENERALIZED_TIME
  PROTO_LIST((
    alloc_ctx                    heap,
    asn__generalized_time       *dest,
    asn__generalized_time       *src
  ));

PUBLIC int    COPY_OBJECT_IDENTIFIER
  PROTO_LIST((
    alloc_ctx                heap,
    asn__object_identifier  *dest,
    asn__object_identifier  *src
  ));

PUBLIC int    COPY_OCTET_STRING
  PROTO_LIST((
    alloc_ctx            heap,
    asn__octet_string   *dest,
    asn__octet_string   *src
  ));

PUBLIC int    COPY_PRINTABLE_STRING
  PROTO_LIST((
    alloc_ctx                heap,
    asn__printable_string   *dest,
    asn__printable_string   *src
  ));

PUBLIC int    COPY_UTC_TIME
  PROTO_LIST((
    alloc_ctx            heap,
    asn__utc_time       *dest,
    asn__utc_time       *src
  ));


/* "BORROW" functions operate like 'dest = src', performing
 * a shallow copy */

PUBLIC int    BORROW_ANY
  PROTO_LIST((
    asn__any    *dest,
    asn__any    *src
  ));

PUBLIC int    BORROW_BMP_STRING
  PROTO_LIST((
    asn__bmp_string   *dest,
    asn__bmp_string   *src
  ));

PUBLIC int    BORROW_GENERALIZED_TIME
  PROTO_LIST((
    asn__generalized_time       *dest,
    asn__generalized_time       *src
  ));

PUBLIC int    BORROW_OBJECT_IDENTIFIER
  PROTO_LIST((
    asn__object_identifier  *dest,
    asn__object_identifier  *src
  ));

PUBLIC int    BORROW_OCTET_STRING
  PROTO_LIST((
    asn__octet_string   *dest,
    asn__octet_string   *src
  ));

PUBLIC int    BORROW_PRINTABLE_STRING
  PROTO_LIST((
    asn__printable_string   *dest,
    asn__printable_string   *src
  ));

PUBLIC int    BORROW_UTC_TIME
  PROTO_LIST((
    asn__utc_time       *dest,
    asn__utc_time       *src
  ));


/* for backwards compatibility */
#define GRAB_ANY                  BORROW_ANY
#define GRAB_BMP_STRING           BORROW_BMP_STRING
#define GRAB_GENERALIZED_TIME     BORROW_GENERALIZED_TIME
#define GRAB_OBJECT_IDENTIFIER    BORROW_OBJECT_IDENTIFIER
#define GRAB_OCTET_STRING         BORROW_OCTET_STRING
#define GRAB_PRINTABLE_STRING     BORROW_PRINTABLE_STRING
#define GRAB_UTC_TIME             BORROW_UTC_TIME


/* the following typedef is not part of the public API, but it
 * is here because a2c_module appears in the .h files */
typedef VOID*   a2c_module;



#ifdef __cplusplus
}
#endif
#endif

