/*
 *
 *    ber_length.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/ber_length.c,v 
 *    $Revision: 1.10 
 *    $Date: 1997/06/06 00:31:08 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */


#include <io_ctx.h>

#include "asn.h"
#include "asn_error.h"
#include "ber_length.h"
#include "runtime.h"


#ifndef LINT
static char rcsid[] = "$Id: ber_length.c,v 1.10 1997/06/06 00:31:08 briank Exp ";
#endif



PRIVATE int
read_length_bytes(instream, buffer, nread, bmax)
    io_ctx   instream;
    UCHAR   *buffer;
    int     *nread;
    int      bmax;
{
    int             c;
    int             len;
    int             i;

    if (bmax < 1) ERETURN(A2C_ERR_BUFFER);

    if (*nread == 0) {
        c = IO_fgetc(instream);
        if (c == EOF) ERETURN(A2C_ERR_EOF);
        buffer[0] = c;
        ++(*nread);
    }

    c = buffer[0];
    if (c != 0x80) {
        if (c & 0x80) {
            len = (c & 0x7F);
            if (len+1 >= bmax) ERETURN(A2C_ERR_BUFFER);
            for (i = 1; i <= len; ++i) {
                c = IO_fgetc(instream);
                if (c == EOF) ERETURN(A2C_ERR_EOF);
                buffer[i] = c;
                ++(*nread);
            }
        }
    }

    return NO_ERROR;
}


PRIVATE int
buffer2length(buffer, blen, len)
    UCHAR         *buffer;
    int            blen;
    length_type   *len;
{
    int             c;
    int             bytes = sizeof(length_type);

    if (blen < 1) ERETURN(A2C_ERR_BAD_LEN);

    *len = 0;

    c = buffer[0];

    if (c == 0x80) {
        if (blen != 1) ERETURN(A2C_ERR_BAD_LEN);
        *len = INDEFINITE_LENGTH;
    }
    else {
        if (c & 0x80) {
            if ((c & 0x7F)+1 != blen) ERETURN(A2C_ERR_BAD_LEN);
            if ((c & 0x7F) > bytes) ERETURN(A2C_ERR_LEN_LEN);
            *len = 0;
            for (bytes = (c & 0x7F); bytes > 0; --bytes) {
                c = *(++buffer);
                *len = (*len << 8) + c;
            }
        }
        else {
            if (blen != 1) ERETURN(A2C_ERR_BAD_LEN);
            *len = c;
        }
    }
    return NO_ERROR;
}


PRIVATE int
get_length(instream, buffer, blen, bufmax, length)
    io_ctx                   instream;
    UCHAR                   *buffer;
    int                     *blen;
    int                      bufmax;
    length_type             *length;
{
    int     status = NO_ERROR;

    status = read_length_bytes(instream, buffer, blen, bufmax);
    if (status != NO_ERROR) ERETURN(status);

    status = buffer2length(buffer, *blen, length);
    if (status != NO_ERROR) ERETURN(status);

    return NO_ERROR;
}






/* returns 0 on error */
PRIVATE unsigned int
sizeof_encoded_length(length)
    length_type  length;
{                         
    unsigned int  size = 0;

    if ((unsigned long)length == INDEFINITE_LENGTH) return 1;

    if      (length < (0x1<<7 )) size = 1;
    else if (length < (0x1<<8 )) size = 2;
    else if (length < (0x1<<16)) size = 3;
    else if (length < (0x1<<24)) size = 4;
    else if ((unsigned long)length < INDEFINITE_LENGTH) size = 5;

    return size;
}               



PRIVATE int
put_length(outstream, length)
    io_ctx       outstream;
    length_type  length;
{
    int    nbytes;
    int    c;

    nbytes = sizeof_encoded_length(length);

    /*  sizeof_encoded_length returns 0 on failure
     *  sizeof(length_type)*2 is larger than is need to hold the length */
    if (nbytes == 0 || nbytes > (int)sizeof(length_type)*2) {
        ERETURN(SE_ERR_INTERNAL);
    }

    if (nbytes == 1) {
        /* short form */
        c = IO_fputc((length&0x7F), outstream);
        if (c == EOF) ERETURN(A2C_ERR_OUTSTREAM_LEN);
    }
    else {
        /* long form */
        unsigned char  array[sizeof(length_type)*2];
        unsigned char  *encoded_length;
        unsigned int   div;
        unsigned int   mod;

        encoded_length = array + nbytes;
        div = length;
        for (c = 1; c < nbytes; ++c) {
            mod = div % 256;
            div /= 256;
            *encoded_length = mod;
            --encoded_length;
        }
        *encoded_length = 0x80|(nbytes-1);
        c = IO_fwrite((char*)encoded_length, 1, nbytes, outstream);
        if (c != nbytes) ERETURN(A2C_ERR_OUTSTREAM_LEN);
    }

    return NO_ERROR;
}


 


PRIVATE int
initiate_get_length(ctx, instream)
    struct asn_decode_ctx_  *ctx;
    io_ctx                   instream;
{
UNUSED_ARG(instream);

    ctx->length.nread = 0;

    if (ctx->insert_def_length_eoc) {
    }
    else {
    }
    
    return NO_ERROR;
}


PRIVATE int
continue_get_length(ctx, instream)
    struct asn_decode_ctx_  *ctx;
    io_ctx                   instream;
{
    length_type              length = 0;
    int status;

    if (ctx->insert_def_length_eoc) {
        set_current_length(ctx, 0);
    }
    else {
        status = get_length(instream,
            ctx->length.data, &ctx->length.nread,
            sizeof(ctx->length.data), &length);
        set_current_length(ctx, length);
        if (status != NO_ERROR) ERETURN(status);
    }

    return NO_ERROR;
}

PRIVATE int
finalize_get_length(ctx, instream)
    struct asn_decode_ctx_  *ctx;
    io_ctx                   instream;
{         
    tag_type                 tag     = 0;
    long                     offset  = 0; 

    ctx->have_length = 1;



    tag = get_current_tag(ctx);
    if (IS_CONSTRUCTED(tag)) {
        offset = IO_ftell(instream);
        if (offset == -1) ERETURN(A2C_ERR_BAD_FTELL);
    }
    else {
        offset = 0;
    }
    set_current_offset(ctx, offset);

    return NO_ERROR;
}    








