/*
 *
 *    ber_tag.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/ber_tag.c,v 
 *    $Revision: 1.10 
 *    $Date: 1997/06/06 00:31:10 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */



#include "asn_error.h"
#include "ber_tag.h"
#include "runtime.h"


#ifndef LINT
static char rcsid[] = "$Id: ber_tag.c,v 1.10 1997/06/06 00:31:10 briank Exp ";
#endif



static int    insert_class_and_method
  PROTO_LIST((
    tag_type        tag,
    unsigned char  *byte 
  ));




PRIVATE int 
read_tag_bytes(instream, buffer, nread, bmax)
    io_ctx   instream;
    UCHAR   *buffer;
    int     *nread;
    int      bmax;
{
    int             c;

    if (bmax < 1) ERETURN(A2C_ERR_BUFFER);

    if (*nread == 0) {
        c = IO_fgetc(instream);
        if (c == EOF) ERETURN(A2C_ERR_EOF);
        buffer[0] = c;
        ++(*nread);
    }

    if ((buffer[0] & 0x1F) == 0x1F) {   /* is high tag form? */
        do {
            if (*nread >= bmax) ERETURN(A2C_ERR_BUFFER);
            c = IO_fgetc(instream);
            if (c == EOF) ERETURN(A2C_ERR_EOF);
            buffer[*nread] = c;
            ++(*nread);
        } while (c & 0x80);             /* while not the last high tag byte */
    }

    return NO_ERROR;
}


PRIVATE int
buffer2tag(buffer, length, tag)
    UCHAR      *buffer;
    int         length;
    tag_type   *tag;
{
    unsigned int    tag_class;
    unsigned int    method;
    int             bits = __N_TAG_BITS;
    int             c;
    int             i;

    if (length < 1) ERETURN(A2C_ERR_TAG_LEN);

    *tag = 0;

    c = buffer[0];
    tag_class   = (c & 0xC0)>>(8-2);
    method      = (c & 0x20)>>(8-3);

    if ((c & 0x1F) == 0x1F) {   /* is high tag form? */
        for (i = 1; i < length; ++i) {
            if (bits < 7+3) ERETURN(A2C_ERR_TAG_LEN);
            c = buffer[i];
            if (c == EOF) ERETURN(A2C_ERR_EOF);
            *tag = (*tag << 7) + (c & 0x7F);
            bits -= 7;
        }
        if (c & 0x80) ERETURN(A2C_ERR_BAD_TAG);
    }
    else {
        *tag = (c & 0x1F);
    }

    switch (tag_class) {
    case 0x0:
        *tag = SET_UNIVERSAL(*tag);
        break;
    case 0x1:
        *tag = SET_APPLICATION(*tag);
        break;
    case 0x2:
        *tag = SET_CONTEXT(*tag);
        break;
    case 0x3:
        *tag = SET_PRIVATE(*tag);
        break;
    default:
        ERETURN(A2C_ERR_CLASS_NUMBER);
        break;
    }

    if (method)  *tag = SET_CONSTRUCTED(*tag);
    else         *tag = SET_PRIMITIVE(*tag);

    return NO_ERROR;
}


PRIVATE int
get_tag(instream, buffer, length, bufmax, tag)
    io_ctx                   instream;
    UCHAR                   *buffer;
    int                     *length;
    int                      bufmax;
    tag_type                *tag;
{
    int     status = NO_ERROR;

    status = read_tag_bytes(instream, buffer, length, bufmax);
    if (status != NO_ERROR) ERETURN(status);

    status = buffer2tag(buffer, *length, tag);
    if (status != NO_ERROR) ERETURN(status);

    return NO_ERROR;
}


/* return 0 on error */
PRIVATE unsigned int
sizeof_encoded_tag(tag)
    tag_type  tag;
{
    unsigned int  size = 0;
    tag_type      tag_number;

    tag_number = TAG_NUMBER(tag);

    if      (tag_number <      0x1f) size = 1;
    else if (tag_number < (0x1<<7 )) size = 2;
    else if (tag_number < (0x1<<14)) size = 3;
    else if (tag_number < (0x1<<21)) size = 4;
    else if (tag_number < (0x1<<28)) size = 5;

    return size;
}



static int
insert_class_and_method(tag, byte)
  tag_type        tag;
  unsigned char  *byte;
{
    /* bit "6" (5) */
    if (IS_CONSTRUCTED(tag)) *byte |= (0x1<<5);

    /* bits "8-7" (7-6) */
    switch(CLASS(tag)) {
    case CLASS_UNIVERSAL:
        break;
    case CLASS_APPLICATION:
        *byte |= (0x1<<6);
        break;
    case CLASS_CONTEXT:
        *byte |= (0x2<<6);
        break;
    case CLASS_PRIVATE:
        *byte |= (0x3<<6);
        break;
    default:
        ERETURN(A2C_ERR_INTERNAL);
    }
    return NO_ERROR;
}


/* returns number of characters put into outstream */
PRIVATE int
put_tag(outstream, tag)
io_ctx      outstream;
tag_type    tag;
{
    int            nbytes;
    int            c;
    int            status = NO_ERROR;


    nbytes = sizeof_encoded_tag(tag);
    /*  sizeof_encoded_tag() returns 0 on failure
     *  sizeof(tag_type)*2 is larger than is need to hold the tag */
    if (nbytes == 0 || nbytes > (int)sizeof(tag_type)*2) {
        ERETURN(SE_ERR_INTERNAL);
    }

    if (nbytes == 1) {
        /* low-tag-number form */
        unsigned char  encoded_tag;

        /* bits "5-1" (4-0) */
        encoded_tag = (unsigned char)(tag & 0x1F);

        status = insert_class_and_method(tag, &encoded_tag);
        if (status != NO_ERROR) ERETURN(status);

        c = IO_fputc(encoded_tag, outstream);
        if (c < 0) ERETURN(A2C_ERR_OUTSTREAM_LEN);
    }
    else {
        /* high-tag-number form */
        unsigned char  array[sizeof(tag_type)*2];
        unsigned char  *encoded_tag = 0;
        unsigned int   div;
        unsigned int   mod;


        encoded_tag = array + nbytes;
        div = TAG_NUMBER(tag);

        mod = div % 128;
        div /= 128;
        *encoded_tag = mod;
        --encoded_tag;

        for (c = 2; c < nbytes; ++c) {
            mod = div % 128;
            div /= 128;
            *encoded_tag = mod | 0x80;
            --encoded_tag;
        }

        /* bits "5-1" (4-0) */
        *encoded_tag = 0x1F;

        status = insert_class_and_method(tag, encoded_tag);
        if (status != NO_ERROR) ERETURN(status);

        c = IO_fwrite((char*)encoded_tag, 1, nbytes, outstream);
        if (c != nbytes) ERETURN(A2C_ERR_OUTSTREAM_LEN);
    }

    return NO_ERROR;
}


PRIVATE int
initiate_get_tag(ctx, instream)
    struct asn_decode_ctx_  *ctx;
    io_ctx                   instream;
{
    ctx->tag.nread = 0;


    ctx->insert_def_length_eoc = insert_def_length_eoc(ctx, instream);

    return NO_ERROR;
}

PRIVATE int
continue_get_tag(ctx, instream)
    struct asn_decode_ctx_  *ctx;
    io_ctx                   instream;
{
    tag_type                 tag     = 0;
    int                      status  = NO_ERROR;

    if (ctx->insert_def_length_eoc) {


        tag = TAG_EOC;
    }
    else {
        status = get_tag(instream,
            ctx->tag.data, &ctx->tag.nread, sizeof(ctx->tag.data), &tag);
        if (status != NO_ERROR) ERETURN(status);
    }
    set_current_tag(ctx, tag);
    
    return NO_ERROR;
}

PRIVATE int 
finalize_get_tag(ctx, instream)
    struct asn_decode_ctx_  *ctx;
    io_ctx                   instream;
{         
UNUSED_ARG(instream);


    ctx->have_tag = 1;



    return NO_ERROR;
}         




