/*  berencode.c

Copyright (C) 1996, Visa International Service Association and MasterCard International Incorporated
All Rights Reserved.

 *    $Source:  
 *    $Revision:  
 *    $Date:  
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $

ekr@terisa.com  Mon Oct  7 17:58:59 1996
 */

#include "berencode.h"

#ifndef LINT
static char rcsid[]="$Id: berencode.c,v 1.14 1997/06/25 03:06:57 briank Exp ";
#endif

/*Based on*/
/*
 *
 *    encode.c
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */


#include <io_ctx_memio.h>
#include <alloc_ctx_pool.h>

#include "asn.h"
#include "asn_encode_ctx.h"
#include "asn_error.h"
#include "a2c.h"
#include "a2c_ctx.h"
#include "a2c_dot_c_file.h"
#include "ber_length.h"
#include "ber_tag.h"
#include "params.h"
#include "runtime.h"
#include "a2c_vtbl.h"


static int    encode_tag
  PROTO_LIST((
    io_ctx                   outstream,
    struct asn_encode_ctx_  *ctx,
    struct a2c_module_      *module,
    UINT4                    selector,
    UINT4                   *index,
    VOID                    *data,
    char                     error[ERROR_MESSAGE_LENGTH]
  ));



static int
encode_tag(outstream, ctx, module, selector, index, data, error)
    io_ctx                   outstream;
    struct asn_encode_ctx_  *ctx;
    struct a2c_module_      *module;
    UINT4                    selector;
    UINT4                   *index;
    VOID                    *data;
    char                     error[ERROR_MESSAGE_LENGTH];
{
    int                       status       = NO_ERROR;
    int                       result       = NO_ERROR;
    struct asn_template_     *current      = 0;
    tag_type                  tag          = 0;
    struct asn_tlv_          *tlv = 0;
UNUSED_ARG(outstream);
UNUSED_ARG(data);
UNUSED_ARG(error);

    current = &(module->types[selector].template[*index]);

    if (! ctx->skip_next_tag) {

        tag = current->tag_class | current->tag_number;

        status = create_asn_tlv(ctx->tmp_heap, (void**)&tlv);
        if (status != NO_ERROR) ERETURN(status);

        tlv->tag = tag;

        status = insert_asn_tlv(ctx, tlv);
        if (status != NO_ERROR) ERETURN(status);

        status = push_asn_tlv(ctx, tlv);
        if (status != NO_ERROR) ERETURN(status);

    }

    switch (current->tag_tagging) {
    case TAGGING_BASE:
        if (current->tag_class == CLASS_UNIVERSAL) {
            switch (current->tag_number) {
            case TAG_SEQUENCE:   /* same as SEQUENCE OF */
            case TAG_SET:        /* same as SET OF */
                tag = ctx->STACK[ctx->stack_index]->tag;
                ctx->STACK[ctx->stack_index]->tag = SET_CONSTRUCTED(tag);
            }
        }
        ctx->skip_next_tag = 0;
        break;

    case TAGGING_EXPLICIT:
        ctx->skip_next_tag = 0;
        tag = ctx->STACK[ctx->stack_index]->tag;
        ctx->STACK[ctx->stack_index]->tag = SET_CONSTRUCTED(tag);
        break;

    case TAGGING_IMPLICIT:
        ctx->skip_next_tag = 1;
        break;

    default:
        ASSERT(!"bad tag_tagging");
    }

    return result;
}






PRIVATE int
__ber_encode(outstream, ctx, module, selector, index, data, error)
    io_ctx                    outstream;
    struct asn_encode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status          = NO_ERROR;
    struct asn_type_info_    *type            = 0;
    struct asn_template_     *current         = 0;
    struct vtbl_             *vtbl            = 0;
    void                     *el_data         = 0;
    int                       optional        = 0;
    int                       verify          = 0;
    tag_type                  tag             = 0;
    int                       create_buffer   = 0;
    int			      skip_tag	      = 0;
    int                       len;
    int                       nread;
    
io_ctx  YYY       = 0;


    type = &(module->types[selector]);

    if (*index >= type->size) ERETURN(A2C_ERR_STRUCT);

    current = &(type->template[*index]);

    optional = current->flags & FLAG_OPTIONAL;


    /* if this element doesn't exist, don't output it */
    if (optional && ! BIT_IS_SET(data, current->number)) {
        status = fast_forward(module, selector, index);
        if (status != NO_ERROR) ERETURN(status);
        return NO_ERROR;
    }

    skip_tag=ctx->skip_next_tag;
    
    /* ANY must have the tag 'encoded' in order to deal with IMPLICIT */
    /* tagging--logic in encode_ANY handles setting the tag correctly */
    if (! (current->tag_class == CLASS_SPECIAL
      && current->tag_number == TAG_CHOICE))
    {
        status = encode_tag(
                outstream, ctx, module, selector, index, data, error);
        if (status != NO_ERROR) ERETURN(status);
    }

    if (current->tag_class == CLASS_SPECIAL) {
        switch (current->tag_number) {
        case TAG_CHOICE:
            create_buffer = 0;
            break;
        case TAG_ANY:
            create_buffer = 0;
            break;
        default:
            ERETURN(A2C_ERR_INTERNAL);
            break;
        }
    }
    else {

        create_buffer = 0;


        tag = ctx->list_tail->tag;

        if (IS_PRIMITIVE(tag) && current->tag_tagging == TAGGING_BASE){
            if (tag != TAG_EOC && tag!=TAG_ANY) create_buffer = 1;
        }
	
    }

    el_data = ((char*)data) + current->offset;


    vtbl = GET_VTBL(current->vtbl);

    (VOID)query_a2c_ctx(ctx, A2C_CTX_QUERY_VERIFY, &verify);
    if (verify) {
        status = vtbl->verify(0, 0, module, selector, index, el_data, error);
        if (status != NO_ERROR) ERETURN(status);
    }

    if (create_buffer) {
        /* create a buffer in which to place the encoded object */
        status = create_memio_ctx(&YYY, 0);
        if (status != NO_ERROR) ERETURN(status);
    }
    else{
      /*The only time we need to worry about outputting TL is
	when we haven't done create_buffer*/
      do{
	if(current->tag_class==CLASS_UNIVERSAL &&
	  current->tag_number==TAG_EOC)
	  break;

	if(current->tag_class==CLASS_SPECIAL)
	  break;
	if(skip_tag)
	  break;
	if(current->tag_tagging==TAGGING_IMPLICIT){
	  struct asn_template_     *next;

	  next = &(type->template[*index+1]);
	  if(next->tag_class == CLASS_SPECIAL &&
	    next->tag_number == TAG_ANY)
	    break;
	}
	
	/*Uh, this had better be constructed, or we've got a severe problem*/
	ctx->list_tail->tag=SET_CONSTRUCTED(tag);

	status=put_tag(outstream,ctx->list_tail->tag);
	if(status) ERETURN(A2C_ERR_INTERNAL);

	/*Put indefinite length, cause we're constructed indefinite length*/
	status=IO_fputc(0x80,outstream);
	if(status != 0x80) ERETURN(A2C_ERR_OUTSTREAM_LEN);
      } while(0);
    }
    
    status = vtbl->encode(
	 create_buffer?YYY:outstream,
	   ctx, module, selector, index, el_data, error);
    if (status != NO_ERROR) {
        if (create_buffer) (VOID)IO_delete(&YYY);
        ERETURN(status);
    }

    if (create_buffer) {
      UCHAR blk[1024];
        /* make a copy of the io_ctx (buffer) and delete the io_ctx */
        status = IO_fseek(YYY, 0, FSEEK_EOF);
        if (status == -1) ERETURN(A2C_ERR_INTERNAL);
        len = IO_ftell(YYY);
        if (len == -1) ERETURN(A2C_ERR_INTERNAL);

	status=put_tag(outstream,ctx->list_tail->tag);
	if(status != NO_ERROR) ERETURN(status);
	
	status=put_length(outstream, len);
	if(status != NO_ERROR) ERETURN(status);
	
        (VOID)IO_fseek(YYY, 0, FSEEK_BOF);
        if (status == -1) ERETURN(A2C_ERR_INTERNAL);
	while(len){
	  nread = IO_fread((char*)blk, 1, MIN(len,(int)sizeof(blk)), YYY);
	  status=IO_fwrite((char*)blk, 1, nread, outstream);
	  if(status!=nread) ERETURN(A2C_ERR_OUTSTREAM_LEN);

	  len-=nread;
	}
        (VOID)IO_delete(&YYY);
	status = pop_asn_tlv(ctx);
	if (status != NO_ERROR) ERETURN(status);
    }
    else {
      if(current->tag_class == CLASS_SPECIAL && current->tag_number==TAG_ANY){
	/*Used to be current->tag_tagging==IMPLICIT*/
	status = pop_asn_tlv(ctx);
	if (status != NO_ERROR) ERETURN(status);
      }
    }
    
    if (current->tag_class == CLASS_UNIVERSAL
     && current->tag_number == TAG_EOC)
    {
        status = pop_asn_tlv(ctx);
        if (status != NO_ERROR) ERETURN(status);
        status = pop_asn_tlv(ctx);
        if (status != NO_ERROR) ERETURN(status);

	/*throw on the terminating 0 length and tag*/
	status=put_tag(outstream,0);
	if(status != NO_ERROR) ERETURN(status);
	status=put_length(outstream, 0);
	if(status != NO_ERROR) ERETURN(status);
    }

    return NO_ERROR;
}



PRIVATE int
ber_encode(app_ctx, outstream, module_opaque, selector, data, error)
    a2c_ctx            app_ctx;
    io_ctx             outstream;
    a2c_module         module_opaque;
    UINT4              selector;
    VOID              *data;
    char               error[ERROR_MESSAGE_LENGTH];
{
    int                       status  = NO_ERROR;
    struct a2c_module_       *module  = (struct a2c_module_*)module_opaque;
    UINT4                     index   = 0;
    struct asn_encode_ctx_    ctx__, *ctx = &ctx__;


    modify_a2c_ctx(app_ctx, A2C_CTX_MODIFY_INDEFINITE,1);

    status = check_parameters(CHECK_ENCODE_PARAMETERS,
                            outstream, module, selector, &index, data, error);
    if (status != NO_ERROR) ERETURN(A2C_ERR_BAD_PARAMETER);

    if (error != 0) error[0] = '\0';

/*****/ 
MEMZERO(ctx__);
ctx->app_ctx = app_ctx;
ctx->data = data;
status = create_pool_ctx(&ctx->tmp_heap);
if (status != NO_ERROR) ERETURN(a2c_extern_error(A2C_ERR_NO_MEMORY));
ctx->STACK[0] = &ctx->wrapper;
/*****/

    status = __ber_encode(outstream, ctx, module, selector, &index, data, error);
    if (status != NO_ERROR) {
        (VOID)AC_delete(&ctx->tmp_heap);
        ERETURN(a2c_extern_error(status));
    }

    if (ctx->stack_index != 0) {
        (VOID)AC_delete(&ctx->tmp_heap);
        ERETURN(A2C_ERR_INTERNAL);
    }

    /* we're done with the object */

    (VOID)AC_delete(&ctx->tmp_heap);
    if (status != NO_ERROR) ERETURN(a2c_extern_error(status));

    return NO_ERROR;
}



/*This ANY writes the entire TLV to outstream, whereas normal
  encode_ANY drops the TL on the list an writes the V to outstream
  (which, of course, would be buffered)*/
int
ber_encode_ANY(outstream, ctx, module, selector, index, data, error)
    io_ctx                    outstream;
    struct asn_encode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
    VOID                     *data;
    char                      error[ERROR_MESSAGE_LENGTH];
{
    int                       status     = NO_ERROR;
    tag_type                  tag        = 0;
    length_type               length     = 0;
    int                       tl_length  = 0;
    io_ctx                    tmp        = 0;
    struct asn__any_         *any        = 0;
    int                       nread      = 0;
    UCHAR                     buffer[16];
UNUSED_ARG(module);
UNUSED_ARG(selector);
UNUSED_ARG(index);
UNUSED_ARG(error);


    any = (struct asn__any_*)data;

    if (any->object == 0) ERETURN(A2C_ERR_VALUE_MISSING);

    status = create_bound_static_memio_ctx(&tmp, any->object, any->length);
    if (status != NO_ERROR) ERETURN(status);

    nread = 0;
    status = get_tag(tmp, buffer, &nread, sizeof(buffer), &tag);
    if (status != NO_ERROR) ERETURN(status);
    tl_length = nread;

    nread = 0;
    status = get_length(tmp, buffer, &nread, sizeof(buffer), &length);
    if (status != NO_ERROR) ERETURN(status);
    tl_length += nread;

    (VOID)IO_delete(&tmp);


    if (ctx->list_tail->tag == (CLASS_SPECIAL|TAG_ANY)) {
        ctx->list_tail->tag = tag;
        ctx->list_tail->length = length;
    }
    else {
        if (IS_CONSTRUCTED(tag)) {
            ctx->list_tail->tag = SET_CONSTRUCTED(ctx->list_tail->tag);
        } 
    }

    /*Output the tag and length*/
    status=put_tag(outstream, ctx->list_tail->tag);
    if(status != NO_ERROR) ERETURN(status);

    status=put_length(outstream, length);
    if(status != NO_ERROR) ERETURN(status);
    
    status = IO_fwrite(((char*)(any->object))+tl_length, 1,
                         any->length-tl_length, outstream);
    if (status != (int)any->length-tl_length) {
        ERETURN(A2C_ERR_OUTSTREAM_LEN);
    }

    return NO_ERROR;
}

