/*
 *
 *    runtime.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/runtime.c,v 
 *    $Revision: 1.10 
 *    $Date: 1997/06/25 03:02:00 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */



#include "asn.h"
#include "asn_decode_ctx.h"
#include "asn_error.h"
#include "a2c_dot_c_file.h"
#include "a2c.h"
#include "params.h"
#include "runtime.h"
#include "a2c_vtbl.h"


#ifndef LINT
static char rcsid[] = "$Id: runtime.c,v 1.10 1997/06/25 03:02:00 briank Exp ";
#endif



      




PRIVATE length_type
get_current_length(ctx)
    struct asn_decode_ctx_   *ctx;
{
    length_type               length    = 0;

    length = ctx->STACK[ctx->stack_index]->length;
    return length;
}

PRIVATE int
set_current_length(ctx, length)
    struct asn_decode_ctx_   *ctx;
    length_type               length;
{
    ctx->STACK[ctx->stack_index]->length = length;
    return NO_ERROR;
}

PRIVATE tag_type
get_current_tag(ctx)
    struct asn_decode_ctx_   *ctx;
{
    tag_type                  tag       = 0;

    tag = ctx->STACK[ctx->stack_index]->tag;
    return tag;
}

PRIVATE int
set_current_tag(ctx, tag)
    struct asn_decode_ctx_   *ctx;
    tag_type                  tag;
{
    ctx->STACK[ctx->stack_index]->tag = tag;
    return NO_ERROR;
}

PRIVATE long
get_current_offset(ctx)
    struct asn_decode_ctx_   *ctx;
{
    long                      offset       = 0;

    offset = ctx->STACK[ctx->stack_index]->offset;
    return offset;
}

PRIVATE int
set_current_offset(ctx, offset)
    struct asn_decode_ctx_   *ctx;
    long                      offset;
{
    ctx->STACK[ctx->stack_index]->offset = offset;
    return NO_ERROR;
}





/* "fast-forward" to end of this element--the end, not past! */
PRIVATE int
fast_forward(module, selector, index_p)
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index_p;
{
    int                    index      = 0;
    UINT4                  me         = 0;
    int                    indent     = 0;
    int                    tmpl_size  = 0;
    struct asn_template_  *current    = 0;

    index = *index_p;

    tmpl_size = module->types[selector].size;
    current = &(module->types[selector].template[index]);
    indent = current->indent;
    me = current->me;

    /* start looking at the next template entry, since it's impossible
     * to tell if the current entry is the end of the object */
    ++index;
    for (; index < tmpl_size; ++index) {
        current = &(module->types[selector].template[index]);
        /* see if this is the next object */
        if (current->indent < indent) break;  /* BREAK */
        else if (current->indent == indent
              && current->me != me) break;    /* BREAK */
    }
    /* the next object was found, so we back up 1 template entry to
     * get to the end of the desired object */
    --index;

    *index_p = index;

    return NO_ERROR;
}


PRIVATE int
check_tag(ctx, module, selector, index)
    struct asn_decode_ctx_   *ctx;
    struct a2c_module_       *module;
    UINT4                     selector;
    UINT4                    *index;
{
    struct asn_template_     *current      = 0;
    tag_type                  desired_tag  = 0;
    tag_type                  decoded_tag  = 0;

    current = &(module->types[selector].template[*index]);

    /* verify that the tag is the appropriate one */
    decoded_tag = get_current_tag(ctx);

    desired_tag = current->tag_class | current->tag_number;

    /* ANY can decode anything except EOC */
    if (decoded_tag == TAG_EOC && desired_tag == (CLASS_SPECIAL | TAG_ANY)) {
        return A2C_WARN_WRONG_TAG;
    }

    if (! ctx->skip_next_tag) {
        if (desired_tag != (CLASS_SPECIAL | TAG_ANY)) {
            if (CLASS(decoded_tag) != CLASS(desired_tag)
             || TAG_NUMBER(decoded_tag) != TAG_NUMBER(desired_tag))
            {
                ctx->skip_next_tag = 0;
                return A2C_WARN_WRONG_TAG;
            }
        }
    }

    ctx->have_tag = 0;
    ctx->have_length = 0;
    ctx->insert_def_length_eoc = 0;

    switch (current->tag_tagging) {
    case TAGGING_BASE:
    case TAGGING_EXPLICIT:
        ctx->skip_next_tag = 0;
        break;

    case TAGGING_IMPLICIT:
        ctx->skip_next_tag = 1;
        break;

    default:
        ASSERT(!"bad tag_tagging");
    }


    return NO_ERROR;
}




