/*
 *
 *    using_any.c
 *
 *    $Source: /usr/local/CVS/a2c/runtime/src/using_any.c,v 
 *    $Revision: 1.11 
 *    $Date: 1997/06/06 00:31:20 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */


#include <io_ctx_memio.h>

#include "a2c.h"
#include "asn_error.h"
#include "asn_types.h"


#ifndef LINT
static char rcsid[] = "$Id: using_any.c,v 1.11 1997/06/06 00:31:20 briank Exp ";
#endif


/* use asn__any instead of io_ctx */
PRIVATE int
decode_using_any(app_ctx, instream, module, selector, heap, data, error)
    a2c_ctx            app_ctx;                       /* IN */
    asn__any          *instream;                      /* IN */
    a2c_module         module;                        /* IN */
    UINT4              selector;                      /* IN */
    alloc_ctx          heap;                          /* IN */   /* may be 0 */
    VOID              *data;                          /* OUT */
    char               error[ERROR_MESSAGE_LENGTH];   /* OUT */  /* may be 0 */
{
    io_ctx     tmpstream  = 0;
    int        status     = NO_ERROR;
    long       pos        = 0;

    status = create_bound_static_memio_ctx
                 (&tmpstream, instream->object, instream->length);
    if (status != NO_ERROR) ERETURN(status);

    status = decode(app_ctx, tmpstream, module, selector, heap, data, error);

    /*  delete the temporary stream, saving the current stream 
     *  position for later */
    pos = IO_ftell(tmpstream);
    (VOID)IO_delete(&tmpstream);

    /* did decode return an error? */
    if (status != NO_ERROR) ERETURN(status);

    /*  make sure we encoded the entire object contained in
     *  the asn__any  */
    if ((unsigned long)pos != instream->length) ERETURN(A2C_ERR_IO);

    return NO_ERROR;
}



/* use asn__any instead of io_ctx */
PRIVATE int
encode_using_any(app_ctx, outstream, module, selector, heap, data, error)
    a2c_ctx            app_ctx;                       /* IN */
    asn__any          *outstream;                     /* OUT */
    a2c_module         module;                        /* IN */
    UINT4              selector;                      /* IN */
    alloc_ctx          heap;                          /* IN */
    VOID              *data;                          /* IN */
    char               error[ERROR_MESSAGE_LENGTH];   /* OUT */  /* may be 0 */
{
    io_ctx    tmpstream  = 0;
    int       status     = NO_ERROR;
    UCHAR    *buffer     = 0;
    int       len;
    int       nread;


    /* note, memio_ctx supports IO_fseek() and IO_ftell(), which will
     * be used below */
    status = create_memio_ctx(&tmpstream, 0);
    if (status != 0) ERETURN(A2C_ERR_NO_MEMORY);

    status = encode(app_ctx, tmpstream, module, selector, data, error);
    if (status != NO_ERROR) {
        (VOID)IO_delete(&tmpstream);
        ERETURN(status);
    }

    /* find out how much data encode() wrote to tmpstream */ 
    if (IO_fseek(tmpstream, 0, FSEEK_EOF) == -1) ERETURN(A2C_ERR_INTERNAL);
    len = IO_ftell(tmpstream);
    if (len == -1) ERETURN(A2C_ERR_INTERNAL);
    if (IO_fseek(tmpstream, 0, FSEEK_BOF) == -1) ERETURN(A2C_ERR_INTERNAL);

    /* make a copy of tmpstream */
    buffer = (UCHAR*)AC_malloc(heap, len);
    nread = IO_fread((char*)buffer, 1, len, tmpstream);
    if (nread != len) ERETURN(A2C_ERR_INTERNAL);
    (VOID)IO_delete(&tmpstream);

    outstream->length = len;
    outstream->object = buffer;

    return NO_ERROR;
}


