/*
 *
 *    alloc_pool_ctx.c
 *
 *    $Source: /usr/local/CVS/alloc_ctx/src/alloc_ctx_pool.c,v 
 *    $Revision: 1.14 
 *    $Date: 1997/06/19 20:49:35 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */




#include "alloc_ctx.h"
#include "alloc_ctx_error.h"
#include "alloc_ctx_pool.h"
#include "std_alloc.h"


#ifndef LINT
static char rcsid[] = "$Id: alloc_ctx_pool.c,v 1.14 1997/06/19 20:49:35 briank Exp ";
#endif






#define FAILURE  0
#define SUCCESS  1


#define CELLADDR(memp)     ((struct cell*)(VOID*)(((UCHAR*)(memp))-(int)(&(((struct cell*)0)->memory))))
#define MEMADDR(cell)      ((VOID*)((cell)->memory.l))
#define ALLOC_LIST(pool)   ((pool)->alloc_list)


/*  size contains the _total_ size of the memory block, that is, including
 *  the size of the cell */
struct cell {
    struct cell   *prev;
    struct cell   *next;
    int            size;
    union {
        long    l[1];
        double  d[1];
    } memory;
};


typedef struct {
    struct cell  *alloc_list;
} POOL; 


INLINE static int    insert_cell
  PROTO_LIST((
    POOL         *pool,
    struct cell  *c 
));

INLINE static int    remove_cell
  PROTO_LIST((
    POOL         *pool,
    struct cell  *c 
));

INLINE static int    free_cell
  PROTO_LIST((
    struct cell  *c 
));

static int    delete_pool_ctx
  PROTO_LIST((
    alloc_ctx  *ctx 
));

static VOID*    pool_malloc
  PROTO_LIST((
    VOID           *pool_opaque,
    unsigned int    size 
));

static VOID*    pool_calloc
  PROTO_LIST((
    VOID           *pool_opaque,
    unsigned int    nelem,
    unsigned int    size 
));

static VOID*    pool_realloc
  PROTO_LIST((
    VOID          *pool_opaque,
    VOID          *p,
    unsigned int   size 
));

static int    pool_free
  PROTO_LIST((
    VOID      *pool_opaque,
    VOID      *p 
));


static struct alloc_ctx_vtbl_  pool_ctx_vtbl = {
    pool_calloc, pool_free, pool_malloc, pool_realloc, delete_pool_ctx
};



INLINE static int
insert_cell(pool, c)
    POOL         *pool;
    struct cell  *c;
{
    if (ALLOC_LIST(pool) != 0) {
        c->next = ALLOC_LIST(pool);
        ALLOC_LIST(pool)->prev = c;
    }
    ALLOC_LIST(pool) = c;

    return SUCCESS;
}


INLINE static int
remove_cell(pool, c)
    POOL         *pool;
    struct cell  *c;
{
    if (c->prev == 0) {
        /* cell is on top of the stack */
        ALLOC_LIST(pool) = c->next;
    }
    else {
        c->prev->next = c->next;
    }

    if (c->next != 0) {
        c->next->prev = c->prev;
    }

    c->prev = c->next = 0;

    return SUCCESS;
}


INLINE static int
free_cell(c)
    struct cell  *c;
{
    (VOID)memset((VOID*)c, 0, c->size);
    (VOID)std_free(std_handle, c);
    return SUCCESS;
}



PUBLIC int
create_pool_ctx(ctx)
    alloc_ctx  *ctx;
{
    struct tmp {
        struct alloc_ctx_  ac;
        POOL               c;
    };

    if (ctx == 0) ERETURN(SE_ERR_ILLEGAL_NULL_ARG);

    *ctx = (alloc_ctx)std_calloc(std_handle, 1, sizeof(struct tmp));
    if (*ctx == 0) ERETURN(SE_ERR_NO_MEMORY);

    (*ctx)->vtbl = &pool_ctx_vtbl;
    (*ctx)->handle = &(((struct tmp*)(*ctx))->c);

    return NO_ERROR;
}


/* returns NO_ERROR, SE_ERR_ILLEGAL_NULL_ARG, or SE_ERR_INTERNAL */
static int
delete_pool_ctx(ctx)
    alloc_ctx  *ctx;
{
    struct cell   *c          = 0;
    struct cell   *hold_next  = 0;
    int            status     = SUCCESS;


    if (ctx == 0) ERETURN(SE_ERR_ILLEGAL_NULL_ARG);

    if (*ctx != 0) {

        /* delete everything from alloc_list */
        c = ALLOC_LIST((POOL*)((*ctx)->handle));
        while (c != 0) {
            hold_next = c->next;
            if (free_cell(c) != SUCCESS) status = FAILURE;
            c = hold_next;
        }
    
        /* delete the context */
        MEMZERO(**(struct alloc_ctx_**)ctx);
        (VOID)std_free(std_handle, *ctx);
    }

    *ctx = 0;

    if (status != SUCCESS) ERETURN(SE_ERR_INTERNAL);

    return NO_ERROR;
}



static VOID*
pool_malloc(pool_opaque, size) 
    VOID           *pool_opaque;
    unsigned int    size;
{
    POOL         *pool = (POOL*)pool_opaque;
    struct cell  *c = 0;
    VOID         *p = 0;
    int           len;

    len = size + sizeof(*c);

    c = (struct cell*)std_malloc(std_handle, len);
    if (c == FAILURE) return(FAILURE);

    c->prev = c->next = 0;
    c->size = len;

    p = MEMADDR(c);

    insert_cell(pool, c);

    return p;
} 


static VOID*
pool_calloc(pool_opaque, nelem, size)
    VOID           *pool_opaque;
    unsigned int    nelem;
    unsigned int    size;
{
    POOL           *pool = (POOL*)pool_opaque;
    struct cell    *c = 0;
    VOID           *p = 0;

    int             len;

    len = (size*nelem) + sizeof(*c);

    c = (struct cell*)std_calloc(std_handle, len, 1);
    if (c == FAILURE) return(FAILURE);

    c->size = len;

    p = MEMADDR(c);

    insert_cell(pool, c);

    return p;
}


static VOID*
pool_realloc(pool_opaque, p, size) 
    VOID          *pool_opaque;
    VOID          *p;
    unsigned int   size;
{
    POOL          *pool = (POOL*)pool_opaque;
    struct cell   *c_old = 0;
    struct cell   *c_new = 0;
    unsigned int   old_size;
    int            len;


    c_old = CELLADDR(p);
    if (c_old == 0) return(FAILURE);

    old_size = c_old->size;

    len = size + sizeof(*c_new);


    remove_cell(pool, c_old);

    c_new = (struct cell*)std_realloc(std_handle, c_old, len);
    if (c_new == 0) {
        /* std_realloc() failed, so we insert the cell back again */
        insert_cell(pool, c_old);
        return(FAILURE);
    }

    c_new->size = len;

    if (c_new != c_old) {
        /* if realloc() has given us a new chunk of memory, 
           then (1) re-allocate the old chunk and (2) zero
           and free it */
        c_old = (struct cell*)std_realloc(std_handle, c_old, c_old->size);
        if (c_old != 0) free_cell(c_old);
    }

    p = MEMADDR(c_new);

    insert_cell(pool, c_new);

    return p;
} 

static int
pool_free(pool_opaque, p)
    VOID      *pool_opaque;
    VOID      *p;
{
    POOL         *pool = (POOL*)pool_opaque;
    struct cell  *c = 0;
    int           status;

    c = CELLADDR(p);
    if (c == 0) return FAILURE;

    remove_cell(pool, c);

    status = free_cell(c);
    ASSERT(status == SUCCESS);
    if (status != SUCCESS) return FAILURE;

    return SUCCESS;
} 







