/*
 *
 *    x509.c
 *
 *    $Source: /usr/local/CVS/SETREF_1_0_Release/certs/src/certs.c,v 
 *    $Revision: 1.14 
 *    $Date: 1997/06/25 16:52:11 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    ekr@terisa.com  Mon Dec  2 15:34:19 1996
 *
 */


#ifndef LINT
static char rcsid[] = "$Id: certs.c,v 1.14 1997/06/25 16:52:11 briank Exp ";
#endif

#include <define_bsafe.h>

#include "a2c.h"
#include "define.h"
#include <x509.h>
#include "x509_useful.h"
#include "x509_decode.h"
#include "certs.h"
#include "certs_utils.h"
#include "kdb.h"
#include <keydata.h>
#include "alloc_ctx_pool.h"
#include "setcert.h"
#include "certs_error.h"
#include "crls.h"

typedef struct {
    asn__any                     *name;
    asn__any                     *key;
    asn__any                     *cert;
} root_key;

static root_key               root_list[] =
{
    { &obj_rca_ca_dn_der, &obj_rca_ca_pub_der, &obj_rca_ca_cert_der}
};

static int TRUSTED_ROOT_COUNT = (sizeof(root_list) / sizeof(root_key));

SET_cert_chain_link_verify_callback verify_link_callback_func=0;
VOID				    *verify_link_callback_arg=0;
static int				    cert_state_depth=0;

#define INIT_CERT_STATE verify_link_callback_func=cb_func; \
			verify_link_callback_arg=cb_arg; \
			cert_state_depth=0
			
static B_ALGORITHM_METHOD    *chooser[] =
{
    &AM_RSA_ENCRYPT,
    &AM_RSA_DECRYPT,
    &AM_SHA,
    0,
};


#define ITEM2ANY(_it,_any) (_any).object=(_it).data; (_any).length=(_it).len
#define ITEM2VALUE(_it,_any) (_any).value=(_it).data; (_any).length=(_it).len

static int add_cert_and_key PROTO_LIST((certs_list ** list,
                     asn__any ** keyp, asn__any * cert, asn__any * key));


int 
check_x509_signed(dbCtx, signedCert, _key)
    DBASE_CTX                    *dbCtx;
    x509_useful__SIGNED          *signedCert;
    asn__any                     *_key;
{
    B_ALGORITHM_OBJ               alg = 0;
    B_KEY_OBJ                     key = 0;
    ITEM                          it;
    int                           r,
                                  _status;
UNUSED_ARG(dbCtx);

    if (r = B_CreateAlgorithmObject(&alg))
        ABORT(r);
    if (r = B_CreateKeyObject(&key))
        ABORT(r);

    it.data = _key->object;
    it.len = _key->length;

    if (r = B_SetKeyInfo(key, KI_RSAPublicBER, (POINTER)&it))
        ABORT(r);
    it.data = signedCert->algorithm.object;
    it.len = signedCert->algorithm.length;

    if (r = B_SetAlgorithmInfo(alg, AI_SHA1WithRSAEncryptionBER, (POINTER)&it))
        ABORT(r);

    if (r = B_VerifyInit(alg, key, chooser, 0))
        ABORT(r);
    if (r = B_VerifyUpdate(alg, signedCert->toBeSigned.object,
                           signedCert->toBeSigned.length, 0))
        ABORT(r);

    /* Wacko ASN.1 BIT STRING encoding */
    if (r = B_VerifyFinal(alg, signedCert->signature.value + 1,
                          signedCert->signature.length - 1, 0, 0))
        ABORT(r);

    _status = 0;

  abort:
    B_DestroyAlgorithmObject(&alg);
    B_DestroyKeyObject(&key);
    if (_status) return CERTS_ERR_BAD_CERTIFICATE;
    return (0);
}

int 
construct_cert_chain_from_dn(dbCtx, dn, use, cb_func, cb_arg, lp, key)
    DBASE_CTX                    *dbCtx;
    asn__any                     *dn;
    int                           use;
    SET_cert_chain_link_verify_callback cb_func;
    VOID				*cb_arg;
    certs_list                  **lp;
    asn__any                    **key;
{
    INIT_CERT_STATE;
    
    return construct_cert_chain_from_dn_and_key(dbCtx, dn, 0, use, lp, key);
}

int 
construct_cert_chain_from_issuerserial(dbCtx, issuer, serial,
                                       use, cb_func, cb_arg, lp, keyp)
    DBASE_CTX                    *dbCtx;
    asn__any                     *issuer,
                                 *serial;
    int                           use;
    SET_cert_chain_link_verify_callback cb_func;
    VOID				*cb_arg;
    certs_list                  **lp;
    asn__any                    **keyp;
{
    int                           r,
                                  _status;
    kdb_objs                     *cert_list = 0;

    INIT_CERT_STATE;
    
    if (r = NDBM_SelectCertByIssuerSerial(&cert_list,
          issuer->object, issuer->length, serial->object, serial->length,
                                          dbCtx->handle))
        ABORT(CERTS_ERR_NO_CERTIFICATE);

    if (! cert_list)
        ABORT(CERTS_ERR_NO_CERTIFICATE);

    if (SET_set_CertIsBad(cert_list->data.object,
                          cert_list->data.length, use))
        ABORT(CERTS_ERR_NO_CERTIFICATE);

    if (r = construct_cert_chain_from_cert(dbCtx,
                                           &cert_list->data, 0, use, lp, keyp))
        ABORT(r);

    _status = 0;

  abort:
    free_kdb_objs(&cert_list);
    return (_status);
}



/*Internal functions*/
int
construct_cert_chain_from_cert(dbCtx, cert, flags, use, list, akey)
    DBASE_CTX                    *dbCtx;
    asn__any                     *cert;
    UINT4                         flags;
    int				  use;
    certs_list                  **list;
    asn__any                    **akey;
{
    int                                _status;
    int                                r;
    x509_useful__SIGNED                signedCert;
    x509_useful__UnsignedCertificate   unsignedCert;
    asn__any                          *key = 0;
    alloc_ctx                          heap = 0;
UNUSED_ARG(flags);

    *list = 0;
    MEMZERO(signedCert);
    MEMZERO(unsignedCert);
    create_pool_ctx(&heap);

    if (r = decode_x509_cert(dbCtx, cert, heap, &signedCert, &unsignedCert))
        ABORT(r);

    cert_state_depth++;
    
    r = construct_cert_chain_from_dn_and_key(dbCtx,
      &unsignedCert.issuer, 0, CERT_USE_CERT, list, &key);

    cert_state_depth--;
    
    if(r) ABORT(r);

    if (r = check_x509_signed(dbCtx, &signedCert, key))
        ABORT(r);

    if(r = check_cert_crl_status(dbCtx, &unsignedCert, heap)){
	if(r != CERTS_ERR_REVOKED)
	    r=CERTS_ERR_NO_CRL;
	ABORT(r);
    }
      
    if(verify_link_callback_func){
      if(r=verify_link_callback_func(verify_link_callback_arg,
	use, cert, &((*list)->certificate)))
	ABORT(r);
    }

    add_cert_and_key(list, akey, cert, &unsignedCert.subjectPublicKeyInfo);

    _status = 0;
  abort:
    free_asn_any(&key);
    if (heap) (VOID)AC_delete(&heap);
    return (_status);

}

int 
construct_cert_chain_from_dn_and_key(dbCtx, dn, pubk, use, lp, keyp)
    DBASE_CTX                    *dbCtx;
    asn__any                     *dn;
    asn__any                     *pubk;
    int                           use;
    certs_list                  **lp;
    asn__any                    **keyp;
{
    int                           r,
                                  _status;
    kdb_objs                     *cert_list = 0;
    int                           root;

    for (root = 0; root < TRUSTED_ROOT_COUNT; root++) {
        if (root_list[root].name->length == dn->length &&
            !memcmp(root_list[root].name->object, dn->object,
                    dn->length))
        {
            if (r = add_cert_and_key(lp, keyp,
                          root_list[root].cert, root_list[root].key))
                ABORT(r);
            return (0);
        }
    }

    /* Ok, it's not a root. Get the certificate */
    if (r = NDBM_SelectCertsBySubject(&cert_list, dn->object,
                                      dn->length, dbCtx->handle))
        ABORT(CERTS_ERR_NO_CERTIFICATE);

    if (! cert_list)
        ABORT(CERTS_ERR_NO_CERTIFICATE);

    if (r = construct_cert_chain_from_certlist(dbCtx, dn, pubk, use, cert_list,
                                               lp, keyp))
        ABORT(r);

    _status = 0;

  abort:
    free_kdb_objs(&cert_list);
    return (_status);
}

int 
construct_cert_chain_from_certlist(dbCtx, dn, pubk, use, cert_list, lp, keyp)
    DBASE_CTX                    *dbCtx;
    asn__any                     *dn;
    asn__any                     *pubk;
    int                           use;
    kdb_objs                     *cert_list;
    certs_list                  **lp;
    asn__any                    **keyp;
{
    alloc_ctx                          heap = 0;
    kdb_objs                          *tmp;
    x509_useful__UnsignedCertificate   unsignedCert;
    x509_useful__SIGNED                signedCert;
    int                                r;
    int                                _status;

    if (cert_list == 0)
        ABORT(CERTS_ERR_NO_CERTIFICATE);

    create_pool_ctx(&heap);

    for (tmp = cert_list; tmp; tmp = tmp->next) {
        MEMZERO(unsignedCert);
        MEMZERO(signedCert);

        if (r = decode_x509_cert(dbCtx, &tmp->data, heap,
                                 &signedCert, &unsignedCert))
            ABORT(r);

        if (unsignedCert.subject.length != dn->length
          || memcmp(unsignedCert.subject.object, dn->object, dn->length))
            ABORT(CERTS_ERR_INTERNAL);  /* Can't happen */

        /* Trim self-signed */
        if (SAME_ANY(unsignedCert.subject, unsignedCert.issuer))
            continue;

        /* Trim certificates with the wrong key */
        if (pubk) {
            if ((unsignedCert.subjectPublicKeyInfo.length != pubk->length)
                || memcmp(unsignedCert.subjectPublicKeyInfo.object, pubk->object,
                          pubk->length))
                continue;
        }

        if (SET_set_CertIsBad(tmp->data.object, tmp->data.length, use))
            continue;

        /* Since we abort if this fails, that means no rollover. Do we
         * need it? */
        if (r = construct_cert_chain_from_cert(dbCtx,
                                               &tmp->data, 0, use, lp, keyp))
            ABORT(r);

        goto ok;
    }

    ABORT(CERTS_ERR_NO_CERTIFICATE);

  ok:
    _status = 0;
  abort:
    if (heap)
        (VOID)AC_delete(&heap);
    return (_status);
}


static int 
add_cert_and_key(list, keyp, cert, key)
    certs_list                  **list;
    asn__any                    **keyp;
    asn__any                     *cert;
    asn__any                     *key;
{
    certs_list                   *certp = 0;
    int                           r,
                                  _status;

    if (keyp) *keyp = 0;

    if (! (certp = (certs_list *) calloc(sizeof(certs_list), 1)))
        ABORT(CERTS_ERR_NO_MEMORY);
    if (r = copy_asn_any(&certp->certificate, cert))
        ABORT(r);
    if (keyp) {
        if (! (*keyp = (asn__any *) malloc(sizeof(asn__any))))
            ABORT(CERTS_ERR_NO_MEMORY);
        if (r = copy_asn_any(*keyp, key))
            ABORT(r);
    }
    certp->next = *list;
    *list = certp;

    _status = 0;

  abort:
    if (_status) {
        if (keyp && *keyp) {
            free((*keyp)->object);
            free(*keyp);
        }
    }
    return (_status);
}

int 
insert_x509_cert(dbCtx, cert)
    DBASE_CTX                    *dbCtx;
    asn__any                     *cert;
{
    x509_useful__SIGNED           signedCert;
    x509_useful__UnsignedCertificate unsignedCert;
    alloc_ctx                     heap;
    int                           _status,
                                  r;
    kdb_objs                      certs;

    certs.next = 0;
    certs.data.object = cert->object;
    certs.data.length = cert->length;
    create_pool_ctx(&heap);

    if (r = decode_x509_cert(0, cert, heap, &signedCert, &unsignedCert))
        ABORT(r);

    if (r = NDBM_InsertCert(unsignedCert.issuer.object,
             unsignedCert.issuer.length, unsignedCert.serialNumber.value,
           unsignedCert.serialNumber.length, unsignedCert.subject.object,
                     unsignedCert.subject.length, &certs, dbCtx->handle))
        ABORT(r);

    _status = 0;

  abort:
    (VOID)AC_delete(&heap);
    return (_status);
}

    

