/*
 *
 *    certs.h
 *
 *    $Source: /usr/local/CVS/SETREF_1_0_Release/certs/src/certs.h,v 
 *    $Revision: 1.10 
 *    $Date: 1997/07/01 01:17:27 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    ekr@terisa.com  Tue Feb  4 10:37:57 1997
 *
 */


#ifndef _certs_h
#define _certs_h

#include "define.h"
#include "a2c.h"
#include "kdb.h"
#include "x509_useful.h"

#ifdef __cplusplus
extern "C" {
#endif

#define CERTS_MODULE_NO_                10
#define CERTS_ERR_NO_CERTIFICATE        ERROR_MAKE(CERTS_MODULE_NO_,1)
#define CERTS_ERR_BAD_CERTIFICATE       ERROR_MAKE(CERTS_MODULE_NO_,2)
#define CERTS_ERR_INTERNAL              ERROR_MAKE(CERTS_MODULE_NO_,3)
#define CERTS_ERR_NO_MEMORY             ERROR_MAKE(CERTS_MODULE_NO_,4)
#define CERTS_ERR_NO_CRL		ERROR_MAKE(CERTS_MODULE_NO_,5)
#define CERTS_ERR_REVOKED		ERROR_MAKE(CERTS_MODULE_NO_,6)
#define SE_NO_CERTIFICATE               CERTS_ERR_NO_CERTIFICATE

#define CERT_USE_ANY            (1<<(7-1))
#define CERT_USE_SIGN		(1<<0)
#define CERT_USE_KEXCH		(1<<1)
#define CERT_USE_CERT           (1<<5)
#define CERT_USE_CRL_SIGN	(1<<6)

/*Note that this is the same as kdb_objs and so can be intermixed */
/*This is also repeated in pkcs7_types.h*/
typedef asn__any              certificate_;
typedef
struct cert_list_ {
    struct cert_list_            *next;
    certificate_                  certificate;
} certs_list;
typedef certs_list         certificate_list;


     
typedef int (*SET_cert_chain_link_verify_callback) PROTO_LIST((
     VOID *callback_arg, int subject_cert_use,
     asn__any *subject_cert, asn__any *issuer_cert));
     
int construct_cert_chain_from_dn
  PROTO_LIST((
    DBASE_CTX              *dbCtx,     /* IN */
    asn__any               *dn,        /* IN */
    int                     use,       /* IN */
    SET_cert_chain_link_verify_callback cb_func,	/*IN*/
    VOID				*cb_arg,	/*IN*/
    certs_list            **lp,        /* OUT */
    asn__any              **key        /* OUT */
  ));

int construct_cert_chain_from_dn_and_key
  PROTO_LIST((
    DBASE_CTX              *dbCtx,     /* IN */
    asn__any               *dn,        /* IN */
    asn__any		    *pubk,     /* IN*/
    int                     use,       /* IN */
    certs_list            **lp,        /* OUT */
    asn__any              **key        /* OUT */
  ));

int construct_cert_chain_from_cert
  PROTO_LIST((
    DBASE_CTX              *dbCtx,     /* IN */
    asn__any               *cert,      /* IN */
    UINT4                   flags,     /* unused */
    int			    use,       /*Use*/
    certs_list            **lp,        /* OUT */
    asn__any              **key        /* OUT */
  ));

int construct_cert_chain_from_certlist
  PROTO_LIST((
    DBASE_CTX              *dbCtx,     /* IN */
    asn__any               *dn,        /* IN */
    asn__any               *pubkey,    /* IN */
    int                     use,       /* IN */
    kdb_objs               *cert_list, /* IN */
    certs_list            **lp,        /* OUT */
    asn__any              **key        /* OUT */
  ));

int construct_cert_chain_from_issuerserial
  PROTO_LIST((
    DBASE_CTX              *dbCtx,     /* IN */
    asn__any               *issuer,    /* IN */
    asn__any               *serial,    /* IN */
    int                     use,       /* IN */
    SET_cert_chain_link_verify_callback cb_func,	/*IN*/
    VOID				*cb_arg,	/*IN*/
    certs_list            **lp,        /* OUT */
    asn__any              **key        /* OUT */
  ));

int insert_x509_cert
  PROTO_LIST((
    DBASE_CTX              *dbCtx,     /* IN */
    asn__any               *cert       /* IN */
  ));

int check_x509_signed PROTO_LIST((
    DBASE_CTX                    *dbCtx,
    x509_useful__SIGNED          *signedCert,
    asn__any                     *_key
));

#ifdef __cplusplus
}
#endif
#endif

