/**
   crls.c

   Copyright (C) 1997, Visa International Service Association and MasterCard International Incorporated
   All Rights Reserved.

 *    $Source:  
 *    $Revision:  
 *    $Date:  
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $

   ekr@terisa.com  Thu Apr  3 14:41:38 1997
 */

#ifndef LINT
static char *RCSSTRING="$Id: crls.c,v 1.3 1997/06/25 16:52:12 briank Exp ";
#endif

#include <aglobal.h>
#include <bsafe.h>
#include "define.h"
#include "a2c.h"
#include <x509.h>
#include "x509_useful.h"
#include "x509_decode.h"
#include "certs.h"
#include "certs_utils.h"
#include "kdb.h"
#include "crls.h"
#include "certs_error.h"
#include "certs_utils.h"

/*Return 0 if OK, otherwise CERTS_ERR_REVOKED or CERTS_ERR_NO_CRL*/
int check_cert_crl_status(dbCtx, unsignedCert, heap)
  DBASE_CTX				*dbCtx;
  x509_useful__UnsignedCertificate	*unsignedCert;
  alloc_ctx				heap;
  {
    x509_useful__SIGNED		signedCRL;
    x509_useful__UnsignedCertificateRevocationList	unsignedCRL;    
    a2c_ctx			a2c=0;
    int				r;
    int				_status;
    kdb_objs			*crl_list=0;
    certs_list			*cert_list=0;
    asn__any			*key=0;
    time_t			t;
    x509__CRLEntries		*entry;
    
    t=time(0);
    
    create_a2c_ctx(&a2c);
    MEMZERO(signedCRL);
    MEMZERO(unsignedCRL);

    if(r = NDBM_SelectCRLByIssuerTime(&crl_list, unsignedCert->
				      issuer.object,
				      unsignedCert->issuer.length,
				      (UINT4)t, dbCtx->handle))
	ABORT(r);

    if(!crl_list)
    {
        delete_a2c_ctx(&a2c);
	return(0);
    }

    if(crl_list->next)
	ABORT(CERTS_ERR_INTERNAL);
    
    if (r = decode_using_any(a2c, &crl_list->data,
			     x509_useful, X509_USEFUL__SIGNED,
                             heap, &signedCRL, 0))
        ABORT(r);

    if (r = decode_using_any(a2c, &(signedCRL.toBeSigned),
			     x509_useful,
			     X509_USEFUL__UNSIGNEDCERTIFICATE,
			     heap, &unsignedCRL, 0))
	ABORT(r);


    if (r = construct_cert_chain_from_dn_and_key(dbCtx, &unsignedCRL.issuer,
						 0, CERT_USE_CRL_SIGN,
						 &cert_list, &key))
	ABORT(r);

    if(r=check_x509_signed(dbCtx, &signedCRL, key))
	ABORT(r);


    for(entry=unsignedCRL.revokedCertificates;entry;entry=entry->next){
	if(SAME_OCTET_STRING(entry->cRLEntry.userCertificate,
			     unsignedCert->serialNumber)){
	    ABORT(CERTS_ERR_REVOKED);
	}
    }

    _status=0;
    
  abort:
    delete_a2c_ctx(&a2c);
    free_kdb_objs(&crl_list);
    free_cert_list(&cert_list);
    free_asn_any(&key);
    return(_status);
  }
    
    

