/*
 *
 *    setcert.c
 * 
 *    $Source: /usr/local/CVS/SETREF_1_0_Release/certs/src/setcert.c,v 
 *    $Revision: 1.6 
 *    $Date: 1997/06/06 00:35:20 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 * 
 *    ekr@terisa.com  Fri Dec 13 18:11:45 1996
 *
 */


#ifndef LINT
static char rcsid[] = "$Id: setcert.c,v 1.6 1997/06/06 00:35:20 briank Exp ";
#endif


#include "define.h"
#include "a2c.h"
#include <x509.h>
#include <x509_useful.h>
#include "certs.h"
#include "certs_error.h"

#include <alloc_ctx_pool.h>

#include "setcert.h"

static int SET_get_extension_value_using_cert
  PROTO_LIST((
    alloc_ctx                     heap,       /* IN */
    a2c_ctx                       codec,      /* IN */
    asn__any                     *enc_cert,   /* IN */
    asn__object_identifier       *extnID,     /* IN */
    asn__any                     *value       /* OUT */
));

int 
SET_set_CertIsBad(data, len, use)
    unsigned char                *data;
    unsigned int                  len;
    int                           use;
{
    a2c_ctx                       codec = 0;
    alloc_ctx                     heap = 0;
    asn__any                      cert;
    asn__any                      value;
    x509__KeyUsage                ku;
    x509__Certificate             c;
    int                           status;

    (VOID) create_a2c_ctx(&codec);
    (VOID) create_pool_ctx(&heap);

    cert.length = len;
    cert.object = data;

    status = decode_using_any(codec, &cert, x509, X509__CERTIFICATE,
                              heap, &c, 0);
    if (status != NO_ERROR)
        goto isbad;

    status = SET_get_extension_value_using_cert(heap, codec, &cert,
                                          &x509__id_ce_keyUsage, &value);
    if (status != NO_ERROR)
        goto isbad;

    status = decode_using_any(codec, &value, x509, X509__KEYUSAGE, heap,
                              ku, 0);
    if (status != NO_ERROR)
        goto isbad;

    switch (use) {
    case CERT_USE_SIGN:
        if (! BIT_IS_SET(ku, x509__KeyUsage__digitalSignature))
            goto isbad;
        break;
    case CERT_USE_KEXCH:
        if (! BIT_IS_SET(ku, x509__KeyUsage__keyEncipherment))
            goto isbad;
        break;
    case CERT_USE_CERT:
        if (! BIT_IS_SET(ku, x509__KeyUsage__keyCertSign))
            goto isbad;
        break;
    case CERT_USE_ANY:
        break;
    default:
        goto isbad;
        break;
    }


    status = 0;
    goto done;

  isbad:
    status = 1;

  done:

    (VOID)AC_delete(&heap);
    delete_a2c_ctx(&codec);
    return status;
}


static int 
SET_get_extension_value_using_cert(heap, codec, enc_cert, extnID, value)
    alloc_ctx                     heap;               /* IN */
    a2c_ctx                       codec;              /* IN */
    asn__any                     *enc_cert;           /* IN */
    asn__object_identifier       *extnID;             /* IN */
    asn__any                     *value;              /* OUT */
{
    int                           status = NO_ERROR;
    x509__Certificate             cert;
    x509__Extensions             *ep = 0;

    MEMZERO(cert);

    status = decode_using_any(codec, enc_cert, x509, X509__CERTIFICATE, heap, &cert, 0);
    if (status != NO_ERROR)
        ERETURN(CERTS_ERR_INTERNAL);
    for (ep = cert.toBeSigned.extensions; ep != 0; ep = ep->next) {
        if (SAME_OBJECT_IDENTIFIER(ep->extension.extnID, *extnID))
            break;
    }

    /* only grab the 1st one */
    if (ep != 0) {
        value->length = ep->extension.extnValue.length;
        value->object = AC_malloc(heap, value->length);
        if (value->object == 0)
            ERETURN(CERTS_ERR_NO_MEMORY);
        memcpy(value->object, ep->extension.extnValue.value, value->length);
    }
    else {
        ERETURN(-1);            /* didn't find it */
    }

    return NO_ERROR;
}

