/*
 *
 *    x509_decode.c
 * 
 *    $Source: /usr/local/CVS/SETREF_1_0_Release/certs/src/x509_decode.c,v 
 *    $Revision: 1.5 
 *    $Date: 1997/06/06 00:35:24 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 * 
 *    ekr@terisa.com  Mon Feb  3 16:54:43 1997
 * 
 */


#ifndef LINT
static char rcsid[] = "$Id: x509_decode.c,v 1.5 1997/06/06 00:35:24 briank Exp ";
#endif

#include <define.h>
#include <a2c.h>
#include <x509_useful.h>
#include "x509_decode.h"
#include "certs_error.h"

int 
decode_x509_cert(cryptCtx, cert, heap, signedCert, unsignedCert)
/*Decode a signed certificate and (optionally) the inner signed data */
    void                         *cryptCtx;
    asn__any                     *cert;
    alloc_ctx                     heap;
    x509_useful__SIGNED          *signedCert;
    x509_useful__UnsignedCertificate *unsignedCert;
{
    char                          error[ERROR_MESSAGE_LENGTH];
    int                           _status,
                                  r;
    a2c_ctx                       a2c = 0;

    create_a2c_ctx(&a2c);

    if (r = modify_a2c_ctx(a2c, A2C_CTX_MODIFY_OPAQUE_CONTEXT, (void *)cryptCtx))
        ABORT(r);

    memset(signedCert, 0, sizeof(x509_useful__SIGNED));
    if (unsignedCert)
        memset(unsignedCert, 0, sizeof(x509_useful__UnsignedCertificate));

    if (r = decode_using_any(a2c, cert, x509_useful, X509_USEFUL__SIGNED,
                             heap, signedCert, error))
        ABORT(r);

    if (unsignedCert) {
        if (r = decode_using_any(a2c, &(signedCert->toBeSigned),
                                 x509_useful,
            X509_USEFUL__UNSIGNEDCERTIFICATE, heap, unsignedCert, error))
            ABORT(r);
    }

    _status = 0;

  abort:
    delete_a2c_ctx(&a2c);
    return (_status);
}

