/* 
 *
 *    default_error_ctx.c
 *
 *    $Source: /usr/local/CVS/error_ctx/src/default_error_ctx.c,v 
 *    $Revision: 1.6 
 *    $Date: 1997/07/01 00:37:49 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */


#ifdef WIN32
#include <io.h>
#endif

#include "error_ctx.h"
#include "error_module.h"


#ifndef LINT
static char rcsid[] = "$Id: default_error_ctx.c,v 1.6 1997/07/01 00:37:49 sayuri Exp ";
#endif


static error_ctx  DEFAULT__ = 0;

#if WIN32
static int    win32_write_error 
  PROTO_LIST((
    VOID     *write_handle,
    char     *buf,
    int       nbyte
  ));
static int 
win32_write_error(write_handle, buf, nbyte)
    VOID     *write_handle;
    char     *buf;
    int       nbyte;
{
    return write((int)write_handle, buf, nbyte);
}
#else 
#include <unistd.h>
static int    unix_write_error 
  PROTO_LIST((
    VOID     *write_handle,
    char     *buf,
    int       nbyte
  ));
static int 
unix_write_error(write_handle, buf, nbyte)
    VOID     *write_handle;
    char     *buf;
    int       nbyte;
{
    return write((int)write_handle, buf, nbyte);
}
#endif

PUBLIC error_ctx
get_default_error_ctx()
{
    VOID        *write_handle;

    if (DEFAULT__ == 0) {
        /* this function call had better work, or we're in big trouble */
#if WIN32
        write_handle = (VOID*)_fileno(stderr);
        (VOID)create_error_ctx(&DEFAULT__, win32_write_error, write_handle);
#else 
        write_handle = (VOID*)STDERR_FILENO;  /* normally, STDERR_FILENO=2 */
        (VOID)create_error_ctx(&DEFAULT__, unix_write_error, write_handle);
#endif
    }
    
    return DEFAULT__;
}

/*  As it is currently implemented, providing a 0 <ctx>
 *  parameter causes the toolkit to automatically generate
 *  another default <DEFAULT_ERROR_CTX>, because the next
 *  time get_default_error_ctx() is called, it notices that
 *  DEFAULT__==0, in which case it creates another <error_ctx>. */

PUBLIC int
set_default_error_ctx(ctx)
    error_ctx   ctx;
{
    DEFAULT__ = ctx;
    return NO_ERROR;
}



