/*
 *
 *    error_ctx.c
 *
 *    $Source: /usr/local/CVS/error_ctx/src/error_ctx.c,v 
 *    $Revision: 1.13 
 *    $Date: 1997/06/25 04:05:42 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */



#include "error_ctx.h"
#include "error_ctx_def.h"
#include "error_module.h"



#ifndef LINT
static char rcsid[] = "$Id: error_ctx.c,v 1.13 1997/06/25 04:05:42 briank Exp ";
#endif




#define error_get_module(err)  ((ERROR_MODULE(err) < E_MAX_MODULE)\
                                 ?error_ctx_module_list__[ERROR_MODULE(err)]:0)


/* global error logging enabled/disabled */
#ifdef LOG_ERRORS_BY_DEFAULT
static int logging_disabled = 0; /* by default disabled */
#else
static int logging_disabled = 1; /* by default NOT disabled (that is, enabled)*/
#endif


static int    error_logging_is_disabled
  PROTO_LIST((
    struct error_ctx_     *ctx                  /* IN */
  ));

static int
error_logging_is_disabled(ctx)
    struct error_ctx_     *ctx;                 /* IN */
{
    if (ctx == 0) return 0;
    if (ctx->flags & ERROR_CTX_FLAG_ALL_LOGGING_DISABLED
     || ctx->write == 0
     || logging_disabled == 1)
    {
        return 1;
    }
    return 0;
}



PUBLIC int
error_explain_error(ctx, err, str)
    error_ctx     ctx;                          /* IN */
    int           err;                          /* IN */
    char          str[ERROR_MESSAGE_LENGTH];    /* IN */
{
    error_module    module;

    if (str != 0) {
        str[0] = '\0';
        module = error_get_module(err);
        if (module)
            (VOID)module->vtbl->explain_error(module, ctx, err, str);
        else
            (VOID)ERROR_MODULE_VTBL_DEFAULT->explain_error(0, ctx, err, str);
    }

    return NO_ERROR;
}


PUBLIC int
error_log_error(ctx_opaque, err)
    error_ctx     ctx_opaque;                   /* IN */
    int           err;                          /* IN */
{
    error_module           module;
    struct error_ctx_     *ctx = (struct error_ctx_*)ctx_opaque;

    if (ctx == 0 || error_logging_is_disabled(ctx)) return NO_ERROR;

    module = error_get_module(err);
    if (module)
        (VOID)module->vtbl->log_error(module, ctx, err);
    else
        (VOID)ERROR_MODULE_VTBL_DEFAULT->log_error(0, ctx, err);

    return NO_ERROR;
}


PUBLIC int
#ifdef USE_ANSI_STDARG
error_log_printf(error_ctx ctx_opaque, int module_num, char *format, ...)
#else
error_log_printf(ctx_opaque, module_num, format, va_alist)
    error_ctx     ctx_opaque;                   /* IN */
    int           module_num;                   /* IN */
    char         *format;                       /* IN */
    va_dcl
#endif
{
    va_list                ap;
    struct error_ctx_     *ctx = (struct error_ctx_*)ctx_opaque;
    error_module           module;
    int                    dummy_error = ERROR_MAKE(module_num, 0);

    if (ctx == 0 || error_logging_is_disabled(ctx)) return NO_ERROR;
    if (format == 0) return NO_ERROR;

    VA_START(ap, format);

    module = error_get_module(dummy_error);
    if (module)
        (VOID)module->vtbl->log_printf(module, ctx, module_num, format, &ap);
    else
        (VOID)ERROR_MODULE_VTBL_DEFAULT->log_printf(0, ctx, module_num, format, &ap);

    va_end(ap);

    return NO_ERROR;
}


PUBLIC int
error_reset_error(ctx_opaque, err)
    error_ctx     ctx_opaque;                   /* IN */
    int           err;                          /* IN */
{
    struct error_ctx_     *ctx = (struct error_ctx_*)ctx_opaque;

    if (ctx != 0) {
        MEMZERO(ctx->error_explanation);
        ctx->error_num = err;
    }

    return NO_ERROR;
}




PUBLIC int
error_disable_error_logging()
{
    logging_disabled = 1;
    return NO_ERROR;
}

PUBLIC int
error_enable_error_logging()
{
    logging_disabled = 0;
    return NO_ERROR;
}




PUBLIC int
error_disable_module_error_logging(module_num)
    int            module_num;               /* IN */
{
    if (module_num >= 0
     && module_num < E_MAX_MODULE
     && error_ctx_module_list__[module_num] != 0)
    {
        error_ctx_module_list__[module_num]->logging_disabled = 1;
    }
    return NO_ERROR;
}

PUBLIC int
error_enable_module_error_logging(module_num)
    int            module_num;               /* IN */
{
    if (module_num >= 0
     && module_num < E_MAX_MODULE
     && error_ctx_module_list__[module_num] != 0)
    {
        error_ctx_module_list__[module_num]->logging_disabled = 0;
    }
    return NO_ERROR;
}







PUBLIC int
create_error_ctx(ctxp, write_func, write_handle)
    error_ctx    *ctxp;                         /* IN */
    int         (*write_func) PROTO_LIST((VOID *write_handle, char *buf, int nbyte));
                                                /* IN, may be 0 */
    VOID         *write_handle;                 /* IN */
{
    struct error_ctx_ *ctx = 0;

    if (ctxp == 0) ERETURN(SE_ERR_ILLEGAL_NULL_ARG);

    ctx = (struct error_ctx_ *)CALLOC(1, sizeof(*ctx));
    if (ctx == 0) ERETURN(SE_ERR_NO_MEMORY);
    ctx->write         = write_func;
    ctx->write_handle  = write_handle;
    if (ERROR_CTX_DEFAULT_ALL_LOGGING_DISABLED) {
       ctx->flags |= ERROR_CTX_FLAG_ALL_LOGGING_DISABLED;
    }
    *ctxp = (error_ctx*)ctx;
    return NO_ERROR;
}

PUBLIC int
delete_error_ctx(ctxp)
    error_ctx    *ctxp;                         /* IN */
{
    if (ctxp == 0) ERETURN(SE_ERR_ILLEGAL_NULL_ARG);

    if (*ctxp != 0) {
        MEMZERO(**(struct error_ctx_**)ctxp);
        (VOID)FREE(*ctxp);
    }
    *ctxp = 0;
    return NO_ERROR;

}

int
#ifdef USE_ANSI_STDARG
modify_error_ctx(error_ctx ctx_opaque, int modify, ...)
#else
modify_error_ctx(ctx_opaque, modify, va_alist)
    error_ctx     ctx_opaque;
    int           modify;
    va_dcl
#endif
{
    va_list               ap;
UNUSED_ARG(ctx_opaque);

    VA_START(ap, modify);

    switch (modify) {
    default:
        va_end(ap);
        ERETURN(SE_ERR_ILLEGAL_NULL_ARG);
        break;
    }

    va_end(ap);

    return NO_ERROR;
}


int
#ifdef USE_ANSI_STDARG
query_error_ctx(error_ctx ctx_opaque, int query, ...)
#else
query_error_ctx(ctx_opaque, query, va_alist)
    error_ctx     ctx_opaque;
    int           query;
    va_dcl
#endif
{
    va_list               ap;
UNUSED_ARG(ctx_opaque);

    VA_START(ap, query);

    switch (query) {
    default:
        va_end(ap);
        ERETURN(SE_ERR_ILLEGAL_NULL_ARG);
        break;
    }

    va_end(ap);

    return NO_ERROR;
}



