/* 
 *
 *    error_module.c
 *
 *    $Source: /usr/local/CVS/error_ctx/src/error_module.c,v 
 *    $Revision: 1.12 
 *    $Date: 1997/06/06 00:39:15 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */



#include "error_ctx.h"
#include "error_ctx_def.h"
#include "error_module.h"


#ifndef LINT
static char rcsid[] = "$Id: error_module.c,v 1.12 1997/06/06 00:39:15 briank Exp ";
#endif




PUBLIC_DEF struct error_module_vtbl_   ERROR_MODULE_VTBL_DEFAULT[] = {{
     default_error_explain_error,
     default_error_log_error,
     default_error_log_printf
}};



/*
 *    ERROR MODULE
 */
static error_msg error_messages__[] = {
  { NO_ERROR                       , "NO_ERROR"                       },
  { SE_ERR_NO_MEMORY               , "A memory error occurred, generally because an attempt to allocate more memory has failed"               },
  { SE_ERR_FATAL_IO                , "SE_ERR_FATAL_IO"                },
  { SE_ERR_INTERNAL                , "An internal program inconsistency was encountered, generally because of a programming bug"                },
  { SE_ERR_ILLEGAL_NULL_ARG        , "One of the parameters passed in as an argument is invalid because of a null pointer"        },
  { SE_ERR_ILLEGAL_ARG             , "One of the parameters passed in as an argument is invalid"             },
  { SE_ERR_BAD_BER_DER_ENCODING    , "SE_ERR_BAD_BER_DER_ENCODING"    },
  { SE_WARN_OPERATION_CANCELLED    , "SE_WARN_OPERATION_CANCELLED"    },
  { 0, 0 } };

PUBLIC_DEF struct error_module_ error_module_0_error__[] =
  {{ 0, "error", error_messages__, 0, ERROR_MODULE_VTBL_DEFAULT }};






PUBLIC int
error_register_module(module)
    error_module   module;                   /* IN */
{
    int   module_num = module->module_num;
    if (module_num < E_MAX_MODULE) error_ctx_module_list__[module_num] = module;
    return NO_ERROR;
}




static int    module_error_logging_is_disabled
  PROTO_LIST((
    int           err               /* IN */
  ));

static int
module_error_logging_is_disabled(err)
    int           err;              /* IN */
{
    int           module_num = ERROR_MODULE(err);

    if (module_num < E_MAX_MODULE && error_ctx_module_list__[module_num] != 0)
    {
        return error_ctx_module_list__[module_num]->logging_disabled;
    }

    /*  This point should never be reached because module_num must
     *  be < E_MAX_MODULE and all modules must be defined.  Therefore,
     *  disable logging in this error condition for production versions
     *  of the code, but enable logging (to show the error condition)
     *  while debugging.  */
#ifdef OPTION_TRACE_ERRORS
    return 0;
#else
    return 1;
#endif
}



PUBLIC int
default_error_log_error(module, ctx_opaque, status)
    error_module  module;           /* may be 0 */
    error_ctx     ctx_opaque;
    int           status;
{
    struct error_ctx_  *ctx = (struct error_ctx_*)ctx_opaque;
    int                 len;
    char                buffer[ERROR_MESSAGE_LENGTH];

    /* 2 cases:
     *    1) ctx == 0 || ctx->write == 0
     *       - return
     *    2) ctx != 0
     *       - log to this context
     */

    if (ctx == 0) return NO_ERROR;
    if (ctx->write == 0) return NO_ERROR;
    if (module_error_logging_is_disabled(status)) return NO_ERROR;
    
    if (module != 0) {
        sprintf(buffer, "module \"%s\", error %d; ",
            module->module_name, ERROR_NUMBER(status));
    }
    else {
        sprintf(buffer, "module %d, error %d; ",
            ERROR_MODULE(status), ERROR_NUMBER(status));
    }

    len = strlen(buffer);
    (VOID)(ctx->write)(ctx->write_handle, buffer, len);

    if (module != 0)
        (VOID)module->vtbl->explain_error(module, ctx, status, buffer);
    else
        (VOID)ERROR_MODULE_VTBL_DEFAULT->explain_error(0, ctx, status, buffer);

    len = strlen(buffer);
    if (len < ERROR_MESSAGE_LENGTH) {
        buffer[len] = '\n';
        ++len;
        (VOID)(ctx->write)(ctx->write_handle, buffer, len);
    }
    else {
        (VOID)(ctx->write)(ctx->write_handle, buffer, len);
        (VOID)(ctx->write)(ctx->write_handle, "\n", 1);
    }

    return NO_ERROR;
}


PUBLIC int
default_error_log_printf(module, ctx_opaque, module_num, format, dot_dot_dot)
    error_module  module;           /* may be 0 */
    error_ctx     ctx_opaque;
    int           module_num;
    char         *format;
    VOID         *dot_dot_dot;
{
    struct error_ctx_  *ctx = (struct error_ctx_*)ctx_opaque;
    int                 dummy_error = ERROR_MAKE(module_num, 0);
    char               *buffer = 0;
    int                 len;
    int                 slen;
    va_list            *app = (va_list*)dot_dot_dot;
UNUSED_ARG(module);

    if (ctx == 0 || ctx->write == 0 || format == 0) return NO_ERROR;

    /*  dummy_error is required because the user passing in  
     *  the module number, so a dummy error from that module
     *  is created just for the purposes of testing to see
     *  if logging is disabled in that module */
    if (module_error_logging_is_disabled(dummy_error)) return NO_ERROR;

    /*  guestimate the size of the buffer we're going to need,
     *  trying to obtain a really big one if possible--give up
     *  if we can't get at least strlen(format) worth of
     *  memory by just writing "format" */
    slen = strlen(format);
    len = (16 * 1024) + (10 * slen);
    while (len >= slen) {
        buffer = (char*)MALLOC(len);
        if (buffer != 0) break;
        len -= 1024;
    }

    if (buffer != 0) {
        (VOID)vsprintf(buffer, format, *app);
        (VOID)(ctx->write)(ctx->write_handle, buffer, strlen(buffer));
    }
    else {
        (VOID)(ctx->write)(ctx->write_handle, format, strlen(format));
    }

    SAFE_FREE(buffer);

    return NO_ERROR;
}


PUBLIC int
default_error_explain_error(module, ctx_opaque, err, str)
    error_module  module;                /* may be 0 */
    error_ctx     ctx_opaque;
    int           err;
    char          str[ERROR_MESSAGE_LENGTH];
{
    struct error_ctx_  *ctx             = (struct error_ctx_*)ctx_opaque;
    error_msg          *error_messages  = 0;
    char               *message         = "unknown error";
    int                 i;
    int                 len;
    int                 copylen;
    int                 remaining;


    if (str == 0) return NO_ERROR;

    /* 4 cases (1-2 are handled identically):
     *    1) ctx == 0
     *       - just build the text description based on err
     *    2) ctx != 0 && (err != ctx->error_num)
     *       - just build the text description based on err
     *    3) ctx != 0 && (err == ctx->error_num)
     *       - build text description + ctx->error_explanation
     */

    if (module != 0) {
        error_messages = module->module_messages;

        for (i = 0; i < ERROR_NUMBER(~0) && error_messages[i].msg != 0; ++i) {
            if (err == error_messages[i].err) {
                message = error_messages[i].msg;
                break;       /* BREAK */
            }
        }
    }

    len = 0;
    remaining = ERROR_MESSAGE_LENGTH;
    str[0] = '\0';


    copylen = MIN((int)strlen(message), remaining);
    strncpy(&str[len], message, copylen);
    str[len+copylen] = '\0';

    if (ctx != 0 && (err == ctx->error_num)) {
        if (strlen(ctx->error_explanation) > 0) {
            len = strlen(str);
            remaining = ERROR_MESSAGE_LENGTH-len;
            if (remaining > 0) {
                copylen = MIN(2, remaining);
                strncpy(&str[len], ", ", copylen);
                str[len+copylen] = '\0';
                len = strlen(str);
                remaining = ERROR_MESSAGE_LENGTH-len;
                if (remaining > 0) {
                    copylen = MIN((int)strlen(ctx->error_explanation), remaining);
                    strncpy(&str[len], ctx->error_explanation, copylen);
                    str[len+copylen] = '\0';
                }
            }
        }
    }

    return NO_ERROR;
}




PUBLIC_DEF int  (*tsw_PF_error_log_error)
  PROTO_LIST((
    error_ctx     ctx,                          /* IN, may be 0 */
    int           err                           /* IN */
  )) = error_log_error;

PUBLIC_DEF int  (*tsw_PF_error_log_printf)
  PROTO_LIST((
    error_ctx     ctx,                          /* IN, may be 0 */
    int           err,                          /* IN */
    char         *format,                       /* IN, may be 0 */
    ...
  )) = error_log_printf;


