/* 
 *
 *    error_module.h
 *
 *    $Source: /usr/local/CVS/error_ctx/src/error_module.h,v 
 *    $Revision: 1.15 
 *    $Date: 1997/06/25 04:05:30 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */


#include <define.h>

#include "error_ctx.h"


#ifndef __ERROR_MODULE_H__
#define __ERROR_MODULE_H__


#ifdef __cplusplus
extern "C" {
#endif


typedef struct error_module_       *error_module;

typedef struct error_msg_ {
    int     err;
    char   *msg;
} error_msg;

struct error_module_vtbl_ {
    int  (*explain_error)    PROTO_LIST((
                               error_module  module,
                               error_ctx     ctx,
                               int           err,
                               char          str[ERROR_MESSAGE_LENGTH]
                             ));
    int  (*log_error)        PROTO_LIST((
                               error_module  module,
                               error_ctx     ctx,
                               int           err
                             ));
    int  (*log_printf)       PROTO_LIST((
                               error_module  module,
                               error_ctx     ctx,
                               int           module_num,
                               char         *format,
                               VOID         *dot_dot_dot
                             ));
};

struct error_module_ {
    UCHAR                       module_num;
    char                       *module_name;
    error_msg                  *module_messages;
    int                         logging_disabled;
    struct error_module_vtbl_  *vtbl;
};



/* error_ctx_module_list__ is defined by the toolkit */
PRIVATE error_module                 error_ctx_module_list__[];




PUBLIC   struct error_module_vtbl_    ERROR_MODULE_VTBL_DEFAULT[];

PUBLIC int    error_register_module
  PROTO_LIST((
    error_module  module                        /* IN */
  ));



PUBLIC int    default_error_log_error
  PROTO_LIST((
    error_module  module,
    error_ctx     ctx,
    int           err
  ));

PUBLIC int    default_error_log_printf
  PROTO_LIST((
    error_module  module,
    error_ctx     ctx,
    int           module_num,
    char         *format,
    VOID         *dot_dot_dot
  ));

PUBLIC int    default_error_explain_error
  PROTO_LIST((
    error_module  module,
    error_ctx     ctx,
    int           err,
    char          str[ERROR_MESSAGE_LENGTH]
  ));



/*
 *    ERETURN and ABORT
 */
#ifdef OPTION_TRACE_ERRORS
#    ifdef __GNUC__
#        define ERRSTR__(str)   "%s at %s:%d in %s(); ",(str),__FILE__,__LINE__,__FUNCTION__
#    else
#        define ERRSTR__(str)   "%s at %s:%d; ",(str),__FILE__,__LINE__
#    endif
#    define CALL_PFUNC__(func)  ((func)!=0)&&(func)
#    define DEBUG_OUT(str,err)  ((VOID)(CALL_PFUNC__(tsw_PF_error_log_printf)(\
                                   DEFAULT_ERROR_CTX,ERROR_MODULE((err)),\
                                   ERRSTR__((str))))\
                                 ,\
                                 (VOID)(CALL_PFUNC__(tsw_PF_error_log_error)(\
                                   DEFAULT_ERROR_CTX,(err))))
#else
#    define DEBUG_OUT(str,err)  ((VOID)0)
#endif


#ifndef ERETURN
#    define ERETURN(err)        return(DEBUG_OUT("ERETURN",(err)),(err))
#endif

#ifndef ERETURN_NULL
#    define ERETURN_NULL(err)   return(DEBUG_OUT("ERETURN_NULL",(err)),(VOID*)0)
#endif

#ifndef ABORT
#    define ABORT(err)          if(1){DEBUG_OUT("ABORT",(err));_status=(err);goto abort;}else((VOID)(0))
#endif

#ifndef ABORT_NULL
#    define ABORT_NULL(err)     if(1){DEBUG_OUT("ABORT_NULL",(err));_status=0;goto abort;}else((VOID)(0))
#endif




/*
 *    GLOBAL POINTERS TO ERROR LOGGING FUNCTIONS
 */
PUBLIC int  (*tsw_PF_error_log_error)
  PROTO_LIST((
    error_ctx     ctx,                          /* IN, may be 0 */
    int           err                           /* IN */
  ));

PUBLIC int  (*tsw_PF_error_log_printf)
  PROTO_LIST((
    error_ctx     ctx,                          /* IN, may be 0 */
    int           err,                          /* IN */
    char         *format,                       /* IN, may be 0 */
    ...
  ));



#ifdef __cplusplus
}
#endif
#endif

