/**
  bcdmf.c

  Copyright (C) 1997, Visa International Service Association and MasterCard International Incorporated
  All Rights Reserved.

 *    $Source:  
 *    $Revision:  
 *    $Date:  
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $

  ekr@terisa.com  Fri Mar  7 15:26:07 1997
 */

#ifndef LINT
static char *RCSSTRING="$Id: bcdmf.c,v 1.2 1997/06/06 00:39:30 briank Exp ";
#endif

#include <aglobal.h>
#include <bsafe.h>
#include <error_module.h>

#include "bcdmf.h"

static B_ALGORITHM_METHOD *BSAFE_DES_CHOOSER[]={
    &AM_DES_CBC_ENCRYPT,
    0
};

/*Expansion keys*/
static unsigned char K1[8]={0xc4, 0x08, 0xb0, 0x54, 0x0b, 0xa1, 0xe0, 0xae};
static unsigned char K2[8]={0xef, 0x2c, 0x04, 0x1c, 0xe6, 0x38, 0x2f, 0xe6};

/*Parity bit mask: Zero parity bits*/
static unsigned char M1[8]={0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe};

/*This mask is all the non-ignored bits*/
static unsigned char M2[8]={0x0e, 0xfe, 0x0e, 0xfe, 0x0e, 0xfe, 0x0e, 0xfe};

static unsigned char IV[8]={0,0,0,0,0,0,0,0};

static int BLK8AND PROTO_LIST((unsigned char *a, unsigned char *b));
static int BLK8XOR PROTO_LIST((unsigned char *a, unsigned char *b));

int
CDMFKeyTransform(key,key_out)
    unsigned char *key;
    unsigned char *key_out;
{
    B_ALGORITHM_OBJ desObj=0;
    B_KEY_OBJ keyObj=0;
    unsigned char out[8],dummy[1];
    unsigned int outlen;
    int r,_status;

    if(r=B_CreateAlgorithmObject(&desObj))
	ABORT(r);
    if(r=B_CreateKeyObject(&keyObj))
	ABORT(r);
    
    /*Mask off the parity bits to get nI1*/
    BLK8AND(key,M1);

    if(r=B_SetKeyInfo(keyObj,KI_DES8,(POINTER)K1))
	ABORT(r);
    if(r=B_SetAlgorithmInfo(desObj,AI_DES_CBC_IV8,(POINTER)IV))
	ABORT(r);
    if(r=B_EncryptInit(desObj,keyObj,BSAFE_DES_CHOOSER,0))
	ABORT(r);
    if(r=B_EncryptUpdate(desObj,out,&outlen,8,key,8,0,0))
	ABORT(r);
    if(outlen!=8)
	ABORT(-1);
    if(r=B_EncryptFinal(desObj,dummy,&outlen,0,0,0))
	ABORT(r);
    if(outlen)
	ABORT(-1);

    BLK8XOR(out,key);     /*This result is I2*/

    BLK8AND(out,M2);      /*Mask off the ignored bits to get I3*/

    B_DestroyKeyObject(&keyObj);
    keyObj=0;
    if(r=B_CreateKeyObject(&keyObj))
	ABORT(r);
    if(r=B_SetKeyInfo(keyObj,KI_DES8,(POINTER)K2))
	ABORT(r);
    if(r=B_EncryptInit(desObj,keyObj,BSAFE_DES_CHOOSER,0))
	ABORT(r);
    if(r=B_EncryptUpdate(desObj,key_out,&outlen,8,out,8,0,0))
	ABORT(r);
    if(outlen!=8)
	ABORT(-1);
    if(r=B_EncryptFinal(desObj,dummy,&outlen,0,0,0))
	ABORT(r);
    if(outlen)
	ABORT(-1);

    _status=0;
 abort:
    B_DestroyAlgorithmObject(&desObj);
    B_DestroyKeyObject(&keyObj);
    return(_status);
}

static int
BLK8AND(a,b)
    unsigned char *a;
    unsigned char *b;
{
    int		i;

    for(i=0;i<8;i++){
	a[i]&=b[i];
    }

    return(0);
}
    
static int
BLK8XOR(a,b)
    unsigned char *a;
    unsigned char *b;
{
    int		i;

    for(i=0;i<8;i++){
	a[i]^=b[i];
    }

    return(0);
}
    

