/* 
 *
 *    bencrypt.c -- encrypt/decrypt a block.
 *
 *    $Source: /usr/local/CVS/SETREF_1_0_Release/glue/src/bencrypt.c,v 
 *    $Revision: 1.11 
 *    $Date: 1997/06/06 00:39:33 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *
 *    Copyright (C) 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    ekr@terisa.com  Fri Feb  7 15:14:15 1997
 *
 */


#ifndef LINT
static char rcsid[] = "$Id: bencrypt.c,v 1.11 1997/06/06 00:39:33 briank Exp ";
#endif


#include <define_bsafe.h>


#include <a2c.h>
#include <alloc_ctx.h>
#include <alloc_ctx_pool.h>
#include <asn1set.h>
#include <error_module.h>
#include <x509.h>

#include "bencrypt.h"
#include "bcdmf.h"

#define  oid_desCDMF   asn1set__id_desCDMF
#define  oid_des_cbc   asn1set__id_desCBC

static B_ALGORITHM_METHOD    *MY_ALGORITHM_CHOOSER[] =
{
    &AM_DES_CBC_ENCRYPT,
    &AM_DES_CBC_DECRYPT,
    (B_ALGORITHM_METHOD *) 0
};

static int prepare_key 
  PROTO_LIST((
    unsigned char                *key,
    unsigned int                  klen,
    B_KEY_OBJ                    *keyo,
    int				 is_cdmf
  ));

static int prepare_algorithm
  PROTO_LIST((       
    unsigned char		*ai,
    unsigned int		ailen,
    B_ALGORITHM_OBJ		*algo,
    int				*is_cdmf
  ));

static int
prepare_key(key, klen, keyo, is_cdmf)
    unsigned char                *key;
    unsigned int                  klen;
    B_KEY_OBJ                    *keyo;
    int				  is_cdmf;
{
    ITEM                          ki; 
    int                           r;
    UCHAR			  new_key[8];
    

    if(is_cdmf){
      if(klen!=8)
	return(-1);
      
      CDMFKeyTransform(key,new_key);
      key=new_key;
    }
    
    if (r = B_CreateKeyObject(keyo))
        return (r);
    ki.data = key;
    ki.len = klen;

    if (r = B_SetKeyInfo(*keyo, KI_Item, (POINTER) & ki))
        return (r);

    return(0);
}

int
prepare_algorithm(ai,ailen,algo,is_cdmf)
    unsigned char		*ai;
    unsigned int		ailen;
    B_ALGORITHM_OBJ		*algo;
    int				*is_cdmf;
{
    asn__any			  algid;
    a2c_ctx			  a2c=0;
    alloc_ctx			  heap=0;
    int				  _status=0;
    x509__AlgorithmIdentifier	  algId;
    int				  r;

    *algo=0;
    
    algid.object=ai;
    algid.length=ailen;
    MEMZERO(algId);
    
    create_a2c_ctx(&a2c);
    create_pool_ctx(&heap);
    
    if(r=decode_using_any(a2c,&algid,x509,X509__ALGORITHMIDENTIFIER,
			  heap, &algId, 0))
	ABORT(r);

    if(SAME_OBJECT_IDENTIFIER(algId.algorithm,oid_des_cbc))
	*is_cdmf=0;
    else if(SAME_OBJECT_IDENTIFIER(algId.algorithm,oid_desCDMF))
	*is_cdmf=1;
    else
	ABORT(-1);

    if(r=B_CreateAlgorithmObject(algo))
	ABORT(r);

    if(algId.parameters.object[0]!=0x04 ||
      algId.parameters.object[1]!=0x08 ||
      algId.parameters.length!=10)
      ABORT(-1);
    

    if(r=B_SetAlgorithmInfo(*algo, AI_DES_CBCPadIV8,
      algId.parameters.object + 2))
	ABORT(r);

    _status=0;

 abort:
    if(_status){
	B_DestroyAlgorithmObject(algo);
    }
    (VOID) AC_delete(&heap);
    delete_a2c_ctx(&a2c);

    return(_status);
}

	
PRIVATE int
encrypt_block(blk, blen, key, klen, ai, ailen, ob, obl)
    unsigned char                *blk;
    unsigned int                  blen;
    unsigned char                *key;
    unsigned int                  klen;
    unsigned char                *ai;
    unsigned int                  ailen;
    unsigned char               **ob;
    unsigned int                 *obl;
{
    int                           r;
    B_KEY_OBJ                     bkey=0;
    B_ALGORITHM_OBJ               alg=0;
    unsigned char                *b;
    unsigned int                  l,
                                  o;
    int				  is_cdmf;

    if(r=prepare_algorithm(ai, ailen, &alg, &is_cdmf))
	return(r);
    
    prepare_key(key, klen, &bkey, is_cdmf);

    l = blen + 8;
    if (! (*ob = b = (unsigned char *)malloc(l)))
        return (-1);
    *obl = 0;
    if (r = B_EncryptInit(alg, bkey, MY_ALGORITHM_CHOOSER, 0))
        return (r);
    if (r = B_EncryptUpdate(alg, b, &o, blen, blk, blen, 0, 0))
        return (r);
    b += o;
    blen -= o;
    *obl += o;
    l -= o;
    if (r = B_EncryptFinal(alg, b, &o, l, 0, 0))
        return (r);
    *obl += o;

    B_DestroyAlgorithmObject(&alg);
    B_DestroyKeyObject(&bkey);
    return (0);
}

PRIVATE int
decrypt_block(blk, blen, key, klen, ai, ailen, ob, obl)
    unsigned char                *blk;
    unsigned int                  blen;
    unsigned char                *key;
    unsigned int                  klen;
    unsigned char                *ai;
    unsigned int                  ailen;
    unsigned char               **ob;
    unsigned int                 *obl;
{
    int                           r;
    B_ALGORITHM_OBJ               alg=0;
    B_KEY_OBJ                     bkey=0;
    unsigned char                *b;
    unsigned int                  l,
                                  o;
    int				  is_cdmf;

    if(r=prepare_algorithm(ai, ailen, &alg, &is_cdmf))
	return(r);
    
    prepare_key(key, klen, &bkey, is_cdmf);

    l = blen + 8;
    if (! (*ob = b = (unsigned char *)malloc(l)))
        return (-1);
    *obl = 0;
    if (r = B_DecryptInit(alg, bkey, MY_ALGORITHM_CHOOSER, 0))
        return (r);
    if (r = B_DecryptUpdate(alg, b, &o, blen, blk, blen, 0, 0))
        return (r);
    b += o;
    blen -= o;
    *obl += o;
    l -= o;
    if (r = B_DecryptFinal(alg, b, &o, l, 0, 0))
        return (r);
    *obl += o;

    B_DestroyAlgorithmObject(&alg);
    B_DestroyKeyObject(&bkey);
    return (0);
}



