/* 
 *
 *    io_ctx.h
 *
 *    $Source: /usr/local/CVS/io_ctx/src/io_ctx.h,v 
 *    $Revision: 1.9 
 *    $Date: 1997/06/06 00:39:50 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */





#ifndef __IO_CTX_H__
#define __IO_CTX_H__



#include <stdio.h>

#include <define.h>


#ifdef __cplusplus
extern "C" {
#endif




#define IO_delete(  ctxp)  \
           SAFE_DELETE_VCALL__(ctxp,((*(ctxp))->vtbl->delete_io_ctx((ctxp))))

#define IO_fgetc(   ctx)         ((ctx)->vtbl->fgetc_((ctx)->handle))
#define IO_fputc(   c, ctx)      ((ctx)->vtbl->fputc_((c), (ctx)->handle))
#define IO_ftell(   ctx)         ((ctx)->vtbl->ftell_((ctx)->handle))
#define IO_fflush(  ctx)         ((ctx)->vtbl->fflush_((ctx)->handle))
#define IO_ungetc(  c, ctx)      ((ctx)->vtbl->ungetc_((c), (ctx)->handle))
#define IO_fread(   ptr, size, nitems, ctx)   \
                    ((ctx)->vtbl->fread_((ptr),(size),(nitems),(ctx)->handle))
#define IO_fwrite(  ptr, size, nitems, ctx)  \
                    ((ctx)->vtbl->fwrite_((ptr),(size),(nitems),(ctx)->handle))
#define IO_fseek(   ctx, offset, pos)         \
                    ((ctx)->vtbl->fseek_((ctx)->handle, (offset), (pos)))

typedef int position;             /* either BOF, CUR, or EOF */ 
#ifndef FSEEK_BOF
#    define FSEEK_BOF   0
#    define FSEEK_CUR   1
#    define FSEEK_EOF   2
#endif




typedef struct io_ctx_ *io_ctx;

struct io_ctx_vtbl_ {
    int     (*fgetc_)   PROTO_LIST((VOID*));
    int     (*fputc_)   PROTO_LIST((int, VOID*));
    int     (*fread_)   PROTO_LIST((char*, int, int, VOID*));
    int     (*fwrite_)  PROTO_LIST((CONST char*, int, int, VOID*));
    int     (*fseek_)   PROTO_LIST((VOID*, long, position));
    int     (*ftell_)   PROTO_LIST((VOID*));
    int     (*fflush_)  PROTO_LIST((VOID*));
    int     (*ungetc_)  PROTO_LIST((int, VOID*));

    /* for deleting an io_ctx */
    int     (*delete_io_ctx)  PROTO_LIST((io_ctx*));
};

struct io_ctx_ {
    struct io_ctx_vtbl_  *vtbl;
    VOID                 *handle;
};


typedef int     (*fgetc_type)     PROTO_LIST((VOID*));
typedef int     (*fputc_type)     PROTO_LIST((int, VOID*));
typedef int     (*fread_type)     PROTO_LIST((char*, int, int, VOID*));
typedef int     (*fwrite_type)    PROTO_LIST((CONST char*, int, int, VOID*));
typedef int     (*fseek_type)     PROTO_LIST((VOID*, long, position));
typedef int     (*ftell_type)     PROTO_LIST((VOID*));
typedef int     (*fflush_type)    PROTO_LIST((VOID*));
typedef int     (*ungetc_type)    PROTO_LIST((int, VOID*));




/* 
 *    ERRORS 
 */
#define IO_CTX_MODULE_NO_             3
#define IO_CTX_ERR_NULL_BUFFER        ERROR_MAKE(IO_CTX_MODULE_NO_, 1)


#ifdef __cplusplus
}
#endif
#endif

