/*
 *
 *    io_ctx_memio.c
 *
 *    $Source: /usr/local/CVS/io_ctx/src/io_ctx_memio.c,v 
 *    $Revision: 1.9 
 *    $Date: 1997/06/06 00:39:58 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */




#include "internal_memory.h"
#include "io_ctx_error.h"
#include "io_ctx_memio.h"


#ifndef LINT
static char rcsid[] = "$Id: io_ctx_memio.c,v 1.9 1997/06/06 00:39:58 briank Exp ";
#endif





static VOID   grow_memory
  PROTO_LIST((
    MEMORY*        mem,
    unsigned int   by_size 
  ));

static VOID   attempt_to_grow_memory
  PROTO_LIST((
    MEMORY         *mem,
    unsigned int    nbytes
  ));

static int    memory_size
  PROTO_LIST((
    MEMORY*        mem 
  ));

static int    memory_error
  PROTO_LIST((
    MEMORY  *mem 
  ));

static int    mem_getc
  PROTO_LIST((
    MEMORY  *mem 
  ));

static int    mem_putc
  PROTO_LIST((
    int     c,
    MEMORY  *mem 
  ));

static int    mem_read
  PROTO_LIST((
    char    *ptr,
    int     size, 
    int     nitems,
    MEMORY  *mem    
  ));

static int    mem_write
  PROTO_LIST((
    char    *ptr,
    int     size, 
    int     nitems,
    MEMORY  *mem    
  ));

static int    mem_seek
  PROTO_LIST((
    MEMORY  *mem,
    long    offset,
    int     whence 
  ));

static int    mem_tell
  PROTO_LIST((
    MEMORY  *mem 
  ));

static int    mem_flush
  PROTO_LIST((
    MEMORY  *mem 
  ));

static int    mem_ungetc
  PROTO_LIST((
    int      c,
    MEMORY  *mem 
  ));



static struct io_ctx_vtbl_  memio_ctx_vtbl = {
    (fgetc_type)    mem_getc,
    (fputc_type)    mem_putc,
    (fread_type)    mem_read,
    (fwrite_type)   mem_write,
    (fseek_type)    mem_seek,
    (ftell_type)    mem_tell,
    (fflush_type)   mem_flush,
    (ungetc_type)   mem_ungetc,
                    delete_memio_ctx,
};



INLINE static int
memory_size(mem)
MEMORY*        mem;
{
    return (int)(mem->end - mem->array);
}

INLINE static int
memory_error(mem)
MEMORY  *mem;
{
  if (mem == 0
   || mem->array == 0
   || mem->cur == 0
   || mem->end == 0
   || mem->cur < mem->array
   || mem->end < mem->array
   ) return 1;

  return 0;
}



static VOID
attempt_to_grow_memory(mem, nbytes)
    MEMORY         *mem;
    unsigned int    nbytes;
{
    int       nremaining;
    int       by_size;

    if (mem->dynamic == 0) return;  /* cannot grow non-dynamic memory */

    nremaining = mem->end - mem->cur;

    if (nremaining < 0) {
        by_size = (-nremaining)+nbytes;
        grow_memory(mem, by_size);
        /* put zeroes into the space between the old mem->end and the mem->cur */
        memset(mem->cur+nremaining, 0, (-nremaining));
    }
    else if ((unsigned int)nremaining < nbytes) grow_memory(mem, nbytes);

    return;
}


static VOID
grow_memory(mem, by_size)
    MEMORY         *mem;
    unsigned int    by_size;
{
    char  *new_array;
    int    size;
    int    delta;

    size = memory_size(mem) + by_size;



    if (mem->special_realloc == 0) {
        new_array = (char*)REALLOC(mem->array, size);
    }
    else {
        new_array = (char*)(mem->special_realloc)(mem->special_ctx, mem->array, size);
    }
    delta = (int)(new_array - (int)mem->array);

    /*  if memory allocation failed, then the buffer isn't grown and thus
     *  the IO_fwrite() or IO_fputc() will fail. */
    if (new_array != 0) {
        if (delta != 0) {
            mem->cur    += delta;
            mem->end    += delta;
            mem->array  += delta;
        }
        mem->end += by_size;
    }

    return;
}


/***********************/

static int
mem_getc(mem)
MEMORY  *mem;
{
  if (memory_error(mem) || mem->cur >= mem->end) return EOF;
  return (int)*(mem->cur++);
}

static int
mem_putc(c, mem)
int      c;
MEMORY  *mem;
{
    if (memory_error(mem)) return EOF;

    attempt_to_grow_memory(mem, 1);
    /* grow_memory() might fail */
    if (mem->cur >= mem->end) return EOF;

/*error_log_printf(DEFAULT_ERROR_CTX, THIS_MODULE_NUMBER,"mem->cur: %08x\n",(UINT4)mem->cur);*/
    *(mem->cur++) = (unsigned char)c;
    return c;
}

static int
mem_read(ptr, size, nitems, mem)
char    *ptr;
int     size; 
int     nitems;
MEMORY  *mem;   
{
    unsigned int  nbytes = (size*nitems);

    if (memory_error(mem)) return 0;
    if (size <= 0 || nitems <= 0) return 0;
    if (mem->cur >= mem->end) return 0;

    if (mem->cur+nbytes > mem->end) nbytes = mem->end - mem->cur;

    memcpy(ptr, mem->cur, nbytes);
    mem->cur += nbytes;
    return (size==0)?0:(nbytes / size);
}

static int
mem_write(ptr, size, nitems, mem)
char    *ptr;
int     size; 
int     nitems;
MEMORY  *mem;   
{
    int  nbytes = (size*nitems);

    if (memory_error(mem)) return 0;
    if (size <= 0 || nitems <= 0) return 0;

    attempt_to_grow_memory(mem, nbytes);
    /* grow_memory() might fail, so nbytes is recomputed if necessary */
    if (mem->end-mem->cur < nbytes) {
        if (mem->cur < mem->end)  nbytes = mem->end - mem->cur;
        else                      nbytes = 0;
    }

    if (nbytes > 0) {
        memcpy(mem->cur, ptr, nbytes);
        mem->cur += nbytes;
    }

    return (nbytes / size);
}

static int
mem_seek(mem, offset, whence)
MEMORY  *mem;
long    offset;
int     whence;
{
    unsigned char *dest;

    if (!mem || mem->array == 0) return -1;

    switch(whence) {
    case FSEEK_BOF:
        dest = mem->array + offset;
        break;
    case FSEEK_CUR:
        dest = mem->cur + offset;
        break;
    case FSEEK_EOF:
        dest = mem->end + offset;
        break;
    default:
        return -1;
        break;
    }

    if (dest < mem->array) return -1;

    mem->cur = dest;
    return 0;
}

static int
mem_tell(mem)
MEMORY  *mem;
{
    if (memory_error(mem)) return -1;

    if (!mem || mem->array == 0) return -1;
    return mem->cur - mem->array;
}

static int
mem_flush(mem)
MEMORY  *mem;
{
    if (memory_error(mem)) return EOF;
    return 0;
}

static int
mem_ungetc(c, mem)
int      c;
MEMORY  *mem;
{
    if (memory_error(mem)) return EOF;
    if (c == EOF || mem->cur <= mem->array) return EOF;
    mem->cur--;
    return c;
}









PUBLIC int
create_memio_ctx(memory_io_ctx, initial_size)
    io_ctx         *memory_io_ctx;      /* OUT */
    unsigned int    initial_size;       /* IN */
{
    unsigned char  *buffer = 0;
    int             status;

    if (memory_io_ctx == 0) ERETURN(SE_ERR_ILLEGAL_NULL_ARG);

    if (initial_size < 1) initial_size = 1;
    buffer = (unsigned char*)MALLOC(initial_size);
    if (buffer == 0) ERETURN(SE_ERR_NO_MEMORY);

    /*  a bound, static memio_ctx is created, but these 
     *  characteristics are overridden below */
    status = create_bound_static_memio_ctx(memory_io_ctx, buffer, 0);
    if (status != NO_ERROR) {
        (VOID)delete_memio_ctx(memory_io_ctx);
        if (status == SE_ERR_NO_MEMORY) ERETURN(SE_ERR_NO_MEMORY);
        ERETURN(SE_ERR_INTERNAL);
    }

    ((MEMORY*)(*memory_io_ctx)->handle)->dynamic       = 1;
    ((MEMORY*)(*memory_io_ctx)->handle)->bound         = 0;

    return NO_ERROR;
}



PUBLIC int
create_bound_static_memio_ctx(memory_io_ctx, buffer, size)
    io_ctx         *memory_io_ctx;     /* OUT */
    unsigned char  *buffer;            /* IN */
    unsigned int    size;              /* IN */
{
    MEMORY  *mem = 0;
    int      status;
    struct tmp_ {
        struct io_ctx_   ctx;
        MEMORY           mem;
    };

    if (memory_io_ctx == 0) ERETURN(SE_ERR_ILLEGAL_NULL_ARG);
    if (buffer == 0) ERETURN(IO_CTX_ERR_NULL_BUFFER);
    if (size > 0) status = buffer[size-1];  /* try for seg fault */

    *memory_io_ctx = (io_ctx)CALLOC(1, sizeof(struct tmp_));
    if (*memory_io_ctx == 0) ERETURN(SE_ERR_NO_MEMORY);
    (*memory_io_ctx)->vtbl = &memio_ctx_vtbl;
    (*memory_io_ctx)->handle = &((struct tmp_*)*memory_io_ctx)->mem;

    mem = (MEMORY*)(*memory_io_ctx)->handle;
    mem->array    = buffer;
    mem->cur      = mem->array;
    mem->end      = mem->array + size;
    mem->dynamic  = 0;
    mem->bound    = 1;

    return NO_ERROR;
}



PRIVATE int
delete_memio_ctx(memory_io_ctx)
    io_ctx         *memory_io_ctx;     /* IN/OUT */
{
    if (memory_io_ctx == 0) ERETURN(SE_ERR_ILLEGAL_NULL_ARG);

    if (*memory_io_ctx != 0) {
        if (((MEMORY*)(*memory_io_ctx)->handle)->bound == 0) {
            SAFE_FREE(((MEMORY*)(*memory_io_ctx)->handle)->array);
        }
        MEMZERO(**(struct io_ctx_**)memory_io_ctx);
        FREE(*memory_io_ctx);
    }
    *memory_io_ctx = 0;  

    return NO_ERROR;
} 





int
acquire_memory_from_memio_ctx(memory_io_ctx, ptr, len)
    io_ctx         memory_io_ctx;     /* IN */
    UCHAR        **ptr;               /* OUT */
    int           *len;               /* OUT */
{
    MEMORY        *mem = (MEMORY*)memory_io_ctx->handle;

    *ptr = 0;

    if (memory_error(mem)) ERETURN(SE_ERR_ILLEGAL_ARG);

    /* from now on it'll be the user's responsibility to free the buffer */
    mem->bound = 1;

    *len = memory_size(mem);
    *ptr = mem->array;
    
    return NO_ERROR;
}









