/*
 *
 *    io_ctx_memio.h
 *
 *    $Source: /usr/local/CVS/io_ctx/src/io_ctx_memio.h,v 
 *    $Revision: 1.9 
 *    $Date: 1997/06/06 00:39:59 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */




#ifndef __IO_CTX_MEMIO_H__
#define __IO_CTX_MEMIO_H__


#include <define.h>

#include "io_ctx.h"


#ifdef __cplusplus
extern "C" {
#endif




/**
 *
 *    @function     create_memio_ctx
 *    @one-liner    create a regular <memio_ctx>, which allocates and
 *                  grows its own memory
 *    @comment      regular <memio_ctx>s allocate and grow their own memory
 *    @comment      the <io_ctx> must be deleted using the <IO_delete()> method
 *    @domain       public
 *    @prototype
 *    int create_memio_ctx(memory_io_ctx, initial_size)
 *        io_ctx         *memory_io_ctx;    // OUT
 *        unsigned int    initial_size;     // IN
 *    @param-out    memory_io_ctx   a pointer to the <io_ctx> to create
 *    @param-in     initial_size    a hint, suggesting the maximum amount
 *                                  of data that will be written to the <io_ctx>
 *    @return       NO_ERROR        on success
 *    @return       SE_ERR_ILLEGAL_NULL_ARG  illegal null pointer parameter
 *    @return       SE_ERR_NO_MEMORY         unable to allocate memory
 *    @return       SE_ERR_INTERNAL          internal inconsistency, perhaps
 *                                           as the result of programmer error
 *    @example
 *    @bugs
 *    @see          io_ctx_memio.h#create_bound_static_memio_ctx
 */

PUBLIC int          create_memio_ctx
  PROTO_LIST((
    io_ctx         *memory_io_ctx,       /* OUT */
    UINT4           initial_size         /* IN */
  ));



/**
 *
 *    @function     create_bound_static_memio_ctx
 *    @one-liner    create a static, bound <memio_ctx>, which uses a
 *                  user-supplied buffer
 *    @comment      bound, static <memio_ctx>s use user-supplied memory, and do
 *                  not grow that memory.
 *                  Attempts to write past the end of this static memory will
 *                  result in an end-of-file (EOF) error.
 *    @comment      the <io_ctx> must be deleted using the <IO_delete()> method
 *    @domain       public
 *    @prototype
 *    int create_bound_static_memio_ctx(memory_io_ctx, buffer, size)
 *        io_ctx         *memory_io_ctx;       // OUT
 *        unsigned char  *buffer;              // IN
 *        unsigned int    size;                // IN
 *    @param-out    memory_io_ctx   a pointer to the <io_ctx> to create
 *    @param-in     buffer          caller-supplied memory to perform I/O on
 *    @param-in     size            the size of the caller-supplied buffer.
 *                                  <size> must not be larger than the
 *                                  actual size of <buffer>.
 *    @return      NO_ERROR        on success
 *    @return      SE_ERR_ILLEGAL_NULL_ARG  illegal null pointer parameter
 *    @return      IO_CTX_ERR_NULL_BUFFER   illegal null <buffer> pointer
 *    @return      SE_ERR_NO_MEMORY         unable to allocate memory
 *    @example
 *    @bugs
 *    @see          io_ctx_memio.h#create_memio_ctx
 */

PUBLIC int          create_bound_static_memio_ctx
  PROTO_LIST((
    io_ctx         *memory_io_ctx,       /* OUT */
    UCHAR          *buffer,              /* IN */
    UINT4           size                 /* IN */
  ));


/**
 *
 *    @function     acquire_memory_from_memio_ctx
 *    @one-liner    obtain the internal buffer pointer and buffer length
 *                  from a memio_ctx
 *    @comment      Obtain the internal buffer pointer and buffer length
 *                  from a memio_ctx.  After a call to
 *                  <acquire_memory_from_memio_ctx()>
 *                  it becomes the responsibility of the user to free the
 *                  buffer returned in `ptr'.  The <IO_delete()> method will
 *                  not free this memory.
 *    @domain       public
 *    @prototype
 *        int acquire_memory_from_memio_ctx(memory_io_ctx, buffer, len)
 *          PROTO_LIST((
 *            io_ctx         memory_io_ctx;     // IN
 *            UCHAR        **buffer;            // OUT
 *            int           *len;               // OUT 
 *    @param-in     memory_io_ctx   the <io_ctx> containing the internal buffer
 *    @param-out    buffer          the internal buffer
 *    @param-out    len             the current length of the buffer
 *    @return       NO_ERROR        on success
 *    @return       SE_ERR_ILLEGAL_ARG       bad `memory_io_ctx' parameter
 *    @example
 *    @bugs         The internal buffer pointer may change as a result of
 *                  a write if additional memory must be allocated.
 *    @see
 */

PUBLIC int    acquire_memory_from_memio_ctx
  PROTO_LIST((
    io_ctx         memory_io_ctx,     /* IN */
    UCHAR        **buffer,            /* OUT */
    int           *len                /* OUT */
  ));



#ifdef __cplusplus
}
#endif
#endif

