/*
 *
 *    io_ctx_stdio.c
 *
 *    $Source: /usr/local/CVS/io_ctx/src/io_ctx_stdio.c,v 
 *    $Revision: 1.11 
 *    $Date: 1997/06/06 00:40:01 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */


#include <stdio.h>

#include <define.h>

#include "io_ctx.h"
#include "io_ctx_error.h"
#include "io_ctx_stdio.h"


#ifndef LINT
static char rcsid[] = "$Id: io_ctx_stdio.c,v 1.11 1997/06/06 00:40:01 briank Exp ";
#endif





static int    delete_stdio_ctx
  PROTO_LIST((
    io_ctx    *ctx_p 
  ));



static struct io_ctx_vtbl_  stdio_ctx_vtbl__ = {
    (fgetc_type)    fgetc,
    (fputc_type)    fputc,
    (fread_type)    fread,
    (fwrite_type)   fwrite,
    (fseek_type)    fseek,
    (ftell_type)    ftell,
    (fflush_type)   fflush,
    (ungetc_type)   ungetc,
                    delete_stdio_ctx,
};





PUBLIC int
create_stdio_ctx(ctx_p, fp)
    io_ctx    *ctx_p;        /* IN/OUT */
    FILE      *fp;           /* IN */
{
    struct io_ctx_ *ctx;

    if (ctx_p == 0) ERETURN(SE_ERR_ILLEGAL_NULL_ARG);
    ctx = (struct io_ctx_ *)MALLOC(sizeof(struct io_ctx_));
    if (ctx==0) ERETURN(SE_ERR_NO_MEMORY);
    ctx->vtbl = &stdio_ctx_vtbl__;
    ctx->handle = (VOID*)fp;
    *ctx_p = ctx;
    return NO_ERROR;
}

/* returns NO_ERROR or SE_ERR_ILLEGAL_NULL_ARG */
static int
delete_stdio_ctx(ctx_p)
    io_ctx    *ctx_p;         /* IN/OUT */
{
    if (ctx_p == 0) ERETURN(SE_ERR_ILLEGAL_NULL_ARG);
    if (*ctx_p != 0) {
        MEMZERO(**(struct io_ctx_**)ctx_p);
        FREE(*ctx_p);
    }
    *ctx_p = 0;
    return NO_ERROR;
}









