/*
 *
 *    kdb.c
 * 
 *    $Source: /usr/local/CVS/SETREF_1_0_Release/kdb/src/kdb.c,v 
 *    $Revision: 1.13 
 *    $Date: 1997/06/06 00:40:30 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 * 
 *    ekr@terisa.com  Mon Feb  3 16:04:08 1997
 *
 */

#ifndef LINT
static char rcsid[] = "$Id: kdb.c,v 1.13 1997/06/06 00:40:30 briank Exp ";
#endif

#include <define.h>
#include <store.h>
#include "kdb.h"
#include "kdb_error.h"

/*Field names */
#define SUBJ_NAME "subjectName"
#define CERT_DATA "certificate"
#define KEY_PRIV "privateKey"
#define CRL_DATA "crlData"

#define CERT_CLASS 'C'
#define KEY_CLASS 'K'
#define CRL_CLASS 'R'
#define DN_CLASS 'D'

#define DB_MODE 420             /* 644 */
#define BLOCK_SIZE 4096

typedef struct {
    DB_CTX_FILE                   shadow,
                                  orig;
    char                         *orig_name;
    int                           access_mode;
} db_obj;


static int writeobj PROTO_LIST((VOID *, int, unsigned char *, unsigned int,
                                 int, char *,
                                 unsigned char *, unsigned int));
static int deleteobj PROTO_LIST((VOID *, int, unsigned char *, unsigned int,
                                 int, char *));
static int firstobjm PROTO_LIST((VOID *, int, unsigned char *,
                                  unsigned int, unsigned char **));
static int nextobjm PROTO_LIST((VOID *, int, unsigned char *,
                                 unsigned int, unsigned char **));
static int readobj PROTO_LIST((VOID *, int, unsigned char *,
                           unsigned int, int, char *, unsigned char **));
static int mergekeys PROTO_LIST((unsigned char *, unsigned int,
                                  unsigned char *, unsigned int,
                                  int, unsigned char **));
static unsigned char         *last_field PROTO_LIST((unsigned char *,
	             unsigned int));

static db_ctx_data            current_key;

int add_kdb_obj PROTO_LIST((
                                  kdb_objs ** list,
                                  UCHAR * data,
                                  unsigned int data_len
));

void fatal_func PROTO_LIST((char *a));

void 
fatal_func(a)
    char                         *a;
{
    error_log_printf(DEFAULT_ERROR_CTX, THIS_MODULE_NUMBER, "GDBM open failed %s\n", a);
}

int 
NDBM_InsertCert(iname, inamel, serial, seriall, subject, subjectl, list, handle)
    unsigned char                *iname,
                                 *serial,
                                 *subject;
    unsigned int                  inamel,
                                  seriall,
                                  subjectl;
    kdb_objs                     *list;
    VOID                         *handle;
{
    unsigned char                *data;
    unsigned int                  datalen;

    data = list->data.object;
    datalen = list->data.length;

    return (NDBM_InsertCertData(iname, inamel, serial, seriall, subject, subjectl, data, datalen,
                                handle));
}

int 
NDBM_InsertCertData(iname, inamel, serial, seriall, subject, subjectl, data, datalen,
                    handle)
    unsigned char                *iname,
                                 *serial,
                                 *subject,
                                 *data;
    unsigned int                  inamel,
                                  seriall,
                                  subjectl;
    unsigned int                  datalen;
    VOID                         *handle;
{
    unsigned char                *k;
    int                           kl;

    if (! (kl = mergekeys(iname, inamel, serial, seriall, 1, &k)))
        goto fail;
    if (writeobj(handle, CERT_CLASS, k, kl, 0, SUBJ_NAME, subject, subjectl))
        goto fail;
    if (writeobj(handle, CERT_CLASS, k, kl, 0, CERT_DATA, data, datalen))
        goto fail;
    SAFE_FREE(k);
    return (0);

  fail:
    if (k)
        SAFE_FREE(k);
    return (0);
}

int 
NDBM_DeleteCertData(iname, inamel, serial, seriall, handle)
    unsigned char                *iname,
                                 *serial;
    unsigned int                  inamel,
                                  seriall;
    VOID                         *handle;
{
    unsigned char                *k;
    int                           kl;

    if (! (kl = mergekeys(iname, inamel, serial, seriall, 1, &k)))
        goto fail;
    if (deleteobj(handle, CERT_CLASS, k, kl, 0, SUBJ_NAME))
        goto fail;
    if (deleteobj(handle, CERT_CLASS, k, kl, 0, CERT_DATA))
        goto fail;
    SAFE_FREE(k);
    return (0);

  fail:
    if (k)
        SAFE_FREE(k);
    return (0);
}

int 
NDBM_SelectCertByIssuerSerial(listObject, issuerName, issuerNameLen, serialNumber,
                              serialNumberLen, handle)
    kdb_objs                    **listObject;
    unsigned char                *issuerName,
                                 *serialNumber;
    unsigned int                  issuerNameLen,
                                  serialNumberLen;
    VOID                         *handle;
{
    unsigned char                *k,
                                 *data;
    int                           kl,
                                  dl;

    if (! (kl = mergekeys(issuerName, issuerNameLen, serialNumber, serialNumberLen, 1, &k)))
        return (KDB_ERR_INTERNAL);
    if ((dl = readobj(handle, CERT_CLASS, k, kl, 0, CERT_DATA, &data)) < 0) {
        SAFE_FREE(k);
        return (0);
    }
    SAFE_FREE(k);
    return (add_kdb_obj(listObject, data, dl));
}

struct keylist {
    unsigned int                  keylen;
    unsigned char                *key;
    struct keylist               *next;
};

int 
NDBM_SelectCertsBySubject(listObject, subjectName, subjectNameLen, handle)
    kdb_objs                    **listObject;
    unsigned char                *subjectName;
    unsigned int                  subjectNameLen;
    VOID                         *handle;
{
    int                           keylen,
                                  certlen,
                                  x,
                                  kl;
    unsigned char                *key,
                                 *cert,
                                 *k;
    struct keylist               *klist,
                                 *tmp,
                                 *cur = NULL;

    /* Having to allocate this array instead of processing on the fly
     * really sucks, but readobj seems to confuse firstobj */
    if ((keylen = firstobjm(handle, CERT_CLASS, subjectName, subjectNameLen, &key)) >= 0) {
        do {
            if (! (tmp = (struct keylist *)MALLOC(sizeof(struct keylist))))
                                              return (KDB_ERR_NO_MEMORY);

            tmp->next = NULL;
            tmp->keylen = keylen;
            if (! (tmp->key = (UCHAR*)MALLOC(keylen)))
                return (KDB_ERR_NO_MEMORY);
            memcpy(tmp->key, key, keylen);

            if (cur) {
                cur->next = tmp;
                cur = tmp;
            }
            else {
                klist = tmp;
                cur = klist;
            }
        } while ((keylen = nextobjm(handle, CERT_CLASS, subjectName, subjectNameLen,
                                    &key)) >= 0);
        for (tmp = klist; tmp;) {
            k = last_field(tmp->key, tmp->keylen);
            kl = (k - tmp->key);
            if ((certlen = readobj(handle, CERT_CLASS, tmp->key + 1, kl - 1, 0, CERT_DATA, &cert)) >= 0) {
                if (x = add_kdb_obj(listObject, cert, certlen))
                    return (x);
            }
            else {
/*          printf("couldn't read\n"); */
            }
            klist = tmp;
            tmp = tmp->next;
            SAFE_FREE(klist->key);
            SAFE_FREE(klist);
        }
    }
    return (0);
}

int 
NDBM_InsertPrivateKey(pubkey, pubkeylen, privkey, privkeylen, handle)
    unsigned char                *pubkey,
                                 *privkey;
    unsigned int                  pubkeylen,
                                  privkeylen;
    VOID                         *handle;
{
    db_obj                       *dbp;

    if (! (dbp = (db_obj *) handle))
        return (-1);
    if (writeobj(handle, KEY_CLASS, pubkey, pubkeylen, 1, KEY_PRIV, privkey, privkeylen) < 0)
        return (KDB_ERR_INTERNAL);
    return (0);
}

int 
NDBM_SelectPrivateKey(privkey, privkeylen, pubkey, pubkeylen, handle)
    unsigned char                *pubkey,
                                 *privkey;
    unsigned int                  pubkeylen,
                                 *privkeylen;
    VOID                         *handle;
{
    unsigned char                *pk;
    int                           pkl;

    if ((pkl = readobj(handle, KEY_CLASS, pubkey, pubkeylen, 1, KEY_PRIV, &pk)) < 0)
        return (KDB_ERR_NOT_FOUND);
    *privkeylen = pkl;
    memcpy(privkey, pk, *privkeylen);
    return (0);
}

int 
NDBM_InsertCRL(issuerName, issuerNameLen, lastUpdate, listObject, handle)
    unsigned char                *issuerName;
    unsigned int                  issuerNameLen;
    UINT4                         lastUpdate;
    kdb_objs                     *listObject;
    VOID                         *handle;
{
    unsigned char                *data;
    unsigned int                  datalen;

    data = listObject->data.object;
    datalen = listObject->data.length;

    return (NDBM_InsertCRLData(issuerName, issuerNameLen, lastUpdate, data, datalen,
                               handle));
}

int 
NDBM_InsertCRLData(issuerName, issuerNameLen, lastUpdate, data, datalen, handle)
    unsigned char                *issuerName,
                                 *data;
    unsigned int                  issuerNameLen;
    UINT4                         lastUpdate;
    unsigned int                  datalen;
    VOID                         *handle;
{
    unsigned char                *k;
    unsigned int                  kl;

    if (! (kl = mergekeys(issuerName, issuerNameLen, (UCHAR*)&lastUpdate, sizeof(
                                                    lastUpdate), 1, &k)))
        return (KDB_ERR_INTERNAL);
    if (writeobj(handle, CRL_CLASS, k, kl, 0, CRL_DATA, data, datalen)) {
        SAFE_FREE(k);
        return (KDB_ERR_INTERNAL);
    }
    SAFE_FREE(k);
    return (0);
}

int 
NDBM_SelectCRLByIssuerTime(listObject, issuerName, issuerNameLen, issuerTime, handle)
    kdb_objs                    **listObject;
    unsigned char                *issuerName;
    unsigned int                  issuerNameLen;
    UINT4                         issuerTime;
    VOID                         *handle;
{
    unsigned char                 bestdata[BLOCK_SIZE];
    unsigned char                *key,
                                 *sl,
                                 *crl;
    int                           keylen,
                                  crllen;
    UINT4                         thistime,
                                  besttime = 0;
UNUSED_ARG(issuerName);
UNUSED_ARG(issuerNameLen);

    if ((keylen = firstobjm(handle, CRL_CLASS, (UCHAR*)NULL, 0, &key)) >= 0) {
        do {
            sl = last_field(key, keylen);
            sl = last_field(key, (sl - key));
            memcpy(&thistime, sl, sizeof(thistime));
            if ((thistime < issuerTime) && (thistime > besttime)) {
                if ((crllen = readobj(handle, CRL_CLASS, key + 2, (sl - key - 2), 0, CRL_DATA, &crl)) < 0)
                    continue;
                besttime = thistime;
                memcpy(bestdata, crl, crllen);
            }
        } while ((keylen = nextobjm(handle, CRL_CLASS, (UCHAR*)NULL, 0, &key)) >= 0);
    }
    if (besttime) return add_kdb_obj(listObject, crl, crllen);
    return (0);
}

int 
NDBM_InitDB(dbc, dbn)
    DBASE_CTX                    *dbc;
    char                         *dbn;
{
    char                         *the_db;
    db_obj                       *dbp;

    if (! (dbp = (db_obj *) MALLOC(sizeof(db_obj))))
        return (-1);
    dbc->reserved = 0;

    the_db = dbn;

    dbc->handle = dbp;
    dbp->shadow = dbp->orig = 0;
    dbp->orig_name = (char *)strdup(dbn);

    if (! (dbp->orig = DB_open(the_db, BLOCK_SIZE, DB_CTX_WRCREAT, DB_MODE,
                                fatal_func)))
        return (-1);
    dbp->access_mode = DB_W_ORIG | DB_R_ORIG;

    return (0);
}

int 
NDBM_CloseDB(handle)
    VOID                         *handle;
{
    db_obj                       *dbp;

    dbp = (db_obj *) (handle);

    if (! dbp)
        return (-1);

    if (dbp->shadow)
        DB_close(dbp->shadow);
    if (dbp->orig)
        DB_close(dbp->orig);
    if (dbp->orig_name)
        SAFE_FREE(dbp->orig_name);
    SAFE_FREE(dbp);

    return (0);
}

int 
NDBM_set_access_mode(handle, mode)
    VOID                         *handle;
    int                           mode;
{
    db_obj                       *dbp;

    dbp = (db_obj *) (handle);
#ifdef BSDI
    if ((dbp->access_mode & DB_W_ORIG) && (!(mode & DB_W_ORIG)) && (dbp->orig)) {
        dbp->orig->sync(dbp->orig, 0);
    }
#endif
    dbp->access_mode = mode;
    return (0);
}

int 
NDBM_get_access_mode(handle)
    VOID                         *handle;
{
    db_obj                       *dbp;

    dbp = (db_obj *) (handle);
    if (! dbp)
        return (-1);

    return (dbp->access_mode);
}

/*Write an object to the database. Depending on the argument,
 * write to the original db (creating if necessary), the shadow
 * db, or both */
static int 
writeobj(db, class, s, sl, addlen, e, d, dl)
    VOID                         *db;
    int                           class;
    unsigned char                *s,
                                 *d;
    char                         *e;
    unsigned int                  sl,
                                  dl;
    int                           addlen;
{
    unsigned char                *k,
                                  keys[BLOCK_SIZE + 1];
    unsigned int                  kl;
    db_ctx_data                   key,
                                  val;
    int                           el;
    db_obj                       *dbp;

    el = strlen(e);
    keys[0] = class;
    if (! (kl = mergekeys(s, sl, (UCHAR*)e, el, addlen, &k)))
        return (-1);
    memcpy(keys + 1, k, kl);
    key.dptr = (char*)keys;
    key.dsize = kl + 1;
#ifdef DEBUG
    printf("\nwrit %.2x ", key.dsize);
    for (i = 0; i < key.dsize; i++)
        printf("%.2x ", key.dptr[i]);
#endif
    val.dptr = (char*)d;
    val.dsize = dl;

    dbp = (db_obj *) db;

    if (dbp->access_mode & DB_W_SHADOW) {
        if (! dbp->shadow)
            return (-1);
        if (DB_store(dbp->shadow, key, val, DB_CTX_REPLACE)) {
            SAFE_FREE(k);
            return (-1);
        }
    }
    if (dbp->access_mode & DB_W_ORIG) {
        if (! dbp->orig) {
            if (! dbp->orig_name)
                return (-1);
            if (! (dbp->orig =
             DB_open(dbp->orig_name, BLOCK_SIZE, DB_CTX_WRCREAT, DB_MODE,
                       fatal_func)))
                return (-1);
        }
        if (DB_store(dbp->orig, key, val, DB_CTX_REPLACE)) {
            SAFE_FREE(k);
            return (-1);
        }
    }

    SAFE_FREE(k);
    return (0);
}

/*Delete an object from the database.*/
static int 
deleteobj(db, class, s, sl, addlen, e)
    VOID                         *db;
    int                           class;
    unsigned char                *s;
    char                         *e;
    unsigned int                  sl;
    int                           addlen;
{
    unsigned char                *k,
                                  keys[BLOCK_SIZE + 1];
    unsigned int                  kl;
    db_ctx_data                   key;
    int                           el;
    db_obj                       *dbp;

    el = strlen(e);
    keys[0] = class;
    if (! (kl = mergekeys(s, sl, (UCHAR*)e, el, addlen, &k)))
        return (-1);
    memcpy(keys + 1, k, kl);
    key.dptr = (char*)keys;
    key.dsize = kl + 1;
#ifdef DEBUG
    printf("\nwrit %.2x ", key.dsize);
    for (i = 0; i < key.dsize; i++)
        printf("%.2x ", key.dptr[i]);
#endif

    dbp = (db_obj *) db;

    if (dbp->access_mode & DB_W_SHADOW) {
        if (! dbp->shadow)
            return (-1);
        if (DB_delete(dbp->shadow, key)) {
            SAFE_FREE(k);
            return (-1);
        }
    }
    if (dbp->access_mode & DB_W_ORIG) {
        if (! dbp->orig) 
	  return(-1);
        if (DB_delete(dbp->orig, key)) {
            SAFE_FREE(k);
            return (-1);
        }
    }

    SAFE_FREE(k);
    return (0);
}

/*Read the first matching object from the database, either from the
 * original or the shadow (if both flags are set, from the shadow */
static int 
firstobjm(db, class, v, vl, r)
    VOID                         *db;
    int                           class;
    unsigned char                *v,
                                **r;
    unsigned int                  vl;
{
    db_ctx_data                   l;
    static char                   key[BLOCK_SIZE];
    db_obj                       *dbp;
    DB_CTX_FILE                   rdb;

    dbp = (db_obj *) db;
    rdb = dbp->access_mode & DB_R_SHADOW ? dbp->shadow : dbp->orig;

    if(current_key.dptr)
      free(current_key.dptr);
    current_key.dptr=0;
    current_key = DB_firstkey(rdb);
    if (current_key.dptr == NULL)
        return (-1);
    if (*(current_key.dptr) == class) {
        memcpy(key, current_key.dptr, current_key.dsize);
        l = DB_fetch(rdb, current_key);
        if (! vl || ((unsigned int)l.dsize == vl) && !memcmp(l.dptr, v, vl)) {
            memcpy(key, current_key.dptr,
                   current_key.dsize);
            *r = (UCHAR*)key;
	    free(l.dptr);
            return (current_key.dsize);
        }
	free(l.dptr);
    }
    return (nextobjm(db, class, v, vl, r));
}

/*Read the next matching object from the database, either from the
 * original or the shadow (if both flags are set, from the shadow */
static int 
nextobjm(db, class, v, vl, r)
    VOID                         *db;
    int                           class;
    unsigned char                *v,
                                **r;
    unsigned int                  vl;
{
    db_ctx_data                   l;
    static char                   key[BLOCK_SIZE];
    db_obj                       *dbp;
    DB_CTX_FILE                   rdb;
    char *tofree;
    dbp = (db_obj *) db;
    rdb = dbp->access_mode & DB_R_SHADOW ? dbp->shadow : dbp->orig;

    while (1) {
        tofree=current_key.dptr;
        current_key = DB_nextkey(rdb, current_key);
	SAFE_FREE(tofree);
        if (! (current_key.dptr))
            break;   /* BREAK */
        if (*(current_key.dptr) != class)
            continue;
        memcpy(key, current_key.dptr, current_key.dsize);
        l = DB_fetch(rdb, current_key);
        if (! vl || ((unsigned int)l.dsize == vl) && !memcmp(l.dptr, v, vl)) {
            *r = (UCHAR*)key;
	    SAFE_FREE(l.dptr);
            return (current_key.dsize);
        }
	SAFE_FREE(l.dptr);

    }
    return (-1);
}

/*Read an object from the database, either from the original or the
 * shadow (if both flags are set, from the shadow */
static int 
readobj(db, class, s, sl, addlen, e, r)
    VOID                         *db;
    int                           class;
    unsigned char                *s,
                                **r;
    char                         *e;
    unsigned int                  sl;
    int                           addlen;
{
    unsigned char                 keys[BLOCK_SIZE],
                                 *k;
    db_ctx_data                   key,
                                  val;
    static unsigned char          v[BLOCK_SIZE];
    unsigned int                  el,
                                  kl;
    db_obj                       *dbp;
    DB_CTX_FILE                   rdb;

    dbp = (db_obj *) db;
    rdb = dbp->access_mode & DB_R_SHADOW ? dbp->shadow : dbp->orig;

    el = strlen(e);
    keys[0] = class;
    if (! (kl = mergekeys(s, sl, (UCHAR*)e, el, addlen, &k)))
        return (-1);
    memcpy(keys + 1, k, kl);
    key.dptr = (char*)keys;
    key.dsize = kl + 1;
#ifdef DEBUG
    printf("\nread %.2x ", key.dsize);
    for (i = 0; i < key.dsize; i++)
        printf("%.2x ", key.dptr[i]);
#endif
    val = DB_fetch(rdb, key);
    if (val.dptr) {
        memcpy(v, val.dptr, val.dsize);
        *r = v;
        SAFE_FREE(k);
	SAFE_FREE(val.dptr);
        return (val.dsize);
    }
    SAFE_FREE(val.dptr);
    SAFE_FREE(k);
    return (-1);
}

static int 
mergekeys(a, al, b, bl, addlen, kp)
    unsigned char                *a,
                                 *b,
                                **kp;
    unsigned int                  al,
                                  bl;
    int                           addlen;
{
    char                         *key;
    char                          tblk[BLOCK_SIZE];   /* allow mergekeys

                                                       * to be called with 
                                                       * values returned
                                                       * from it */

    key = (char *)MALLOC(BLOCK_SIZE);
    if ((al + bl + 1) > BLOCK_SIZE)
        return (0);
    if (addlen)
        tblk[0] = al;
    memcpy(tblk + addlen, a, al);
    tblk[al + addlen] = bl;
    memcpy(tblk + al + addlen + 1, b, bl);
    *kp = (UCHAR*)key;
    memcpy(key, tblk, al + bl + addlen + 1);
    return (al + bl + addlen + 1);
}

/*last_field -- find the lastfield in a key */
static unsigned char         *
last_field(f, fl)
    unsigned char                *f;
    unsigned int                  fl;
{
    unsigned char                *c;

    c = f + 1;
    while ((c + *c + 2) < (f + fl)) {
        c += (*c + 1);
    }
    return (c);
}


int 
add_kdb_obj(list, data, data_len)
    kdb_objs                    **list;
    UCHAR                        *data;
    unsigned int                  data_len;
{
    kdb_objs                     *obj = 0;
    int                           _status;

    if (! (obj = (kdb_objs *) calloc(sizeof(kdb_objs), 1)))
        ABORT(KDB_ERR_NO_MEMORY);

    if (! (obj->data.object = (UCHAR *) malloc(data_len)))
        ABORT(KDB_ERR_NO_MEMORY);
    memcpy(obj->data.object, data, obj->data.length = data_len);

    obj->next = *list;
    *list = obj;
    _status = 0;

  abort:
    if (_status && obj) {
        if (obj->data.object)
            free(obj->data.object);
        free(obj);
    }

    return (_status);
}

int 
free_kdb_objs(listp)
    kdb_objs                    **listp;
{
    kdb_objs                     *thisp,
                                 *nextp;

    thisp = *listp;
    *listp = 0;

    while (thisp) {
        nextp = thisp->next;
        free(thisp->data.object);
        free(thisp);
        thisp = nextp;
    }

    return (0);
}

