/*
 *
 *    passphr.c
 *
 *    $Source: /usr/local/CVS/SETREF_1_0_Release/kdb/src/passphr.c,v 
 *    $Revision: 1.6 
 *    $Date: 1997/06/06 00:40:37 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    ekr@terisa.com  Wed Feb  5 11:13:47 1997
 *
 */


#ifndef LINT
static char rcsid[] = "$Id: passphr.c,v 1.6 1997/06/06 00:40:37 briank Exp ";
#endif

#include <define_bsafe.h>

#include "a2c.h"
#include "define.h"
#include "pkcs8.h"
#include "passphr.h"
#include "kdb_error.h"
#include <alloc_ctx_pool.h>

static B_ALGORITHM_METHOD    *PBE_CHOOSER[] =
{
    &AM_MD5,
    &AM_DES_CBC_ENCRYPT,
    &AM_DES_CBC_DECRYPT
};

/**
  PKCS8 Decrypt a key using password.
  and MD5withDESCBC  
  */
int 
password_unprotect_key(password, password_len,
          encrypted_key, encrypted_key_len, private_key, private_key_len)
    UCHAR                        *password;
    unsigned int                  password_len;
    UCHAR                        *encrypted_key;
    unsigned int                  encrypted_key_len;
    UCHAR                        *private_key;
    unsigned int                 *private_key_len;
{
    pkcs8__EncryptedPrivateKeyInfo enc;
    asn__any                      in;
    ITEM                          key_item,
                                  alg_ber;
    alloc_ctx                     heap = 0;
    int                           r,
                                  _status;
    char                          error[ERROR_MESSAGE_LENGTH];
    B_KEY_OBJ                     pbeKey = 0;
    B_ALGORITHM_OBJ               pbeAlg;
    unsigned int                  out_len;
    a2c_ctx                       a2c = 0;

    MEMZERO(enc);
    create_pool_ctx(&heap);
    create_a2c_ctx(&a2c);

    in.object = encrypted_key;
    in.length = encrypted_key_len;

    if (r = decode_using_any(a2c, &in, pkcs8, PKCS8__ENCRYPTEDPRIVATEKEYINFO,
                             heap, &enc, error))
        ABORT(r);

    if (r = B_CreateKeyObject(&pbeKey))
        ABORT(r);
    key_item.data = password;
    key_item.len = password_len;
    if (r = B_SetKeyInfo(pbeKey, KI_Item, (POINTER) & key_item))
        ABORT(r);
    if (r = B_CreateAlgorithmObject(&pbeAlg))
        ABORT(r);
    alg_ber.data = enc.encryptionAlgorithm.object;
    alg_ber.len = enc.encryptionAlgorithm.length;
    if (r = B_SetAlgorithmInfo(pbeAlg, AI_MD5WithDES_CBCPadBER,
                               (POINTER) & alg_ber))
        ABORT(r);
    if (r = B_DecryptInit(pbeAlg, pbeKey, PBE_CHOOSER, 0))
        ABORT(r);
    if (r = B_DecryptUpdate(pbeAlg, private_key, &out_len,
                            MAX_PRIVATE_KEY_LEN, enc.encryptedData.value,
                            enc.encryptedData.length, 0, 0))
        ABORT(r);
    if (r = B_DecryptFinal(pbeAlg, private_key + out_len, private_key_len,
                           MAX_PRIVATE_KEY_LEN - out_len, 0, 0))
        ABORT(r);
    *private_key_len += out_len;

    _status = 0;

  abort:
    (VOID)delete_a2c_ctx(&a2c);
    B_DestroyAlgorithmObject(&pbeAlg);
    B_DestroyKeyObject(&pbeKey);
    (VOID)AC_delete(&heap);
    return (_status);
}

/**
  PKCS8 Encrypt a key using password
  and MD5withDESCBC
  */
int 
password_protect_key(password, password_len,
          private_key, private_key_len, encrypted_key, encrypted_key_len)
    UCHAR                        *password;
    unsigned int                  password_len;
    UCHAR                        *private_key;
    unsigned int                  private_key_len;
    UCHAR                        *encrypted_key;
    unsigned int                 *encrypted_key_len;
{
    pkcs8__EncryptedPrivateKeyInfo enc;
    asn__any                      out;
    int                           r,
                                  _status;
    char                          error[ERROR_MESSAGE_LENGTH];
    alloc_ctx                     heap = 0;
    a2c_ctx                       a2c = 0;
    B_KEY_OBJ                     pbeKey = 0;
    B_ALGORITHM_OBJ               pbeAlg;
    unsigned int                  out_len;
    ITEM                          key_item,
                                 *alg_ber;
    B_PBE_PARAMS                  pbePar;
    UCHAR                         salt[8];
    unsigned int                  rnd;
    unsigned int                  ek_len;

    MEMZERO(enc);
    create_a2c_ctx(&a2c);
    create_pool_ctx(&heap);

    if (r = B_CreateKeyObject(&pbeKey))
        ABORT(r);
    key_item.data = password;
    key_item.len = password_len;
    if (r = B_SetKeyInfo(pbeKey, KI_Item, (POINTER) & key_item))
        ABORT(r);
    if (r = B_CreateAlgorithmObject(&pbeAlg))
        ABORT(r);

    srand(time(0));
    rnd = rand();
    memcpy(salt, (char *)&rnd, 4);
    rnd = rand();
    memcpy(salt + 4, (char *)&rnd, 4);
    pbePar.salt = salt;
    pbePar.iterationCount = 5;

    if (r = B_SetAlgorithmInfo(pbeAlg, AI_MD5WithDES_CBCPad, (POINTER) & pbePar))
        ABORT(r);

    if (r = B_EncryptInit(pbeAlg, pbeKey, PBE_CHOOSER, 0))
        ABORT(r);
    if (r = B_EncryptUpdate(pbeAlg, encrypted_key, &out_len,
      MAX_ENCRYPTED_PRIVATE_KEY_LEN, private_key, private_key_len, 0, 0))
        ABORT(r);

    if (r = B_EncryptFinal(pbeAlg, encrypted_key + out_len, &ek_len,
                           MAX_ENCRYPTED_PRIVATE_KEY_LEN - out_len, 0, 0))
        ABORT(r);

    if (r = B_GetAlgorithmInfo((POINTER *) & alg_ber, pbeAlg,
                               AI_MD5WithDES_CBCPadBER))
        ABORT(r);

    ek_len += out_len;

    enc.encryptionAlgorithm.object = alg_ber->data;
    enc.encryptionAlgorithm.length = alg_ber->len;
    enc.encryptedData.value = encrypted_key;
    enc.encryptedData.length = ek_len;

    if (r = encode_using_any(a2c, &out, pkcs8, PKCS8__ENCRYPTEDPRIVATEKEYINFO,
                             heap, &enc, error))
        ABORT(r);

    memcpy(encrypted_key, out.object, *encrypted_key_len = out.length);

    _status = 0;

  abort:
    B_DestroyKeyObject(&pbeKey);
    B_DestroyAlgorithmObject(&pbeAlg);
    (VOID)delete_a2c_ctx(&a2c);
    if (heap) (VOID)AC_delete(&heap);
    return (_status);
}

