/*
 *
 *    demo.c
 *
 *    $Source: /usr/local/CVS/SETREF_1_0_Release/demo/src/demo.c,v 
 *    $Revision: 1.18 
 *    $Date: 1997/06/26 01:48:37 
 *    $Name: SETREF_1_0_Release 
 *    $Disclaimer: 
 *    ----------------------------------------------------------------------
 *    SET Draft Reference Implementation Disclaimer/License
 *    June 30, 1997
 *    
 *    This is a beta version of the SET Draft Reference Implementation
 *    ("DRI").  It is a work in progress and is being provided at this
 *    stage and in this form to help developers and other interested
 *    parties to understand the SET protocol.  This version of the DRI
 *    provides the core functionality required to process a simple
 *    transaction.  This release does not support certificate authority
 *    messages. Nor does it support the SET option to perform
 *    transactions without cardholder certificates.  For more details
 *    on the functionality in this beta version, please refer to the
 *    attached documentation.  
 *    
 *    Later versions of the DRI will include additional functionality
 *    and may include other changes.  DO NOT RELY UPON THIS CODE IN YOUR
 *    DEVELOPMENT.  If you discover a bug, please let us know by posting
 *    a message to the SET-DEV mailing list.  Our lawyers made us include
 *    the next four paragraphs, so PLEASE READ THE TEXT BELOW because it
 *    describes what you can do with the DRI.
 *     
 *    Copyright (C) 1996, 1997, Visa International Service Association and
 *    MasterCard International Incorporated.  All rights reserved.  
 *    Permission to copy and implement the material contained herein is
 *    granted subject to the conditions that (i) any copy or re-publication
 *    must bear this legend in full; (ii) any derivative work must bear
 *    a notice that it is not the Draft Reference Implementation ("DRI")
 *    for SET and (iii) that none of the copyright holders shall have any
 *    responsibility or liability whatsoever to any other party arising
 *    from the use or publication of the material contained herein.
 *    
 *    Visa and MasterCard ("Licensors") hereby grant you ("Licensee") a
 *    limited license to (i) make, use, copy and modify the DRI for the
 *    purpose of developing software that complies with the SET
 *    Specification ("SET Products") and for the purpose of achieving a
 *    better understanding of the SET Protocol; and (ii) distribute SET
 *    Products that incorporate all or a portion of the DRI and other
 *    software, which together are provided under a defined interface
 *    different from that provided by the DRI's API.  Licensee shall not
 *    redistribute or sell a license to the DRI nor shall Licensee
 *    incorporate any part of the DRI into a SET Toolkit.  A "Toolkit"
 *    means a product that provides comprehensive facilities to enable
 *    other parties to create software products for distribution. 
 *    Bolt-on applications and payment modules shall not be considered
 *    Toolkits for the purpose of this license.  The DRI is not intended
 *    to be used as a Toolkit.
 *    
 *    Licensors make no representation or warranty of any kind, express
 *    or implied, relating to the DRI.  Licensors are making the DRI
 *    publicly available as an aid to understanding, creating, and
 *    testing SET Products.  Licensees are solely responsible for any
 *    use of the DRI and agree to hold Licensors harmless for any claim
 *    that may arise out of any product created with the DRI.  
 *    
 *    Licensee expressly acknowledges Terisa Systems, Inc., a Delaware
 *    Corporation with offices at 4984 El Camino Real, Los Altos,
 *    California, as a third party beneficiary of this license.
 *    
 *    This software is controlled material under U.S. law and the laws
 *    of other jurisdictions and you hereby acknowledge and agree that
 *    you must fully comply with all the current laws of the United States
 *    including, without limitation, rules and regulations of the United
 *    States Office of Export Administration and other applicable U.S.
 *    governmental agencies.  
 *    ----------------------------------------------------------------------
 *     $
 *
 *    Copyright (C) 1996, 1997, Visa International Service Association and MasterCard International Incorporated
 *    All Rights Reserved.
 *
 *    briank@terisa.com
 *
 */


#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifdef WIN32
#include <fcntl.h>
#include <io.h>
#endif


#include <io_ctx_memio.h>
#include <io_ctx_stdio.h>

#include <alloc_ctx_pool.h>
#include <cache_ctx_db.h>
#include <cache_state.h>
#include <log_ctx_db.h>
#include <set_msg.h>

#include "../../set_msg/src/certthumb.h"
#include "../../keydata/src/keydata.h"

#ifdef sunos413
#include <memory.h>
extern int fread PROTO_LIST((char *ptr,int size,int nitems, FILE *stream));
extern int fwrite PROTO_LIST((char *ptr,int size,int nitems, FILE *stream));
extern int fseek PROTO_LIST((FILE *stream,long offset, int ptrname));
extern int _flsbuf PROTO_LIST((unsigned char ,...));
extern int fflush PROTO_LIST((FILE *stream));
extern int fprintf PROTO_LIST((FILE *,char *,...));
extern int vfprintf PROTO_LIST((FILE*, char*, ...));     
extern int vsprintf PROTO_LIST((char*, char*, ...));     
extern int printf PROTO_LIST((char *,...));
extern int fclose PROTO_LIST((FILE *stream));
extern int fputc PROTO_LIST((char c, FILE *stream));
extern int fgetc PROTO_LIST((FILE *stream));
extern int ungetc PROTO_LIST((char c, FILE *stream));
extern int sscanf PROTO_LIST((char *s, char *format, ...));
extern int fscanf PROTO_LIST((FILE *s, char *format, ...));  
extern int strcasecmp PROTO_LIST((char *,char *));
extern int strncasecmp PROTO_LIST((char *,char *,int ));
extern time_t time PROTO_LIST((time_t *));
#endif










/*  this pseudo-cache state is used just in this demo app, since for
 *  demo purposes I don't want to maintain a seperate application-level
 *  database  */
static struct cache_state_ DCAPAMT[] = 
    { { "demo_capAmt", &asn1set, ASN1SET__CURRENCYAMOUNT } };
static struct cache_state_ DCAPREVAMT[] = 
    { { "demo_capRevAmt", &asn1set, ASN1SET__CURRENCYAMOUNT } };

#define TWO_MONTHS         (60*60*24*31*2)    /* 2 months worth of seconds */

static char   *ACQ_MSG           = 0;         /* gateway->cardholder message */
static char   *ENGLISH           = "en";      /* RFC-1766 language code */
static char   *BRANDID           = "Terisa Test Brand:Product Type";
static char   *ORDER_DESC        = "order description";
static char   *LOCALID_C         = "lidc-lidc-lidc-lidc-";
static char   *LOCALID_M         = "lidm-lidm-lidm-lidm-";
static char   *LOCALID_EE        = "lidee-lidee-lidee-li";
static char   *LOCALID_CA        = "lidca-lidca-lidca-li";
static char   *POLICY_TEXT       = "policy text";
static char   *LOGO_URL          = "http://www.terisa.com/media/logo.big.gif";
static char   *XID               = "xid-xid-xid-xid-xid-";
static char   *BIN               = "111122";
static char   *PAN               = "1111222233334444";
static char   *MERCHANTBIN       = "987654";
static char   *MERCHANTID        = "MerchantID";
static int     REGFORMID         = 56789;
static char   *CARDEXPIRY        = "199901";
static char   *PANSECRET         = "pansecretpansecretpa";

static int     ERROR             = 1;      /* "unspecifiedFailure" */
static int     US_DOLLAR         = 840;    /* ISO-4217 currency code for US$ */
static int     AUTHAMT           = 2995;
static int     CAPAMT            = 2995;
static int     PURCHAMT          = 2995;
static int     EXPONENT          = -2;
static int     TOTAL_TRANS       = 2;      /* total # of recurring payments */
static int     CERT_REQ_TYPE     = asn1set__RequestType__cardInitialSig;

static int     USE_AUTHTOKEN     = 0;
static int     USE_SIGNATURE     = 0;
static int     USE_LID_M         = 0;
static int     USE_CAPTOKEN      = 0;
static int     USE_PANTOKEN      = 0;
static int     USE_RECURRING     = 0;
static int     USE_REFERRAL      = 0;

static int     SHOW_ACTION       = 1;

static char   *CARDHOLDER_SW     = "Terisa Demo Cardholder Software v1.0";
static char   *MERCHANT_SW       = "Terisa Demo Merchant Software v1.0";
static char   *GATEWAY_SW        = "Terisa Demo Payment Gateway v1.0";
static char   *CCA_SW            = "Terisa Demo CCA Software v1.0";
static char   *MCA_SW            = "Terisa Demo MCA Software v1.0";
static char   *PCA_SW            = "Terisa Demo PCA Software v1.0";
static char   *CARDHOLDER_LOG    = "LOG.car";
static char   *MERCHANT_LOG      = "LOG.mer";
static char   *GATEWAY_LOG       = "LOG.pay";
static char   *CCA_LOG           = "LOG.cca";
static char   *MCA_LOG           = "LOG.mca";
static char   *PCA_LOG           = "LOG.pca";





static io_ctx             instream = 0;
static io_ctx             outstream = 0;
static log_ctx            history;
static set_app_ctx        app_ctx = 0;
static alloc_ctx          heap = 0;
static int                status;
static cache_ctx          msg_cache[16] = { 0 };
static asn1set__SWIdent   swIdent;

/* TEMPORARY HACK FOR TESTING */
char *suffix = 0;
int DB_ROLE PROTO_LIST((char *s));
int DB_ROLE(s)char* s;{suffix=s;return 0;}

int tohex PROTO_LIST((UCHAR *binary, int binarylen, char *hex));
int tohex(binary, binarylen, hex)
    UCHAR       *binary;
    int          binarylen;
    char        *hex;
{
    int          i;

    for (i = 0; i < binarylen; ++i) {
        sprintf((char*)&hex[i*2], "%02x", binary[i]);
    }
    return 0;
}

char *lid2filename PROTO_LIST((asn1set__LocalID *lid));
char *lid2filename(lid)
    asn1set__LocalID  *lid;
{
    static char  filename[80];
    tohex(lid->value, lid->length, filename);
    if(suffix)strcat(filename, suffix);
    return filename;
}

char *rrpid2filename PROTO_LIST((asn1set__RRPID *rrpid));
char *rrpid2filename(rrpid)
    asn1set__RRPID    *rrpid;
{
    static char  filename[80];
    tohex(rrpid->value, rrpid->length, filename);
    if(suffix)strcat(filename, suffix);
    return filename;
}

char *xid2filename PROTO_LIST((asn1set__XID *xid));
char *xid2filename(xid)
    asn1set__XID      *xid;
{
    static char  filename[80];
    tohex(xid->value, xid->length, filename);
    if(suffix)strcat(filename, suffix);
    return filename;
}

int callback PROTO_LIST((VOID *callback_ctx, int reason));
int callback(callback_ctx, reason)
    VOID *callback_ctx;
    int reason;
{
    return NO_ERROR;
}







#  define SET_SELF_ID_USING_SIG_DN(id, dn)  (*id=dn)
#  define SET_PEER_ID_USING_KEY_DN(id, dn)  (*id=dn)

int SET_SELF_ID_car_sig PROTO_LIST((cert_identity*));
int SET_SELF_ID_car_sig (id)
    cert_identity       *id;
{
    static asn__any     dn;
    dn.length = len_card_s_dn_der;
    dn.object = card_s_dn_der;
    SET_SELF_ID_USING_SIG_DN(id, &dn);
    return 0;
}
int SET_SELF_ID_mer_sig PROTO_LIST((cert_identity*));
int SET_SELF_ID_mer_sig (id)
    cert_identity       *id;
{
    static asn__any     dn;
    dn.length = len_mer_s_dn_der;
    dn.object = mer_s_dn_der;
    SET_SELF_ID_USING_SIG_DN(id, &dn);
    return 0;
}
int SET_SELF_ID_pay_sig PROTO_LIST((cert_identity*));
int SET_SELF_ID_pay_sig (id)
    cert_identity       *id;
{
    static asn__any     dn;
    dn.length = len_pgwy_s_dn_der;
    dn.object = pgwy_s_dn_der;
    SET_SELF_ID_USING_SIG_DN(id, &dn);
    return 0;
}
int SET_SELF_ID_cca_sig PROTO_LIST((cert_identity*));
int SET_SELF_ID_cca_sig (id)
    cert_identity       *id;
{
    static asn__any     dn;
    dn.length = len_cca_s_dn_der;
    dn.object = cca_s_dn_der;
    SET_SELF_ID_USING_SIG_DN(id, &dn);
    return 0;
}
int SET_SELF_ID_mca_sig PROTO_LIST((cert_identity*));
int SET_SELF_ID_mca_sig (id)
    cert_identity       *id;
{
    static asn__any     dn;
    dn.length = len_mca_s_dn_der;
    dn.object = mca_s_dn_der;
    SET_SELF_ID_USING_SIG_DN(id, &dn);
    return 0;
}
int SET_SELF_ID_pca_sig PROTO_LIST((cert_identity*));
int SET_SELF_ID_pca_sig (id)
    cert_identity       *id;
{
    static asn__any     dn;
    dn.length = len_pca_s_dn_der;
    dn.object = pca_s_dn_der;
    SET_SELF_ID_USING_SIG_DN(id, &dn);
    return 0;
}
int SET_PEER_ID_pay_key PROTO_LIST((cert_identity*));
int SET_PEER_ID_pay_key (id)
    cert_identity       *id;
{
    static asn__any     dn;
    dn.length = len_pgwy_k_dn_der;
    dn.object = pgwy_k_dn_der;
    SET_PEER_ID_USING_KEY_DN(id, &dn);
    return 0;
}




#define ROLE_CARDHOLDER    1
#define ROLE_MERCHANT      2
#define ROLE_GATEWAY       3
#define ROLE_CCA           4
#define ROLE_MCA           5
#define ROLE_PCA           6
int CURRENT_ROLE PROTO_LIST((int role));
int CURRENT_ROLE(role)
    int role;
{
    extern int DB_ROLE PROTO_LIST((char*));

    /* The demo code plays the part of the cardholder, merchant, and
       gateway software.  This hack is used to "switch hit" between
       these roles. */

    switch (role) {
    case ROLE_CARDHOLDER:
        DB_ROLE(".car");
        break;

    case ROLE_MERCHANT:
        DB_ROLE(".mer");
        break;

    case ROLE_GATEWAY:
        DB_ROLE(".pay");
        break;

    case ROLE_CCA:
        DB_ROLE(".cca");
        break;

    case ROLE_MCA:
        DB_ROLE(".mca");
        break;

    case ROLE_PCA:
        DB_ROLE(".pca");
        break;
    }
    return 0;
}


int startup PROTO_LIST((int role, char *sw, char *log));
int startup(role, sw, log)
    int    role;
    char  *sw;
    char  *log;
{
    CURRENT_ROLE(role);

    swIdent = sw;

    status = create_log_ctx_db(&history, log);
    assert(status == NO_ERROR);

    status = create_set_app_ctx(&app_ctx, &swIdent, history);
    assert(status == NO_ERROR);

    /* this is a demo program, so big gaps between messages is permitted */
    status = modify_set_app_ctx(app_ctx, SET_APP_CTX_MODIFY_MAX_CLOCK_SKEW, TWO_MONTHS);
    assert(status == NO_ERROR);

    return NO_ERROR;
}

int finish PROTO_LIST((VOID));
int
finish()
{
    status = delete_set_app_ctx(&app_ctx);
    assert(status == NO_ERROR);

    status = LOG_delete(&history);
    assert(status == NO_ERROR);

    return NO_ERROR;
}



int read_Error PROTO_LIST((VOID));
int
read_Error()
{
    set_msg                   error = 0;
    int                       msg_type;

    if (SHOW_ACTION) fprintf(stderr,"read_Error\n");

    /* for a demo Error, it doesn't much matter which startup is used */
    status = startup(ROLE_CARDHOLDER, CARDHOLDER_SW, CARDHOLDER_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&error, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get message */
    status = receive_set_msg(error, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);

    status = decode_set_msg(error, app_ctx);
    assert(status == NO_ERROR);

    if (SHOW_ACTION) {
        fprintf(stderr, "SET Error message contained error code: %ld\n",
           safe_cast_Error(error)->errorTBS.errorCode);
    }

    status = delete_set_msg(&error);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}


int write_Error PROTO_LIST((VOID));
int
write_Error()
{
    set_msg                   msg = 0;
    Error                     error = 0;
    int                       msg_type;
    char                     *cache_name;
    UCHAR                     buffer[20000];

    /*  first, the last message read in instream is re-read
     *  second, an error based on that message is sent
     */

    if (SHOW_ACTION) fprintf(stderr,"write_Error\n");

    /* for a demo Error, it doesn't much matter which startup is used */
    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* since the last message in is what caused the error, instream
     * is used to find the message that generated the error */
    assert(instream != 0);
    assert(IO_fseek(instream, 0, FSEEK_BOF) != -1);

    status = create_set_msg(&msg, app_ctx, heap);
    assert(status == NO_ERROR);
    status = receive_set_msg(msg, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);

    /* if XID isn't present, then just fail (this is a simple demo app) */
    assert(IS_PRESENT(msg->messageWrapper.messageHeader.messageIDs,
                      asn1set__MessageIDs__xID));
    cache_name = xid2filename(&msg->messageWrapper.messageHeader.messageIDs.xID);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);
    status = CACHE_put(msg_cache[0], STATE_ERROR_RRPID, &msg->messageWrapper.messageHeader.rrpid);
    assert(status == NO_ERROR);

    status = delete_set_msg(&msg);
    assert(status == NO_ERROR);

    /* build Error message */
    status = create_set_msg(&msg, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__error;
    status = make_set_msg(msg, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    error = safe_cast_Error(msg);

    error->errorTBS.errorCode = ERROR;

    /* since the last message in is what caused the error, instream
     * is used to find the message that generated the error */
    assert(instream != 0);
    assert(IO_fseek(instream, 0, FSEEK_BOF) != -1);
    error->errorTBS.errorMsg.u.badWrapper.length =
                           IO_fread((char*)buffer, 1, sizeof(buffer), instream);
    assert(error->errorTBS.errorMsg.u.badWrapper.length > 0);
    error->errorTBS.errorMsg.u.badWrapper.value = buffer;
    error->errorTBS.errorMsg._choice = asn1set__ErrorMsg__badWrapper;

    /* send Error */
    status = send_set_msg(msg, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = delete_set_msg(&msg);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return 0;
}


int ee_reads_Me_AqCInitRes PROTO_LIST((asn1set__LocalID*));
int
ee_reads_Me_AqCInitRes(lidee)
    asn1set__LocalID         *lidee;        /* OUT */
{
    set_msg                   response = 0;
    Me_AqCInitRes             me_AqCInitRes = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"ee_reads_Me_AqCInitRes\n");

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__cardRenewalSig:
        fprintf(stderr,"bad cert request type (cardholder)\n");
        exit(1);
        break;
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__merRenewalBoth:
        status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
        assert(status == NO_ERROR);
        break;
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__pgwyRenewalBoth:
        status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
        assert(status == NO_ERROR);
        break;
    default:
        fprintf(stderr,"bad cert request type\n");
        exit(1);
    }

    /* create a blank SET message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get Me_AqCInitRes */
    status = receive_set_msg(response, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__me_AqCInitRes);

    status = decode_set_msg(response, app_ctx);
    assert(status == NO_ERROR);

    me_AqCInitRes = safe_cast_Me_AqCInitRes(response);
    assert(me_AqCInitRes != 0);

    /* copy the LocalID for use later */
    COPY_OCTET_STRING(heap, lidee, &me_AqCInitRes->me_AqCInitResTBS.lid_EE);

    /* open the cache associated with the LocalID in the message */
    cache_name = lid2filename(lidee);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(response, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process Me_AqCInitRes */
    /* ... */

    if (SHOW_ACTION) {
        fprintf(stderr, "RequestType: %ld\n",
                me_AqCInitRes->me_AqCInitResTBS.requestType);
    }

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int ca_writes_Me_AqCInitRes PROTO_LIST((asn1set__LocalID*));
int
ca_writes_Me_AqCInitRes(lidca)
    asn1set__LocalID         *lidca;         /* IN */
{
    set_msg                   response = 0;
    Me_AqCInitRes             me_AqCInitRes = 0;
    int                       msg_type;
    char                     *cache_name;
    asn__any                  cert;
    UCHAR                    *ee_k_cert_der;
    int                       len_ee_k_cert_der;
    cert_identity             sigCID;


    if (SHOW_ACTION) fprintf(stderr,"ca_writes_Me_AqCInitRes\n");

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__cardRenewalSig:
        fprintf(stderr,"bad cert request type (cardholder)\n");
        exit(1);
        break;
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__merRenewalBoth:
        status = startup(ROLE_MCA, MCA_SW, MCA_LOG);
        assert(status == NO_ERROR);
        len_ee_k_cert_der = len_mca_k_cert_der;
        ee_k_cert_der = mca_k_cert_der;
        SET_SELF_ID_mca_sig(&sigCID);
        break;
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__pgwyRenewalBoth:
        status = startup(ROLE_PCA, PCA_SW, PCA_LOG);
        assert(status == NO_ERROR);
        len_ee_k_cert_der = len_pca_k_cert_der;
        ee_k_cert_der = pca_k_cert_der;
        SET_SELF_ID_pca_sig(&sigCID);
        break;
    default:
        fprintf(stderr,"bad cert request type\n");
        exit(1);
    }

    /* open the cache associated with the appropriate transaction */
    cache_name = lid2filename(lidca);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build Me_AqCInitRes message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__me_AqCInitRes;
    status = make_set_msg(response, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    me_AqCInitRes = safe_cast_Me_AqCInitRes(response);
    assert(me_AqCInitRes != 0);

    /* insert user-layer data */

    cert.length = len_ee_k_cert_der;
    cert.object = ee_k_cert_der;
    status = build_CertThumb(app_ctx, heap, &cert,
                             &me_AqCInitRes->me_AqCInitResTBS.caeThumb);
    assert(status == NO_ERROR);

    me_AqCInitRes->caSignatureCID = sigCID;

    me_AqCInitRes->me_AqCInitResTBS.regFormOrReferral._choice = asn1set__RegFormOrReferral__regFormData;
    SET_PRESENT(me_AqCInitRes->me_AqCInitResTBS.regFormOrReferral.u.regFormData, asn1set__RegFormData__regTemplate);
    me_AqCInitRes->me_AqCInitResTBS.regFormOrReferral.u.regFormData.policy.u.visibleString = POLICY_TEXT;
    me_AqCInitRes->me_AqCInitResTBS.regFormOrReferral.u.regFormData.policy._choice = asn1set__PolicyText__visibleString;

    me_AqCInitRes->me_AqCInitResTBS.regFormOrReferral.u.regFormData.regTemplate.regFormID = REGFORMID;
    me_AqCInitRes->me_AqCInitResTBS.regFormOrReferral.u.regFormData.regTemplate.brandLogoURL = LOGO_URL;

    me_AqCInitRes->me_AqCInitResTBS.regFormOrReferral.u.regFormData.regTemplate.regFieldSeq =
         (asn1set__RegFieldSeq*)AC_calloc(heap, 1, sizeof(asn1set__RegFieldSeq));
    me_AqCInitRes->me_AqCInitResTBS.regFormOrReferral.u.regFormData.regTemplate.regFieldSeq->regField.fieldName.u.
         visibleString = "registration template field name 1";
    me_AqCInitRes->me_AqCInitResTBS.regFormOrReferral.u.regFormData.regTemplate.regFieldSeq->regField.fieldName._choice = 
         asn1set__FieldName__visibleString;

    /* send Me_AqCInitRes */
    status = send_set_msg(response, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}



int ca_reads_Me_AqCInitReq PROTO_LIST((asn1set__LocalID*));
int
ca_reads_Me_AqCInitReq(lidca)
    asn1set__LocalID         *lidca;         /* OUT */
{
    set_msg                   request = 0;
    Me_AqCInitReq             me_AqCInitReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"ca_reads_Me_AqCInitReq\n");

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__cardRenewalSig:
        fprintf(stderr,"bad cert request type (cardholder)\n");
        exit(1);
        break;
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__merRenewalBoth:
        status = startup(ROLE_MCA, MCA_SW, MCA_LOG);
        assert(status == NO_ERROR);
        break;
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__pgwyRenewalBoth:
        status = startup(ROLE_PCA, PCA_SW, PCA_LOG);
        assert(status == NO_ERROR);
        break;
    default:
        fprintf(stderr,"bad cert request type\n");
        exit(1);
    }

    /* first, a LocalID-CA must be assigned */
    lidca->length = strlen(LOCALID_CA);
    lidca->value = AC_malloc(heap, lidca->length);
                       /* this should be random */
    memcpy(lidca->value, LOCALID_CA, lidca->length);

    /* create the cache to be associated with this transaction */
    cache_name = lid2filename(lidca);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get Me_AqCInitReq */
    status = receive_set_msg(request, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__me_AqCInitReq);

    status = decode_set_msg(request, app_ctx);
    assert(status == NO_ERROR);

    status = verify_set_msg(request, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    me_AqCInitReq = safe_cast_Me_AqCInitReq(request);
    assert(me_AqCInitReq != 0);

    /* process Me_AqCInitReq */
    /* ... */

    if (SHOW_ACTION) {
        fprintf(stderr, "RequestType: %ld\n",
            me_AqCInitReq->messageWrapper.message.u.me_AqCInitReq.requestType);
    }

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}



int ee_writes_Me_AqCInitReq PROTO_LIST((asn1set__LocalID*));
int
ee_writes_Me_AqCInitReq(lidee)
    asn1set__LocalID       *lidee;        /* OUT */
{
    set_msg                 request  = 0;
    Me_AqCInitReq           me_AqCInitReq = 0;
    int                     msg_type;
    char                   *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"ee_writes_Me_AqCInitReq\n");

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__cardRenewalSig:
        fprintf(stderr,"bad cert request type (cardholder)\n");
        exit(1);
        break;
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__merRenewalBoth:
        status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
        assert(status == NO_ERROR);
        break;
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__pgwyRenewalBoth:
        status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
        assert(status == NO_ERROR);
        break;
    default:
        fprintf(stderr,"bad cert request type\n");
        exit(1);
    }

    /* first, a LocalID must be assigned */
    lidee->length = strlen(LOCALID_EE);
    lidee->value = AC_malloc(heap, lidee->length);
    memcpy(lidee->value, LOCALID_EE, lidee->length);

    /* second, the cache to be associated with this LocalID is created */
    cache_name = lid2filename(lidee);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* third, the LocalID is placed in the cache */
    status = CACHE_put(msg_cache[0], STATE_LOCALID_EE, lidee);
    assert(status == NO_ERROR);

    /* build Me_AqCInitReq message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__me_AqCInitReq;
    status = make_set_msg(request, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    me_AqCInitReq = safe_cast_Me_AqCInitReq(request);
    assert(me_AqCInitReq != 0);

    /* insert user-layer data */

    me_AqCInitReq->messageWrapper.message.u.me_AqCInitReq.language = ENGLISH;

    me_AqCInitReq->messageWrapper.message.u.me_AqCInitReq.brandID.u.visibleString =
                                                    BRANDID;
    me_AqCInitReq->messageWrapper.message.u.me_AqCInitReq.brandID._choice =
                                           asn1set__BrandID__visibleString;

    me_AqCInitReq->messageWrapper.message.u.me_AqCInitReq.requestType = 
                                                                 CERT_REQ_TYPE;

    me_AqCInitReq->messageWrapper.message.u.me_AqCInitReq.idData.u.
                merchantAcquirerID.merchantBIN = MERCHANTBIN;

    me_AqCInitReq->messageWrapper.message.u.me_AqCInitReq.idData.u.
                merchantAcquirerID.merchantID.u.visibleString = MERCHANTID;
    me_AqCInitReq->messageWrapper.message.u.me_AqCInitReq.idData.u.
                merchantAcquirerID.merchantID._choice =
                                         asn1set__MerchantID__visibleString;

    me_AqCInitReq->messageWrapper.message.u.me_AqCInitReq.idData._choice =
                                         asn1set__IDData__merchantAcquirerID;

    /* send Me_AqCInitReq */
    status = send_set_msg(request, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int ee_reads_CertInqRes PROTO_LIST((asn1set__LocalID*));
int
ee_reads_CertInqRes(lidee)
    asn1set__LocalID         *lidee;        /* OUT */
{
    set_msg                   response = 0;
    CertInqRes                certInqRes = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"ee_reads_CertInqRes\n");

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__cardRenewalSig:
        status = startup(ROLE_CARDHOLDER, CARDHOLDER_SW, CARDHOLDER_LOG);
        assert(status == NO_ERROR);
        break;
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__merRenewalBoth:
        status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
        assert(status == NO_ERROR);
        break;
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__pgwyRenewalBoth:
        status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
        assert(status == NO_ERROR);
        break;
    default:
        fprintf(stderr,"bad cert request type\n");
        exit(1);
    }

    /* create a blank SET message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get CertInqRes */
    status = receive_set_msg(response, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__certInqRes);

    status = decode_set_msg(response, app_ctx);
    assert(status == NO_ERROR);

    certInqRes = safe_cast_CertInqRes(response);
    assert(certInqRes != 0);

    /* copy the LocalID for use later */
    COPY_OCTET_STRING(heap, lidee, &certInqRes->certResData.lid_EE);

    /* open the cache associated with the LocalID in the message */
    cache_name = lid2filename(lidee);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(response, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process CertInqRes */
    /* ... */

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    if (SHOW_ACTION) fprintf(stderr, "Done....\n");

    return NO_ERROR;
}




int ca_writes_CertInqRes PROTO_LIST((asn1set__LocalID*));
int
ca_writes_CertInqRes(lidca)
    asn1set__LocalID         *lidca;         /* IN */
{
    set_msg                   response = 0;
    CertInqRes                certInqRes = 0;
    int                       msg_type;
    char                     *cache_name;
    int                       len_ee_s_cert_der;
    UCHAR                    *ee_s_cert_der;
    int                       len_ee_k_cert_der;
    UCHAR                    *ee_k_cert_der;


    if (SHOW_ACTION) fprintf(stderr,"ca_writes_CertInqRes\n");

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__cardRenewalSig:
        status = startup(ROLE_CCA, CCA_SW, CCA_LOG);
        assert(status == NO_ERROR);
        len_ee_s_cert_der = len_card_s_cert_der;
        ee_s_cert_der = card_s_cert_der;
        len_ee_k_cert_der = 0;
        ee_k_cert_der = 0;
        break;
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__merRenewalBoth:
        status = startup(ROLE_MCA, MCA_SW, MCA_LOG);
        assert(status == NO_ERROR);
        len_ee_s_cert_der = len_mer_s_cert_der;
        ee_s_cert_der = mer_s_cert_der;
        len_ee_k_cert_der = len_mer_k_cert_der;
        ee_k_cert_der = mer_k_cert_der;
        break;
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__pgwyRenewalBoth:
        status = startup(ROLE_PCA, PCA_SW, PCA_LOG);
        assert(status == NO_ERROR);
        len_ee_s_cert_der = len_pgwy_s_cert_der;
        ee_s_cert_der = pgwy_s_cert_der;
        len_ee_k_cert_der = len_pgwy_k_cert_der;
        ee_k_cert_der = pgwy_k_cert_der;
        break;
    default:
        fprintf(stderr,"bad cert request type\n");
        exit(1);
    }

    /* open the cache associated with the appropriate transaction */
    cache_name = lid2filename(lidca);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build CertInqRes message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__certInqRes;
    status = make_set_msg(response, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    certInqRes = safe_cast_CertInqRes(response);
    assert(certInqRes != 0);

    /* insert user-layer data */

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__cardRenewalSig:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__merRenewalBoth:
    case asn1set__RequestType__pgwyRenewalBoth:
        certInqRes->signatureCertificate.length = len_ee_s_cert_der;
        certInqRes->signatureCertificate.object = ee_s_cert_der;
        SET_PRESENT(*certInqRes, CertRes__signatureCertificate);
        break;
    }

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__merRenewalBoth:
    case asn1set__RequestType__pgwyRenewalBoth:
        certInqRes->keyExchangeCertificate.length = len_ee_k_cert_der;
        certInqRes->keyExchangeCertificate.object = ee_k_cert_der;
        SET_PRESENT(*certInqRes, CertRes__keyExchangeCertificate);
        break;
    }

    /*  because hardcoded certs are being used, we cannot
     *  use a random nonceCCA, but rather we must use
     *  a hardcoded nonceCCA 
     *
     *  this is incorrect, but is used here because
     *  of the hardcoded certificate  */
    if (IS_PRESENT(certInqRes->certResData.certStatus, asn1set__CertStatus__nonceCCA))
    {
        /* always have the panSecret turn out to be "pansecretpansecretpa" */
        /* 1. subtract NonceCCA from PANSecret to obtain CardSecret */
        int     i;
        UCHAR   cardsecret[40];

        for (i = 0; i < (int)certInqRes->certResData.certStatus.nonceCCA.length; ++i)
        {
            cardsecret[i] = certInqRes->panSecret.value[i]
                          ^ certInqRes->certResData.certStatus.nonceCCA.value[i];
        }
        /* 2. add desired PANSecret to CardSecret to obtain new NonceCCA */ 
        for (i = 0; i < (int)certInqRes->certResData.certStatus.nonceCCA.length; ++i)
        {
            certInqRes->certResData.certStatus.nonceCCA.value[i] =
                                                cardsecret[i] ^ PANSECRET[i];
        }
        /* 3. copy desired PANSecret into message */
        memcpy(certInqRes->panSecret.value, PANSECRET, strlen(PANSECRET));
    }

    certInqRes->certResData.certStatus.certStatusCode = asn1set__CertStatusCode__requestComplete;



    /* send CertInqRes */
    status = send_set_msg(response, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int ca_reads_CertInqReq PROTO_LIST((asn1set__LocalID*));
int
ca_reads_CertInqReq(lidca)
    asn1set__XID             *lidca;         /* OUT */
{
    set_msg                   request = 0;
    CertInqReq                certInqReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"ca_reads_CertInqReq\n");

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__cardRenewalSig:
        status = startup(ROLE_CCA, CCA_SW, CCA_LOG);
        assert(status == NO_ERROR);
        break;
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__merRenewalBoth:
        status = startup(ROLE_MCA, MCA_SW, MCA_LOG);
        assert(status == NO_ERROR);
        break;
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__pgwyRenewalBoth:
        status = startup(ROLE_PCA, PCA_SW, PCA_LOG);
        assert(status == NO_ERROR);
        break;
    default:
        fprintf(stderr,"bad cert request type\n");
        exit(1);
    }

    /* create a blank SET message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get CertInqReq */
    status = receive_set_msg(request, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__certInqReq);

    status = decode_set_msg(request, app_ctx);
    assert(status == NO_ERROR);

    certInqReq = safe_cast_CertInqReq(request);
    assert(certInqReq != 0);

    /* copy the lid */
    COPY_OCTET_STRING(heap, lidca, &certInqReq->certInqReqTBS.lid_CA);

    /* create the cache to be associated with this cert request */
    cache_name = lid2filename(lidca);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(request, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process CertInqReq */
    /* ... */

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int ee_writes_CertInqReq PROTO_LIST((asn1set__LocalID*));
int
ee_writes_CertInqReq(lidee)
    asn1set__LocalID       *lidee;        /* IN */
{
    set_msg                 request  = 0;
    CertInqReq              certInqReq = 0;
    int                     msg_type;
    char                   *cache_name;
    cert_identity           sigCID;
    int                     len_ee_s_priv_der;
    UCHAR                  *ee_s_priv_der = 0;

    if (SHOW_ACTION) fprintf(stderr,"ee_writes_CertInqReq\n");

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__cardRenewalSig:
        status = startup(ROLE_CARDHOLDER, CARDHOLDER_SW, CARDHOLDER_LOG);
        assert(status == NO_ERROR);
        SET_SELF_ID_car_sig(&sigCID);
        len_ee_s_priv_der = len_card_s_priv_der;
        ee_s_priv_der = card_s_priv_der;
        break;
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__merRenewalBoth:
        status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
        assert(status == NO_ERROR);
        SET_SELF_ID_mer_sig(&sigCID);
        len_ee_s_priv_der = len_mer_s_priv_der;
        ee_s_priv_der = mer_s_priv_der;
        break;
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__pgwyRenewalBoth:
        status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
        assert(status == NO_ERROR);
        SET_SELF_ID_pay_sig(&sigCID);
        len_ee_s_priv_der = len_pgwy_s_priv_der;
        ee_s_priv_der = pgwy_s_priv_der;
        break;
    default:
        fprintf(stderr,"bad cert request type\n");
        exit(1);
    }

    /* the cache to be associated with this LocalID is opened */
    cache_name = lid2filename(lidee);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build CertInqReq message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__certInqReq;
    status = make_set_msg(request, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    certInqReq = safe_cast_CertInqReq(request);
    assert(certInqReq != 0);

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__pgwyInitialBoth:
        /* no previous cert is available */
        break;

    case asn1set__RequestType__cardRenewalSig:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__merRenewalBoth:
    case asn1set__RequestType__pgwyRenewalBoth:
        certInqReq->eeSignatureCID = sigCID;
        SET_PRESENT(*certInqReq, CertInqReq__eeSignatureCID);
        break;
    }

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__cardRenewalSig:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__merRenewalBoth:
    case asn1set__RequestType__pgwyRenewalBoth:

        certInqReq->eeSignatureKey.length = len_ee_s_priv_der;
        certInqReq->eeSignatureKey.object = ee_s_priv_der;
        SET_PRESENT(*certInqReq, CertInqReq__eeSignatureKey);
        break;
    }

    /* send CertInqReq */
    status = send_set_msg(request, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}



int ee_reads_CertRes PROTO_LIST((asn1set__LocalID*));
int
ee_reads_CertRes(lidee)
    asn1set__LocalID         *lidee;        /* OUT */
{
    set_msg                   response = 0;
    CertRes                   certRes = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"ee_reads_CertRes\n");

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__cardRenewalSig:
        status = startup(ROLE_CARDHOLDER, CARDHOLDER_SW, CARDHOLDER_LOG);
        assert(status == NO_ERROR);
        break;
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__merRenewalBoth:
        status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
        assert(status == NO_ERROR);
        break;
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__pgwyRenewalBoth:
        status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
        assert(status == NO_ERROR);
        break;
    default:
        fprintf(stderr,"bad cert request type\n");
        exit(1);
    }

    /* create a blank SET message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get CertRes */
    status = receive_set_msg(response, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__certRes);

    status = decode_set_msg(response, app_ctx);
    assert(status == NO_ERROR);

    certRes = safe_cast_CertRes(response);
    assert(certRes != 0);

    /* copy the LocalID for use later */
    COPY_OCTET_STRING(heap, lidee, &certRes->certResData.lid_EE);

    /* open the cache associated with the LocalID in the message */
    cache_name = lid2filename(lidee);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(response, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process CertRes */
    /* ... */

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    if (SHOW_ACTION) fprintf(stderr, "Done....\n");

    return NO_ERROR;
}




int ca_writes_CertRes PROTO_LIST((asn1set__LocalID*));
int
ca_writes_CertRes(lidca)
    asn1set__LocalID         *lidca;         /* IN */
{
    set_msg                   response = 0;
    CertRes                   certRes = 0;
    int                       msg_type;
    char                     *cache_name;
    int                       len_ee_s_cert_der;
    UCHAR                    *ee_s_cert_der;
    int                       len_ee_k_cert_der;
    UCHAR                    *ee_k_cert_der;


    if (SHOW_ACTION) fprintf(stderr,"ca_writes_CertRes\n");

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__cardRenewalSig:
        status = startup(ROLE_CCA, CCA_SW, CCA_LOG);
        assert(status == NO_ERROR);
        len_ee_s_cert_der = len_card_s_cert_der;
        ee_s_cert_der = card_s_cert_der;
        len_ee_k_cert_der = 0;
        ee_k_cert_der = 0;
        break;
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__merRenewalBoth:
        status = startup(ROLE_MCA, MCA_SW, MCA_LOG);
        assert(status == NO_ERROR);
        len_ee_s_cert_der = len_mer_s_cert_der;
        ee_s_cert_der = mer_s_cert_der;
        len_ee_k_cert_der = len_mer_k_cert_der;
        ee_k_cert_der = mer_k_cert_der;
        break;
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__pgwyRenewalBoth:
        status = startup(ROLE_PCA, PCA_SW, PCA_LOG);
        assert(status == NO_ERROR);
        len_ee_s_cert_der = len_pgwy_s_cert_der;
        ee_s_cert_der = pgwy_s_cert_der;
        len_ee_k_cert_der = len_pgwy_k_cert_der;
        ee_k_cert_der = pgwy_k_cert_der;
        break;
    default:
        fprintf(stderr,"bad cert request type\n");
        exit(1);
    }

    /* open the cache associated with the appropriate transaction */
    cache_name = lid2filename(lidca);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build CertRes message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__certRes;
    status = make_set_msg(response, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    certRes = safe_cast_CertRes(response);
    assert(certRes != 0);

    /* insert user-layer data */

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__cardRenewalSig:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__merRenewalBoth:
    case asn1set__RequestType__pgwyRenewalBoth:
        certRes->signatureCertificate.length = len_ee_s_cert_der;
        certRes->signatureCertificate.object = ee_s_cert_der;
        SET_PRESENT(*certRes, CertRes__signatureCertificate);
        break;
    }

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__merRenewalBoth:
    case asn1set__RequestType__pgwyRenewalBoth:
        certRes->keyExchangeCertificate.length = len_ee_k_cert_der;
        certRes->keyExchangeCertificate.object = ee_k_cert_der;
        SET_PRESENT(*certRes, CertRes__keyExchangeCertificate);
        break;
    }

    /*  because hardcoded certs are being used, we cannot
     *  use a random nonceCCA, but rather we must use
     *  a hardcoded nonceCCA 
     *
     *  this is incorrect, but is used here because
     *  of the hardcoded certificate  */
    if (IS_PRESENT(certRes->certResData.certStatus, asn1set__CertStatus__nonceCCA))
    {
        /* always have the panSecret turn out to be "pansecretpansecretpa" */
        /* 1. subtract NonceCCA from PANSecret to obtain CardSecret */
        int     i;
        UCHAR   cardsecret[40];

        for (i = 0; i < (int)certRes->certResData.certStatus.nonceCCA.length; ++i)
        {
            cardsecret[i] = certRes->panSecret.value[i]
                          ^ certRes->certResData.certStatus.nonceCCA.value[i];
        }
        /* 2. add desired PANSecret to CardSecret to obtain new NonceCCA */ 
        for (i = 0; i < (int)certRes->certResData.certStatus.nonceCCA.length; ++i)
        {
            certRes->certResData.certStatus.nonceCCA.value[i] =
                                                cardsecret[i] ^ PANSECRET[i];
        }
        /* 3. copy desired PANSecret into message */
        memcpy(certRes->panSecret.value, PANSECRET, strlen(PANSECRET));
    }

    certRes->certResData.certStatus.certStatusCode = asn1set__CertStatusCode__requestComplete;



    /* send CertRes */
    status = send_set_msg(response, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int ca_reads_CertReq PROTO_LIST((asn1set__LocalID*));
int
ca_reads_CertReq(lidca)
    asn1set__LocalID         *lidca;         /* OUT */
{
    set_msg                   request = 0;
    CertReq                   certReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"ca_reads_CertReq\n");

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__cardRenewalSig:
        status = startup(ROLE_CCA, CCA_SW, CCA_LOG);
        assert(status == NO_ERROR);
        break;
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__merRenewalBoth:
        status = startup(ROLE_MCA, MCA_SW, MCA_LOG);
        assert(status == NO_ERROR);
        break;
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__pgwyRenewalBoth:
        status = startup(ROLE_PCA, PCA_SW, PCA_LOG);
        assert(status == NO_ERROR);
        break;
    default:
        fprintf(stderr,"bad cert request type\n");
        exit(1);
    }

    /* create a blank SET message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get CertReq */
    status = receive_set_msg(request, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__certReq);

    status = decode_set_msg(request, app_ctx);
    assert(status == NO_ERROR);

    certReq = safe_cast_CertReq(request);
    assert(certReq != 0);

    /* either copy the LocalID for use later, or create LocalID */
    if (IS_PRESENT(certReq->certReqData, asn1set__CertReqData__lid_CA)) {
        COPY_OCTET_STRING(heap, lidca, &certReq->certReqData.lid_CA);
    }
    else {
        lidca->length = strlen(LOCALID_CA);
        lidca->value = AC_malloc(heap, lidca->length);
                           /* this should be random */
        memcpy(lidca->value, LOCALID_CA, lidca->length);
    }

    /* open the cache associated with the LocalID in the message */
    cache_name = lid2filename(lidca);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    if (! IS_PRESENT(certReq->certReqData, asn1set__CertReqData__lid_CA)) {
        status = CACHE_put(msg_cache[0], STATE_LOCALID_CA, lidca);
        assert(status == NO_ERROR);
    }

    status = verify_set_msg(request, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process CertReq */
    /* ... */

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}


asn__any bogus_signer;
alloc_ctx bogus_heap;

int ee_writes_CertReq PROTO_LIST((asn1set__LocalID*));
int
ee_writes_CertReq(lidee)
    asn1set__LocalID       *lidee;        /* IN */
{
    set_msg                 request  = 0;
    CertReq                 certReq = 0;
    int                     msg_type;
    char                   *cache_name;
    int                     len_ee_k_pub_der;
    UCHAR                  *ee_k_pub_der;
    int                     len_ee_s_pub_der;
    UCHAR                  *ee_s_pub_der;
    int                     len_ee_s_priv_der;
    UCHAR                  *ee_s_priv_der;
    cert_identity           sigCID;


    if (SHOW_ACTION) fprintf(stderr,"ee_writes_CertReq\n");

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__cardRenewalSig:
        status = startup(ROLE_CARDHOLDER, CARDHOLDER_SW, CARDHOLDER_LOG);
        assert(status == NO_ERROR);
        len_ee_s_priv_der = len_card_s_priv_der;
        ee_s_priv_der = card_s_priv_der;
        len_ee_s_pub_der = len_card_s_pub_der;
        ee_s_pub_der = card_s_pub_der;
        SET_SELF_ID_car_sig(&sigCID);
        break;
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__merRenewalBoth:
        status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
        assert(status == NO_ERROR);
        len_ee_s_priv_der = len_mer_s_priv_der;
        ee_s_priv_der = mer_s_priv_der;
        len_ee_s_pub_der = len_mer_s_pub_der;
        ee_s_pub_der = mer_s_pub_der;
        len_ee_k_pub_der = len_mer_k_pub_der;
        ee_k_pub_der = mer_k_pub_der;
        SET_SELF_ID_mer_sig(&sigCID);
        break;
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__pgwyRenewalBoth:
        status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
        assert(status == NO_ERROR);
        len_ee_s_priv_der = len_pgwy_s_priv_der;
        ee_s_priv_der = pgwy_s_priv_der;
        len_ee_s_pub_der = len_pgwy_s_pub_der;
        ee_s_pub_der = pgwy_s_pub_der;
        len_ee_k_pub_der = len_pgwy_k_pub_der;
        ee_k_pub_der = pgwy_k_pub_der;
        SET_SELF_ID_pay_sig(&sigCID);
        break;
    default:
        fprintf(stderr,"bad cert request type\n");
        exit(1);
    }

    /* the cache to be associated with this LocalID is opened */
    cache_name = lid2filename(lidee);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build CertReq message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__certReq;
    status = make_set_msg(request, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    certReq = safe_cast_CertReq(request);
    assert(certReq != 0);

    /* insert user-layer data */

    certReq->certReqData.regFormID = REGFORMID;

    certReq->certReqData.regForm = (asn1set__RegForm*)
                            AC_calloc(heap, 1, sizeof(asn1set__RegForm));
    certReq->certReqData.regForm->regFormItems.fieldName.u.visibleString =
             "registration template field name 1";
    certReq->certReqData.regForm->regFormItems.fieldName._choice =
             asn1set__FieldName__visibleString;

    certReq->certReqData.regForm->regFormItems.fieldValue.u.setString.u.visibleString =
         "registration template value 1";
    certReq->certReqData.regForm->regFormItems.fieldValue._choice =
             asn1set__FieldValue__setString;
    certReq->certReqData.regForm->regFormItems.fieldValue.u.setString._choice =
             asn1set__SETStringub_FieldValue__visibleString;

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__pgwyInitialBoth:
        /* no previous cert is available */
        break;

    case asn1set__RequestType__cardRenewalSig:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__merRenewalBoth:
    case asn1set__RequestType__pgwyRenewalBoth:
        certReq->eeSignatureCID = sigCID;
        SET_PRESENT(*certReq, CertReq__eeSignatureCID);
        break;
    }


    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__cardRenewalSig:
        certReq->acctInfo.u.panData0.pan = PAN;
        certReq->acctInfo.u.panData0.cardExpiry = CARDEXPIRY;
        break;

    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__merRenewalBoth:
    case asn1set__RequestType__pgwyRenewalBoth:
        break;
    }

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__cardInitialSig:
    case asn1set__RequestType__merInitialSig:
    case asn1set__RequestType__pgwyInitialSig:
    case asn1set__RequestType__cardRenewalSig:
    case asn1set__RequestType__merRenewalSig:
    case asn1set__RequestType__pgwyRenewalSig:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__merRenewalBoth:
    case asn1set__RequestType__pgwyRenewalBoth:

        certReq->eeSignatureKey.length = len_ee_s_priv_der;
        certReq->eeSignatureKey.object = ee_s_priv_der;
        SET_PRESENT(*certReq, CertReq__eeSignatureKey);

        certReq->certReqData.publicKeySorE.publicKeyS.length = len_ee_s_pub_der;
        certReq->certReqData.publicKeySorE.publicKeyS.object = ee_s_pub_der;
        SET_PRESENT(certReq->certReqData.publicKeySorE,
                    asn1set__PublicKeySorE__publicKeyS);
        break;
    }

    switch (CERT_REQ_TYPE) {
    case asn1set__RequestType__merInitialEnc:
    case asn1set__RequestType__pgwyInitialEnc:
    case asn1set__RequestType__merRenewalEnc:
    case asn1set__RequestType__pgwyRenewalEnc:
    case asn1set__RequestType__merInitialBoth:
    case asn1set__RequestType__pgwyInitialBoth:
    case asn1set__RequestType__merRenewalBoth:
    case asn1set__RequestType__pgwyRenewalBoth:
        certReq->certReqData.publicKeySorE.publicKeyE.length = len_ee_k_pub_der;
        certReq->certReqData.publicKeySorE.publicKeyE.object = ee_k_pub_der;
        SET_PRESENT(certReq->certReqData.publicKeySorE,
                    asn1set__PublicKeySorE__publicKeyE);
        break;
    }

    /* send CertReq */
    status = send_set_msg(request, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}





int cardholder_reads_RegFormRes PROTO_LIST((asn1set__LocalID*));
int
cardholder_reads_RegFormRes(lidee)
    asn1set__LocalID         *lidee;        /* OUT */
{
    set_msg                   response = 0;
    RegFormRes                regFormRes = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"cardholder_reads_RegFormRes\n");

    status = startup(ROLE_CARDHOLDER, CARDHOLDER_SW, CARDHOLDER_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get RegFormRes */
    status = receive_set_msg(response, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__regFormRes);

    status = decode_set_msg(response, app_ctx);
    assert(status == NO_ERROR);

    regFormRes = safe_cast_RegFormRes(response);
    assert(regFormRes != 0);

    /* copy the LocalID for use later */
    COPY_OCTET_STRING(heap, lidee, &regFormRes->regFormResTBS.lid_EE);

    /* open the cache associated with the LocalID in the message */
    cache_name = lid2filename(lidee);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(response, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process RegFormRes */
    /* ... */

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}





int cca_writes_RegFormRes PROTO_LIST((asn1set__LocalID*));
int
cca_writes_RegFormRes(lidca)
    asn1set__LocalID         *lidca;         /* IN */
{
    set_msg                   response = 0;
    RegFormRes                regFormRes = 0;
    int                       msg_type;
    char                     *cache_name;


    if (SHOW_ACTION) fprintf(stderr,"cca_writes_RegFormRes\n");

    status = startup(ROLE_CCA, CCA_SW, CCA_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = lid2filename(lidca);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);


    /* build RegFormRes message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__regFormRes;
    status = make_set_msg(response, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    regFormRes = safe_cast_RegFormRes(response);
    assert(regFormRes != 0);

    /* insert user-layer data */

    if (USE_REFERRAL) {
assert("Not yet implemented"&&0);
        regFormRes->regFormResTBS.formOrReferal._choice = asn1set__RegFormOrReferral__referralData;
    }
    else {
        regFormRes->regFormResTBS.formOrReferal.u.regFormData.policy.u.
            visibleString = POLICY_TEXT;
        regFormRes->regFormResTBS.formOrReferal.u.regFormData.policy._choice =
            asn1set__PolicyText__visibleString;

        regFormRes->regFormResTBS.formOrReferal.u.regFormData.regTemplate.
            brandLogoURL = LOGO_URL;
        SET_PRESENT(regFormRes->regFormResTBS.formOrReferal.u.regFormData.regTemplate,
                    asn1set__RegTemplate__brandLogoURL);

        regFormRes->regFormResTBS.formOrReferal.u.regFormData.regTemplate.
             regFormID = REGFORMID;

        regFormRes->regFormResTBS.formOrReferal.u.regFormData.regTemplate.
             regFieldSeq = (asn1set__RegFieldSeq*)
                          AC_calloc(heap, 1, sizeof(asn1set__RegFieldSeq));
        regFormRes->regFormResTBS.formOrReferal.u.regFormData.regTemplate.
             regFieldSeq->regField.fieldName.u.visibleString =
             "registration template field name 1";
        regFormRes->regFormResTBS.formOrReferal.u.regFormData.regTemplate.
             regFieldSeq->regField.fieldName._choice =
             asn1set__FieldName__visibleString;

        regFormRes->regFormResTBS.formOrReferal._choice = asn1set__RegFormOrReferral__regFormData;
    }

    /* send RegFormRes */
    status = send_set_msg(response, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int cca_reads_RegFormReq PROTO_LIST((asn1set__LocalID*));
int
cca_reads_RegFormReq(lidca)
    asn1set__LocalID         *lidca;         /* OUT */
{
    set_msg                   request = 0;
    RegFormReq                regFormReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"cca_reads_RegFormReq\n");

    status = startup(ROLE_CCA, CCA_SW, CCA_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get RegFormReq */
    status = receive_set_msg(request, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__regFormReq);

    status = decode_set_msg(request, app_ctx);
    assert(status == NO_ERROR);

    /* first, a LocalID-CA must be assigned */
    lidca->length = strlen(LOCALID_CA);
    lidca->value = AC_malloc(heap, lidca->length);
                       /* this should be random */
    memcpy(lidca->value, LOCALID_CA, lidca->length);

    /* create the cache to be associated with this transaction */
    cache_name = lid2filename(lidca);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* place the lidca into the cache */
    status = CACHE_put(msg_cache[0], STATE_LOCALID_CA, lidca);
    assert(status == NO_ERROR);

    status = verify_set_msg(request, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    regFormReq = safe_cast_RegFormReq(request);
    assert(regFormReq != 0);

    /* process RegFormReq */
    /* ... */

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}



int cardholder_writes_RegFormReq PROTO_LIST((asn1set__LocalID*));
int
cardholder_writes_RegFormReq(lidee)
    asn1set__LocalID       *lidee;        /* IN */
{
    set_msg                 request  = 0;
    RegFormReq              regFormReq = 0;
    int                     msg_type;
    char                   *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"cardholder_writes_RegFormReq\n");

    status = startup(ROLE_CARDHOLDER, CARDHOLDER_SW, CARDHOLDER_LOG);
    assert(status == NO_ERROR);

    /* the cache to be associated with this LocalID is opened */
    cache_name = lid2filename(lidee);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build RegFormReq message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__regFormReq;
    status = make_set_msg(request, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    regFormReq = safe_cast_RegFormReq(request);
    assert(regFormReq != 0);

    /* insert user-layer data */

    regFormReq->regFormReqData.language = ENGLISH;

    regFormReq->regFormReqData.requestType = CERT_REQ_TYPE;

    regFormReq->panOnly.pan = PAN;

    /* send RegFormReq */
    status = send_set_msg(request, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}



int cardholder_reads_CardCInitRes PROTO_LIST((asn1set__LocalID*));
int
cardholder_reads_CardCInitRes(lidee)
    asn1set__LocalID         *lidee;        /* OUT */
{
    set_msg                   response = 0;
    CardCInitRes              cardCInitRes = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"cardholder_reads_CardCInitRes\n");

    status = startup(ROLE_CARDHOLDER, CARDHOLDER_SW, CARDHOLDER_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get CardCInitRes */
    status = receive_set_msg(response, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__cardCInitRes);

    status = decode_set_msg(response, app_ctx);
    assert(status == NO_ERROR);

    cardCInitRes = safe_cast_CardCInitRes(response);
    assert(cardCInitRes != 0);

    /* copy the LocalID for use later */
    COPY_OCTET_STRING(heap, lidee, &cardCInitRes->cardCInitResTBS.lid_EE);

    /* open the cache associated with the LocalID in the message */
    cache_name = lid2filename(lidee);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(response, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process CardCInitRes */
    /* ... */

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int cca_writes_CardCInitRes PROTO_LIST((asn1set__LocalID*));
int
cca_writes_CardCInitRes(lidca)
    asn1set__LocalID         *lidca;         /* IN */
{
    set_msg                   response = 0;
    CardCInitRes              cardCInitRes = 0;
    int                       msg_type;
    char                     *cache_name;
    asn__any                  cert;


    if (SHOW_ACTION) fprintf(stderr,"cca_writes_CardCInitRes\n");

    status = startup(ROLE_CCA, CCA_SW, CCA_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = lid2filename(lidca);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build CardCInitRes message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__cardCInitRes;
    status = make_set_msg(response, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    cardCInitRes = safe_cast_CardCInitRes(response);
    assert(cardCInitRes != 0);

    /* insert user-layer data */

    cert.length = len_cca_k_cert_der;
    cert.object = cca_k_cert_der;
    status = build_CertThumb(app_ctx, heap, &cert,
                             &cardCInitRes->cardCInitResTBS.caeThumb);
    assert(status == NO_ERROR);

    SET_SELF_ID_cca_sig(&cardCInitRes->caSignatureCID);

    /* send CardCInitRes */
    status = send_set_msg(response, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}



int cca_reads_CardCInitReq PROTO_LIST((asn1set__LocalID*));
int
cca_reads_CardCInitReq(lidca)
    asn1set__LocalID         *lidca;         /* OUT */
{
    set_msg                   request = 0;
    CardCInitReq              cardCInitReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"cca_reads_CardCInitReq\n");

    status = startup(ROLE_CCA, CCA_SW, CCA_LOG);
    assert(status == NO_ERROR);

    /* first, a LocalID-CA must be assigned */
    lidca->length = strlen(LOCALID_CA);
    lidca->value = AC_malloc(heap, lidca->length);
                       /* this should be random */
    memcpy(lidca->value, LOCALID_CA, lidca->length);

    /* create the cache to be associated with this transaction */
    cache_name = lid2filename(lidca);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get CardCInitReq */
    status = receive_set_msg(request, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__cardCInitReq);

    status = decode_set_msg(request, app_ctx);
    assert(status == NO_ERROR);

    status = verify_set_msg(request, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    cardCInitReq = safe_cast_CardCInitReq(request);
    assert(cardCInitReq != 0);

    /* process CardCInitReq */
    /* ... */

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}



int cardholder_writes_CardCInitReq PROTO_LIST((asn1set__LocalID*));
int
cardholder_writes_CardCInitReq(lidee)
    asn1set__LocalID       *lidee;        /* OUT */
{
    set_msg                 request  = 0;
    CardCInitReq            cardCInitReq = 0;
    int                     msg_type;
    char                   *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"cardholder_writes_CardCInitReq\n");

    status = startup(ROLE_CARDHOLDER, CARDHOLDER_SW, CARDHOLDER_LOG);
    assert(status == NO_ERROR);

    /* first, a LocalID must be assigned */
    lidee->length = strlen(LOCALID_EE);
    lidee->value = AC_malloc(heap, lidee->length);
    memcpy(lidee->value, LOCALID_EE, lidee->length);

    /* second, the cache to be associated with this LocalID is created */
    cache_name = lid2filename(lidee);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* third, the LocalID is placed in the cache */
    status = CACHE_put(msg_cache[0], STATE_LOCALID_EE, lidee);
    assert(status == NO_ERROR);

    /* build CardCInitReq message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__cardCInitReq;
    status = make_set_msg(request, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    cardCInitReq = safe_cast_CardCInitReq(request);
    assert(cardCInitReq != 0);

    /* insert user-layer data */

    cardCInitReq->messageWrapper.message.u.cardCInitReq.brandID.u.visibleString =
                                                    BRANDID;
    cardCInitReq->messageWrapper.message.u.cardCInitReq.brandID._choice =
                                           asn1set__BrandID__visibleString;

    /* send CardCInitReq */
    status = send_set_msg(request, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}






int merchant_reads_BatchAdminRes PROTO_LIST((asn1set__XID*));
int
merchant_reads_BatchAdminRes(xid)
    asn1set__XID           *xid;      /* IN */
{
    set_msg                   response = 0;
    BatchAdminRes             batchAdminRes = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_reads_BatchAdminRes\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get BatchAdminRes */
    status = receive_set_msg(response, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__batchAdminRes);

    status = decode_set_msg(response, app_ctx);
    assert(status == NO_ERROR);

    batchAdminRes = safe_cast_BatchAdminRes(response);
    assert(batchAdminRes != 0);

    /* open the cache associated with the XID
     * this isn't entirely correct.  in real life, we'd use
     * the RRPID in the message to find the XIDs used to
     * create the batch, and then we'd open the caches
     * associated with those XIDs.  however, for simplicity,
     * we haven't set up the RRPID->XID list mapping. */

    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(response, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    if (SHOW_ACTION) fprintf(stderr, "Done....\n");

    return NO_ERROR;
}




int gateway_writes_BatchAdminRes PROTO_LIST((asn1set__RRPID*));
int
gateway_writes_BatchAdminRes(rrpid)
    asn1set__RRPID           *rrpid;       /* IN */
{
    set_msg                   response = 0;
    BatchAdminRes             batchAdminRes = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"gateway_writes_BatchAdminRes\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = rrpid2filename(rrpid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build BatchAdminRes message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__batchAdminRes;
    status = make_set_msg(response, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    batchAdminRes = safe_cast_BatchAdminRes(response);
    assert(batchAdminRes != 0);

    /* insert user-layer data */
    SET_SELF_ID_pay_sig(&batchAdminRes->gatewaySignatureCID);

    /* send BatchAdminRes */
    status = send_set_msg(response, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}



int gateway_reads_BatchAdminReq PROTO_LIST((asn1set__RRPID*));
int
gateway_reads_BatchAdminReq(rrpid)
    asn1set__RRPID           *rrpid;        /* OUT */
{
    set_msg                   request = 0;
    BatchAdminReq             batchAdminReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"gateway_reads_BatchAdminReq\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get BatchAdminReq */
    status = receive_set_msg(request, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__batchAdminReq);

    status = decode_set_msg(request, app_ctx);
    assert(status == NO_ERROR);

    batchAdminReq = safe_cast_BatchAdminReq(request);
    assert(batchAdminReq != 0);

    COPY_OCTET_STRING(heap, rrpid,
                      &batchAdminReq->batchAdminReqData.batchAdminRRTags.rrpid);

    /* create the cache to be associated with this request */
    cache_name = rrpid2filename(rrpid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(request, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process BatchAdminReq */
    /* ... */

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int merchant_writes_BatchAdminReq PROTO_LIST((asn1set__XID*));
int
merchant_writes_BatchAdminReq(xid)
    asn1set__RRPID         *xid;        /* IN */
{
    set_msg                 request  = 0;
    BatchAdminReq           batchAdminReq = 0;
    int                     msg_type;
    char                   *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_writes_BatchAdminReq\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* build BatchAdminReq message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* the cache to be associated with this XID is opened */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__batchAdminReq;
    status = make_set_msg(request, app_ctx, msg_cache, msg_type);
    assert(status == NO_ERROR);

    batchAdminReq = safe_cast_BatchAdminReq(request);
    assert(batchAdminReq != 0);

    batchAdminReq->batchAdminReqData.batchAdminRRTags.merTermIDs.merchantID.u.visibleString =
                                          "MerchantID";
    batchAdminReq->batchAdminReqData.batchAdminRRTags.merTermIDs.merchantID._choice =
                                          asn1set__MerchantID__visibleString;

    SET_SELF_ID_mer_sig(&batchAdminReq->merchantSignatureCID);

    /* send BatchAdminReq */
    status = send_set_msg(request, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}



int merchant_reads_PCertRes PROTO_LIST((asn1set__RRPID*));
int
merchant_reads_PCertRes(rrpid)
    asn1set__RRPID           *rrpid;       /* OUT */
{
    set_msg                   response = 0;
    PCertRes                  pCertRes = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_reads_PCertRes\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get PCertRes */
    status = receive_set_msg(response, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__pCertRes);

    status = decode_set_msg(response, app_ctx);
    assert(status == NO_ERROR);

    pCertRes = safe_cast_PCertRes(response);
    assert(pCertRes != 0);

    /* copy the rrpid */
    COPY_OCTET_STRING(heap, rrpid, &pCertRes->pCertResTBS.pCertRRTags.rrpid);

    /* open the cache associated with the LocalID in the message */
    cache_name = rrpid2filename(rrpid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(response, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    if (SHOW_ACTION) fprintf(stderr, "Done....\n");

    return NO_ERROR;
}




int gateway_writes_PCertRes PROTO_LIST((asn1set__RRPID*));
int
gateway_writes_PCertRes(rrpid)
    asn1set__RRPID           *rrpid;       /* IN */
{
    set_msg                   response = 0;
    PCertRes                  pCertRes = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"gateway_writes_PCertRes\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = rrpid2filename(rrpid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build PCertRes message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__pCertRes;
    status = make_set_msg(response, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    pCertRes = safe_cast_PCertRes(response);
    assert(pCertRes != 0);

    /* insert user-layer data */
    SET_SELF_ID_pay_sig(&pCertRes->gatewaySignatureCID);

    /* send PCertRes */
    status = send_set_msg(response, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}


int gateway_reads_PCertReq PROTO_LIST((asn1set__RRPID*));
int
gateway_reads_PCertReq(rrpid)
    asn1set__RRPID           *rrpid;        /* OUT */
{
    set_msg                   request = 0;
    PCertReq                  pCertReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"gateway_reads_PCertReq\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get PCertReq */
    status = receive_set_msg(request, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__pCertReq);

    status = decode_set_msg(request, app_ctx);
    assert(status == NO_ERROR);

    pCertReq = safe_cast_PCertReq(request);
    assert(pCertReq != 0);

    rrpid->length = pCertReq->pCertReqData.pCertRRTags.rrpid.length;
    rrpid->value = pCertReq->pCertReqData.pCertRRTags.rrpid.value;

    /* create the cache to be associated with this request */
    cache_name = rrpid2filename(rrpid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(request, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process PCertReq */
    /* ... */

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int merchant_writes_PCertReq PROTO_LIST((asn1set__RRPID*));
int
merchant_writes_PCertReq(rrpid)
    asn1set__RRPID         *rrpid;        /* OUT */
{
    set_msg                 request  = 0;
    PCertReq                pCertReq = 0;
    int                     msg_type;
    char                   *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_writes_PCertReq\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* build PCertReq message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* since the RRPID is unknown at this point, no cache can
     * be opened and hence 0 is passed as the cache parameter
     * to make_set_msg() */

    msg_type = asn1set__Message__pCertReq;
    status = make_set_msg(request, app_ctx, 0, msg_type);
    assert(status == NO_ERROR);

    pCertReq = safe_cast_PCertReq(request);
    assert(pCertReq != 0);

    pCertReq->pCertReqData.brandAndBINSeq = (asn1set__BrandAndBINSeq*)AC_calloc(heap, 1, sizeof(asn1set__BrandAndBINSeq));
    assert(pCertReq->pCertReqData.brandAndBINSeq != 0);
    pCertReq->pCertReqData.brandAndBINSeq->brandAndBIN.brandID.u.visibleString = 
                                          BRANDID;
    pCertReq->pCertReqData.brandAndBINSeq->brandAndBIN.brandID._choice = 
                                          asn1set__BrandID__visibleString;

    pCertReq->pCertReqData.pCertRRTags.merTermIDs.merchantID.u.visibleString =
                                          "MerchantID";
    pCertReq->pCertReqData.pCertRRTags.merTermIDs.merchantID._choice =
                                          asn1set__MerchantID__visibleString;

    SET_SELF_ID_mer_sig(&pCertReq->merchantSignatureCID);

    /* copy the RRPID for use later */
    COPY_OCTET_STRING(heap, rrpid, &pCertReq->pCertReqData.pCertRRTags.rrpid);

    /* the cache to be associated with this RRPID is opened */
    cache_name = lid2filename(rrpid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* send PCertReq */
    status = send_set_msg(request, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}








int merchant_reads_CredRevRes PROTO_LIST((asn1set__XID*));
int
merchant_reads_CredRevRes(xid)
    asn1set__XID             *xid;         /* OUT */
{
    set_msg                   msg = 0;
    CredRevRes                 credRevRes = 0;
    int                       msg_type;
    char                     *cache_name;
    int                       i;
    asn1set__CapRevOrCredResItemSeq  *seq;

    if (SHOW_ACTION) fprintf(stderr,"merchant_reads_CredRevRes\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&msg, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get CredRevRes */
    status = receive_set_msg(msg, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__credRevRes);

    status = decode_set_msg(msg, app_ctx);
    assert(status == NO_ERROR);

    credRevRes = safe_cast_CredRevRes(msg);
    assert(credRevRes != 0);

    /* open the cache(s) associated with the XID(s) in the message */
    i = 0;
    for (seq = credRevRes->capRevOrCredResData.capRevOrCredResItemSeq; seq != 0; seq = seq->next)
    {

        /* copy the XID for use later */
        COPY_OCTET_STRING(heap, &xid[i],
                          &seq->capRevOrCredResItem.transIDs.xid);

        cache_name = xid2filename(xid);
        status = create_cache_ctx_db(&msg_cache[i], cache_name);
        assert(status == NO_ERROR);
        ++i;
    }
    msg_cache[i] = 0; /* zero after the last cache */
    xid[i].length = 0; /* zero after the last xid */

    status = verify_set_msg(msg, app_ctx, msg_cache);
    assert(status == NO_ERROR);

    i = 0;
    for (seq = credRevRes->capRevOrCredResData.capRevOrCredResItemSeq; seq != 0; seq = seq->next)
    {
        CACHE_delete(&msg_cache[i]);
        assert(status == NO_ERROR);
        ++i;
    }

    /* process CredRevRes */
    /* ... */

    status = delete_set_msg(&msg);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    if (SHOW_ACTION) fprintf(stderr, "Done....\n");

    return NO_ERROR;
}


int gateway_writes_CredRevRes PROTO_LIST((asn1set__XID*));
int
gateway_writes_CredRevRes(xid)
    asn1set__XID             *xid;         /* IN */
{
    set_msg                   response = 0;
    CredRevRes                 credRevRes = 0;
    int                       msg_type;
    char                     *cache_name;
    asn1set__CapRevOrCredResPayload   *payload;

    if (SHOW_ACTION) fprintf(stderr,"gateway_writes_CredRevRes\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__credRevRes;
    /* build CredRevRes message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    status = make_set_msg(response, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    credRevRes = safe_cast_CredRevRes(response);
    assert(credRevRes != 0);

    /* insert user-layer data */
    payload = &credRevRes->capRevOrCredResData.capRevOrCredResItemSeq->capRevOrCredResItem.capRevOrCredResPayload;

    payload->capRevOrCredCode = asn1set__CapRevOrCredCode__success;

    /*  this pseudo-cache state is used just in this demo app, since for
     *  demo purposes I don't want to maintain a seperate application-level
     *  database  */
    status = CACHE_get(*msg_cache, DCAPREVAMT,
                       &payload->capRevOrCredActualAmt, heap);
    assert(status == NO_ERROR);

    /* send CredRevRes */
    status = send_set_msg(response, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}


int gateway_reads_CredRevReq PROTO_LIST((asn1set__XID*));
int
gateway_reads_CredRevReq(xid)
    asn1set__XID             *xid;         /* OUT */
{
    set_msg                   request = 0;
    CredRevReq                credRevReq = 0;
    int                       msg_type;
    char                     *cache_name;
    int                       i;
    asn1set__CapRevOrCredReqItemSeq   *seq;
    struct CapTokenSeq_               *capTokenSeq;

    if (SHOW_ACTION) fprintf(stderr,"gateway_reads_CredRevReq\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get CredRevReq */
    status = receive_set_msg(request, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__credRevReq);

    status = decode_set_msg(request, app_ctx);
    assert(status == NO_ERROR);

    credRevReq = safe_cast_CredRevReq(request);
    assert(credRevReq != 0);

    /* open the cache(s) associated with the XID(s) in the message */
    i = 0;
    capTokenSeq = credRevReq->capTokenSeq;
    for (seq = credRevReq->capRevOrCredReqData.capRevOrCredReqItemSeq; seq != 0; seq=seq->next)
    {

        /* copy the XID for use later */
        COPY_OCTET_STRING(heap, &xid[i],
                          &seq->capRevOrCredReqItem.transIDs.xid);

        cache_name = xid2filename(&xid[i]);
        status = create_cache_ctx_db(&msg_cache[i], cache_name);
        assert(status == NO_ERROR);

        /*  this pseudo-cache state is used just in this demo app, since for
         *  demo purposes I don't want to maintain a seperate application-level
         *  database  */
        status = CACHE_put(msg_cache[i], DCAPREVAMT, &credRevReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem.capRevOrCredReqAmt);
        assert(status == NO_ERROR);

        ++i;
        capTokenSeq = capTokenSeq->next;
    }
    msg_cache[i] = 0; /* zero after the last cache */
    xid[i].length = 0; /* zero after the last xid */

    status = verify_set_msg(request, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    i = 0;
    for (seq = credRevReq->capRevOrCredReqData.capRevOrCredReqItemSeq; seq != 0; seq=seq->next)
    {
        status = CACHE_delete(&msg_cache[i]);
        assert(status == NO_ERROR);
        ++i;
    }

    /* process CredRevReq */
    /* ... */

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int merchant_writes_CredRevReq PROTO_LIST((asn1set__XID*, cert_identity, cert_identity));
int
merchant_writes_CredRevReq(xid, merchantSignatureCID, gatewayKeyExchangeCID)
    asn1set__XID             *xid;                     /* IN */
    cert_identity             merchantSignatureCID;    /* IN */
    cert_identity             gatewayKeyExchangeCID;   /* IN */
{
    set_msg                   request = 0;
    CredRevReq                credRevReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_writes_CredRevReq\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);
    msg_cache[1] = 0;

    /* build CredRevReq message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__credRevReq;
    status = make_set_msg(request, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    credRevReq = safe_cast_CredRevReq(request);
    assert(credRevReq != 0);

    /* insert user-layer data */

    credRevReq->capRevOrCredReqData.capRevOrCredRRTags.merTermIDs.merchantID.u.visibleString =
                                        "MerchantID";
    credRevReq->capRevOrCredReqData.capRevOrCredRRTags.merTermIDs.merchantID._choice =
                                        asn1set__MerchantID__visibleString;

    credRevReq->capRevOrCredReqData.capRevOrCredRRTags.currentDate = time(0);

    credRevReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem.capRevOrCredReqAmt.currency = US_DOLLAR;
    credRevReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem.capRevOrCredReqAmt.amount = CAPAMT;
    credRevReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem.capRevOrCredReqAmt.amtExp10 = EXPONENT;
    SET_PRESENT(credRevReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem, asn1set__CapRevOrCredReqItem__capRevOrCredReqAmt);

    credRevReq->merchantSignatureCID = merchantSignatureCID;
    credRevReq->gatewayKeyExchangeCID = gatewayKeyExchangeCID;

    /* send CredRevReq */
    status = send_set_msg(request, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}


int merchant_reads_CredRes PROTO_LIST((asn1set__XID*));
int
merchant_reads_CredRes(xid)
    asn1set__XID             *xid;         /* OUT */
{
    set_msg                   msg = 0;
    CredRes                   credRes = 0;
    int                       msg_type;
    char                     *cache_name;
    int                       i;
    asn1set__CapRevOrCredResItemSeq   *seq;

    if (SHOW_ACTION) fprintf(stderr,"merchant_reads_CredRes\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&msg, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get CredRes */
    status = receive_set_msg(msg, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__credRes);

    status = decode_set_msg(msg, app_ctx);
    assert(status == NO_ERROR);

    credRes = safe_cast_CredRes(msg);
    assert(credRes != 0);

    /* open the cache(s) associated with the XID(s) in the message */
    i = 0;
    for (seq = credRes->capRevOrCredResData.capRevOrCredResItemSeq; seq != 0; seq = seq->next)
    {

        /* copy the XID for use later */
        COPY_OCTET_STRING(heap, &xid[i],
                          &seq->capRevOrCredResItem.transIDs.xid);

        cache_name = xid2filename(xid);
        status = create_cache_ctx_db(&msg_cache[i], cache_name);
        assert(status == NO_ERROR);
        ++i;
    }
    msg_cache[i] = 0; /* zero after the last cache */
    xid[i].length = 0; /* zero after the last xid */

    status = verify_set_msg(msg, app_ctx, msg_cache);
    assert(status == NO_ERROR);

    i = 0;
    for (seq = credRes->capRevOrCredResData.capRevOrCredResItemSeq; seq != 0; seq = seq->next)
    {
        CACHE_delete(&msg_cache[i]);
        assert(status == NO_ERROR);
        ++i;
    }

    /* process CredRes */
    /* ... */

    status = delete_set_msg(&msg);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    if (SHOW_ACTION) fprintf(stderr, "Done....\n");

    return NO_ERROR;
}


int gateway_writes_CredRes PROTO_LIST((asn1set__XID*));
int
gateway_writes_CredRes(xid)
    asn1set__XID             *xid;         /* IN */
{
    set_msg                   response = 0;
    CredRes                   credRes = 0;
    int                       msg_type;
    char                     *cache_name;
    asn1set__CapRevOrCredResPayload   *payload;

    if (SHOW_ACTION) fprintf(stderr,"gateway_writes_CredRes\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__credRes;
    /* build CredRes message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    status = make_set_msg(response, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    credRes = safe_cast_CredRes(response);
    assert(credRes != 0);

    /* insert user-layer data */
    payload = &credRes->capRevOrCredResData.capRevOrCredResItemSeq->capRevOrCredResItem.capRevOrCredResPayload;

    payload->capRevOrCredCode = asn1set__CapRevOrCredCode__success;

    /*  this pseudo-cache state is used just in this demo app, since for
     *  demo purposes I don't want to maintain a seperate application-level
     *  database  */
    status = CACHE_get(*msg_cache, DCAPREVAMT,
                       &payload->capRevOrCredActualAmt, heap);
    assert(status == NO_ERROR);

    /* send CredRes */
    status = send_set_msg(response, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}


int gateway_reads_CredReq PROTO_LIST((asn1set__XID*));
int
gateway_reads_CredReq(xid)
    asn1set__XID             *xid;         /* OUT */
{
    set_msg                   request = 0;
    CredReq                    credReq = 0;
    int                       msg_type;
    char                     *cache_name;
    int                       i;
    asn1set__CapRevOrCredReqItemSeq   *seq;
    struct CapTokenSeq_               *capTokenSeq;

    if (SHOW_ACTION) fprintf(stderr,"gateway_reads_CredReq\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get CredReq */
    status = receive_set_msg(request, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__credReq);

    status = decode_set_msg(request, app_ctx);
    assert(status == NO_ERROR);

    credReq = safe_cast_CredReq(request);
    assert(credReq != 0);

    /* open the cache(s) associated with the XID(s) in the message */
    i = 0;
    capTokenSeq = credReq->capTokenSeq;
    for (seq = credReq->capRevOrCredReqData.capRevOrCredReqItemSeq; seq != 0; seq=seq->next)
    {

        /* copy the XID for use later */
        COPY_OCTET_STRING(heap, &xid[i],
                          &seq->capRevOrCredReqItem.transIDs.xid);

        cache_name = xid2filename(&xid[i]);
        status = create_cache_ctx_db(&msg_cache[i], cache_name);
        assert(status == NO_ERROR);

        /*  this pseudo-cache state is used just in this demo app, since for
         *  demo purposes I don't want to maintain a seperate application-level
         *  database  */
        status = CACHE_put(msg_cache[i], DCAPREVAMT, &credReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem.capRevOrCredReqAmt);
        assert(status == NO_ERROR);

        ++i;
        capTokenSeq = capTokenSeq->next;
    }
    msg_cache[i] = 0; /* zero after the last cache */
    xid[i].length = 0; /* zero after the last xid */

    status = verify_set_msg(request, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    i = 0;
    for (seq = credReq->capRevOrCredReqData.capRevOrCredReqItemSeq; seq != 0; seq=seq->next)
    {
        status = CACHE_delete(&msg_cache[i]);
        assert(status == NO_ERROR);
        ++i;
    }

    /* process CredReq */
    /* ... */

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int merchant_writes_CredReq PROTO_LIST((asn1set__XID*, cert_identity, cert_identity));
int
merchant_writes_CredReq(xid, merchantSignatureCID, gatewayKeyExchangeCID)
    asn1set__XID             *xid;                     /* IN */
    cert_identity             merchantSignatureCID;    /* IN */
    cert_identity             gatewayKeyExchangeCID;   /* IN */
{
    set_msg                   request = 0;
    CredReq                   credReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_writes_CredReq\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);
    msg_cache[1] = 0;

    /* build CredReq message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__credReq;
    status = make_set_msg(request, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    credReq = safe_cast_CredReq(request);
    assert(credReq != 0);

    /* insert user-layer data */

    credReq->capRevOrCredReqData.capRevOrCredRRTags.merTermIDs.merchantID.u.visibleString =
                                        "MerchantID";
    credReq->capRevOrCredReqData.capRevOrCredRRTags.merTermIDs.merchantID._choice =
                                        asn1set__MerchantID__visibleString;

    credReq->capRevOrCredReqData.capRevOrCredRRTags.currentDate = time(0);

    credReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem.capRevOrCredReqAmt.currency = US_DOLLAR;
    credReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem.capRevOrCredReqAmt.amount = CAPAMT;
    credReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem.capRevOrCredReqAmt.amtExp10 = EXPONENT;
    SET_PRESENT(credReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem, asn1set__CapRevOrCredReqItem__capRevOrCredReqAmt);

    credReq->merchantSignatureCID = merchantSignatureCID;
    credReq->gatewayKeyExchangeCID = gatewayKeyExchangeCID;

    /* send CredReq */
    status = send_set_msg(request, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int merchant_reads_CapRevRes PROTO_LIST((asn1set__XID*));
int
merchant_reads_CapRevRes(xid)
    asn1set__XID             *xid;         /* OUT */
{
    set_msg                   msg = 0;
    CapRevRes                 capRevRes = 0;
    int                       msg_type;
    char                     *cache_name;
    int                       i;
    asn1set__CapRevOrCredResItemSeq *seq;

    if (SHOW_ACTION) fprintf(stderr,"merchant_reads_CapRevRes\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&msg, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get CapRevRes */
    status = receive_set_msg(msg, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__capRevRes);

    status = decode_set_msg(msg, app_ctx);
    assert(status == NO_ERROR);

    capRevRes = safe_cast_CapRevRes(msg);
    assert(capRevRes != 0);

    /* open the cache(s) associated with the XID(s) in the message */
    i = 0;
    for (seq = capRevRes->capRevOrCredResData.capRevOrCredResItemSeq; seq != 0; seq = seq->next)
    {

        /* copy the XID for use later */
        COPY_OCTET_STRING(heap, &xid[i],
                          &seq->capRevOrCredResItem.transIDs.xid);

        cache_name = xid2filename(xid);
        status = create_cache_ctx_db(&msg_cache[i], cache_name);
        assert(status == NO_ERROR);
        ++i;
    }
    msg_cache[i] = 0; /* zero after the last cache */
    xid[i].length = 0; /* zero after the last xid */

    status = verify_set_msg(msg, app_ctx, msg_cache);
    assert(status == NO_ERROR);

    i = 0;
    for (seq = capRevRes->capRevOrCredResData.capRevOrCredResItemSeq; seq != 0; seq = seq->next)
    {
        CACHE_delete(&msg_cache[i]);
        assert(status == NO_ERROR);
        ++i;
    }

    /* process CapRevRes */
    /* ... */

    status = delete_set_msg(&msg);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    if (SHOW_ACTION) fprintf(stderr, "Done....\n");

    return NO_ERROR;
}


int gateway_writes_CapRevRes PROTO_LIST((asn1set__XID*));
int
gateway_writes_CapRevRes(xid)
    asn1set__XID             *xid;         /* IN */
{
    set_msg                   response = 0;
    CapRevRes                 capRevRes = 0;
    int                       msg_type;
    char                     *cache_name;
    asn1set__CapRevOrCredResPayload   *payload;

    if (SHOW_ACTION) fprintf(stderr,"gateway_writes_CapRevRes\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__capRevRes;
    /* build CapRevRes message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    status = make_set_msg(response, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    capRevRes = safe_cast_CapRevRes(response);
    assert(capRevRes != 0);

    /* insert user-layer data */
    payload = &capRevRes->capRevOrCredResData.capRevOrCredResItemSeq->capRevOrCredResItem.capRevOrCredResPayload;

    payload->capRevOrCredCode = asn1set__CapRevOrCredCode__success;

    /*  this pseudo-cache state is used just in this demo app, since for
     *  demo purposes I don't want to maintain a seperate application-level
     *  database  */
    status = CACHE_get(*msg_cache, DCAPREVAMT,
                       &payload->capRevOrCredActualAmt, heap);
    assert(status == NO_ERROR);

    /* send CapRevRes */
    status = send_set_msg(response, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}


int gateway_reads_CapRevReq PROTO_LIST((asn1set__XID*));
int
gateway_reads_CapRevReq(xid)
    asn1set__XID             *xid;         /* OUT */
{
    set_msg                   request = 0;
    CapRevReq                    capRevReq = 0;
    int                       msg_type;
    char                     *cache_name;
    int                       i;
    asn1set__CapRevOrCredReqItemSeq  *seq;
    struct CapTokenSeq_              *capTokenSeq;

    if (SHOW_ACTION) fprintf(stderr,"gateway_reads_CapRevReq\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get CapRevReq */
    status = receive_set_msg(request, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__capRevReq);

    status = decode_set_msg(request, app_ctx);
    assert(status == NO_ERROR);

    capRevReq = safe_cast_CapRevReq(request);
    assert(capRevReq != 0);

    /* open the cache(s) associated with the XID(s) in the message */
    i = 0;
    capTokenSeq = capRevReq->capTokenSeq;
    for (seq = capRevReq->capRevOrCredReqData.capRevOrCredReqItemSeq; seq != 0; seq = seq->next)
    {

        /* copy the XID for use later */
        COPY_OCTET_STRING(heap, &xid[i],
                          &seq->capRevOrCredReqItem.transIDs.xid);

        cache_name = xid2filename(&xid[i]);
        status = create_cache_ctx_db(&msg_cache[i], cache_name);
        assert(status == NO_ERROR);

        if (IS_PRESENT(capRevReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem, asn1set__CapRevOrCredReqItem__capRevOrCredReqAmt))
        {
            /*  this pseudo-cache state is used just in this demo app, 
             *  since for demo purposes I don't want to maintain a seperate 
             *  application-level database  */
            status = CACHE_put(msg_cache[i], DCAPREVAMT, &capRevReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem.capRevOrCredReqAmt);
            assert(status == NO_ERROR);
        }

        ++i;
        capTokenSeq = capTokenSeq->next;
    }
    msg_cache[i] = 0; /* zero after the last cache */
    xid[i].length = 0; /* zero after the last xid */

    status = verify_set_msg(request, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    i = 0;
    for (seq = capRevReq->capRevOrCredReqData.capRevOrCredReqItemSeq; seq != 0; seq = seq->next)
    {
        status = CACHE_delete(&msg_cache[i]);
        assert(status == NO_ERROR);
        ++i;
    }

    /* process CapRevReq */
    /* ... */

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int merchant_writes_CapRevReq PROTO_LIST((asn1set__XID*, cert_identity, cert_identity));
int
merchant_writes_CapRevReq(xid, merchantSignatureCID, gatewayKeyExchangeCID)
    asn1set__XID             *xid;                     /* IN */
    cert_identity             merchantSignatureCID;    /* IN */
    cert_identity             gatewayKeyExchangeCID;   /* IN */
{
    set_msg                   request = 0;
    CapRevReq                 capRevReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_writes_CapRevReq\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);
    msg_cache[1] = 0;

    /* build CapRevReq message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__capRevReq;
    status = make_set_msg(request, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    capRevReq = safe_cast_CapRevReq(request);
    assert(capRevReq != 0);

    /* insert user-layer data */

    capRevReq->capRevOrCredReqData.capRevOrCredRRTags.merTermIDs.merchantID.u.visibleString =
                                        "MerchantID";
    capRevReq->capRevOrCredReqData.capRevOrCredRRTags.merTermIDs.merchantID._choice =
                                        asn1set__MerchantID__visibleString;

    capRevReq->capRevOrCredReqData.capRevOrCredRRTags.currentDate = time(0);

    capRevReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem.capRevOrCredReqAmt.currency = US_DOLLAR;
    capRevReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem.capRevOrCredReqAmt.amount = CAPAMT;
    capRevReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem.capRevOrCredReqAmt.amtExp10 = EXPONENT;
    SET_PRESENT(capRevReq->capRevOrCredReqData.capRevOrCredReqItemSeq->capRevOrCredReqItem, asn1set__CapRevOrCredReqItem__capRevOrCredReqAmt);

    capRevReq->merchantSignatureCID = merchantSignatureCID;
    capRevReq->gatewayKeyExchangeCID = gatewayKeyExchangeCID;

    /* send CapRevReq */
    status = send_set_msg(request, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int merchant_reads_AuthRevRes PROTO_LIST((asn1set__XID*, cert_identity*, cert_identity*));
int
merchant_reads_AuthRevRes(xid, merchantSignatureCID, gatewayKeyExchangeCID)
    asn1set__XID             *xid;                     /* OUT */
    cert_identity            *merchantSignatureCID;    /* OUT */
    cert_identity            *gatewayKeyExchangeCID;   /* OUT */
{
    set_msg                   response = 0;
    AuthRevRes                authRevRes = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_reads_AuthRevRes\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get AuthRevRes */
    status = receive_set_msg(response, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__authRevRes);

    status = decode_set_msg(response, app_ctx);
    assert(status == NO_ERROR);

    authRevRes = safe_cast_AuthRevRes(response);
    assert(authRevRes != 0);


    status = make_key_exchange_cert_identity_using_signature_cert_identity(
                    app_ctx, heap,
                    gatewayKeyExchangeCID, authRevRes->gatewaySignatureCID);
    status = make_signature_cert_identity_using_key_exchange_cert_identity(
                    app_ctx, heap,
                    merchantSignatureCID, authRevRes->merchantKeyExchangeCID);
    assert(status == NO_ERROR);

    /* without authResDataNew, there aren't
     * any TransIDs in the message, so the message 
     * cache can't be located (unless we're willing to
     * trust the message wrapper, which we shouldn't) */
    memset(xid, 0, sizeof(*xid));

    /* copy the XID for use later */
    COPY_OCTET_STRING(heap, xid, &authRevRes->authRevResData.authResDataNew.transIDs.xid);

    /* open the cache associated with the XID in the message */
    cache_name = xid2filename(xid);

    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(response, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process AuthRevRes */
    /* ... */

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    if (SHOW_ACTION) fprintf(stderr, "Done....\n");

    return NO_ERROR;
}




int gateway_writes_AuthRevRes PROTO_LIST((asn1set__XID*));
int
gateway_writes_AuthRevRes(xid)
    asn1set__XID             *xid;         /* IN */
{
    set_msg                   response = 0;
    AuthRevRes                authRevRes = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"gateway_writes_AuthRevRes\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build AuthRevRes message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__authRevRes;
    status = make_set_msg(response, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    authRevRes = safe_cast_AuthRevRes(response);
    assert(authRevRes != 0);

    /* insert user-layer data */

    if (USE_CAPTOKEN) {
      /* send CapToken to merchant (to be included in CapReq) */
      if (SHOW_ACTION) fprintf(stderr,"using CapToken\n");
      authRevRes->capTokenNew.capTokenData.authAmt.currency  = US_DOLLAR;
      authRevRes->capTokenNew.capTokenData.authAmt.amount    = AUTHAMT;
      authRevRes->capTokenNew.capTokenData.authAmt.amtExp10  = EXPONENT;
      SET_PRESENT(authRevRes->capTokenNew, CapToken__capTokenData);
      SET_PRESENT(*authRevRes, AuthRevRes__capTokenNew);

      if (USE_PANTOKEN) {
          /* send PANToken back to merchant */
          SET_PRESENT(*authRevRes, CapToken__panToken);
      }
    }


    authRevRes->authRevResData.authNewAmt.currency = US_DOLLAR;
    authRevRes->authRevResData.authNewAmt.amount   = AUTHAMT;
    authRevRes->authRevResData.authNewAmt.amtExp10 = EXPONENT;

    authRevRes->authRevResData.authResDataNew.authResPayloadNew.authHeader.
                                       authCode = asn1set__AuthCode__approved;

    /* send AuthRevRes */
    status = send_set_msg(response, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}

int gateway_reads_AuthRevReq PROTO_LIST((asn1set__XID*));
int
gateway_reads_AuthRevReq(xid)
    asn1set__XID             *xid;         /* OUT */
{
    set_msg                   request = 0;
    AuthRevReq                authRevReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"gateway_reads_AuthRevReq\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get AuthRevReq */
    status = receive_set_msg(request, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__authRevReq);

    status = decode_set_msg(request, app_ctx);
    assert(status == NO_ERROR);

    authRevReq = safe_cast_AuthRevReq(request);
    assert(authRevReq != 0);

    /* copy the XID for use later */
    switch (authRevReq->pi_content._choice) {
    case asn1set__PI__piUnsigned:
        COPY_OCTET_STRING(heap, xid,
                          &authRevReq->pi_content.u.piUnsigned.piHead.transIDs.xid);

        break;
    case asn1set__PI__piDualSigned:
        COPY_OCTET_STRING(heap, xid,
                          &authRevReq->pi_content.u.piDualSigned.piData.piHead.transIDs.xid);
        break;
    case asn1set__PI__authToken:
        COPY_OCTET_STRING(heap, xid,
                          &authRevReq->pi_content.u.authToken.authTokenData.transIDs.xid);
        break;
/*TODO: remove now! */
default:
assert(!"blah blah blah");
break;
    }

    /* open the cache associated with the XID in the message */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(request, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process AuthRevReq */
    /* ... */

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}






int merchant_writes_AuthRevReq PROTO_LIST((asn1set__XID*));
int
merchant_writes_AuthRevReq(xid)
    asn1set__XID             *xid;         /* IN */
{
    set_msg                   msg = 0;
    AuthRevReq                authRevReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_writes_AuthRevReq\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build AuthRevReq message */
    status = create_set_msg(&msg, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__authRevReq;
    status = make_set_msg(msg, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    authRevReq = safe_cast_AuthRevReq(msg);
    assert(authRevReq != 0);

    /* insert user-layer data */
    authRevReq->authRevReqData.authRevTags.authRevRRTags.merTermIDs.merchantID.u.visibleString = "MerchantID";
    authRevReq->authRevReqData.authRevTags.authRevRRTags.merTermIDs.merchantID._choice =
                                         asn1set__MerchantID__visibleString;

    authRevReq->authRevReqData.authRevTags.authRevRRTags.currentDate = time(0);

    authRevReq->authRevReqData.authNewAmt.currency = US_DOLLAR;
    authRevReq->authRevReqData.authNewAmt.amount   = AUTHAMT;
    authRevReq->authRevReqData.authNewAmt.amtExp10 = EXPONENT;

    /* send AuthRevReq */
    status = send_set_msg(msg, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&msg);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int cardholder_reads_InqRes PROTO_LIST((asn1set__LocalID*));
int
cardholder_reads_InqRes(lidc)
    asn1set__LocalID         *lidc;        /* OUT */
{
    set_msg                   msg = 0;
    InqRes                      inqRes = 0;
    int                       msg_type;
    char                     *cache_name;
    struct AcqCardMsgSeq_    *seq;


    if (SHOW_ACTION) fprintf(stderr,"cardholder_reads_InqRes\n");

    status = startup(ROLE_CARDHOLDER, CARDHOLDER_SW, CARDHOLDER_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&msg, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get InqRes */
    status = receive_set_msg(msg, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__inqRes);

    status = decode_set_msg(msg, app_ctx);
    assert(status == NO_ERROR);

    inqRes = safe_cast_InqRes(msg);
    assert(inqRes != 0);

    /* copy the LocalID for use later */
    COPY_OCTET_STRING(heap, lidc, &inqRes->pResData.transIDs.lid_C);

    /* open the cache associated with the LocalID in the message */
    cache_name = lid2filename(lidc);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(msg, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    if (SHOW_ACTION) {
        for (seq = inqRes->acqCardMsgSeq; seq != 0; seq = seq->next) {
            if (IS_PRESENT(*seq, AcqCardMsgSeq__acqCardMsg)) {
                fprintf(stderr, "AcqCardCodeMsg: '%s'\n",
                        seq->acqCardMsg.acqCardCodeMsg.acqCardMsgData.
                        acqCardText.u.visibleString);
            }
        }
    }

    /* process InqRes */
    /* ... */

    status = delete_set_msg(&msg);
    assert(status == NO_ERROR);

    if (SHOW_ACTION) fprintf(stderr, "Done....\n");

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}



int merchant_writes_InqRes PROTO_LIST((asn1set__XID*));
int
merchant_writes_InqRes(xid)
    asn1set__XID             *xid;        /* IN */
{
    set_msg                   msg = 0;
    InqRes                      inqRes = 0;
    int                       msg_type;
    char                     *cache_name;
    asn1set__PResPayloadSeq  *seq;

    if (SHOW_ACTION) fprintf(stderr,"merchant_writes_InqRes\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build InqRes message */
    status = create_set_msg(&msg, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__inqRes;
    status = make_set_msg(msg, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    inqRes = safe_cast_InqRes(msg);
    assert(inqRes != 0);

    /* insert user-layer data */

    inqRes->pResData.pResPayloadSeq = (asn1set__PResPayloadSeq*)
           AC_calloc(heap, 1, sizeof(*(inqRes->pResData.pResPayloadSeq)));

    seq = inqRes->pResData.pResPayloadSeq;
    seq->pResPayload.completionCode = asn1set__CompletionCode__capturePerformed;

    SET_PRESENT(seq->pResPayload, asn1set__PResPayload__results);
    seq->pResPayload.results.authStatus.authDate = time(0); /* made up */
    seq->pResPayload.results.authStatus.authCode = asn1set__AuthCode__approved;
    seq->pResPayload.results.authStatus.authRatio = AUTHAMT/PURCHAMT;
    SET_PRESENT(seq->pResPayload.results, asn1set__Results__authStatus);

    seq->pResPayload.results.capStatus.capDate = time(0); /* made up */
    seq->pResPayload.results.capStatus.capCode = asn1set__CapCode__success;
    seq->pResPayload.results.capStatus.capRatio = CAPAMT/PURCHAMT;
    SET_PRESENT(seq->pResPayload.results, asn1set__Results__capStatus);

    /* send InqRes */
    status = send_set_msg(msg, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&msg);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}





int merchant_reads_InqReq PROTO_LIST((asn1set__XID*));
int
merchant_reads_InqReq(xid)
    asn1set__XID             *xid;         /* OUT */
{
    set_msg                   request = 0;
    InqReq                    inqReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_reads_InqReq\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get InqReq */
    status = receive_set_msg(request, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__inqReq);

    status = decode_set_msg(request, app_ctx);
    assert(status == NO_ERROR);

    inqReq = safe_cast_InqReq(request);
    assert(inqReq != 0);

    /* copy the xid */
    COPY_OCTET_STRING(heap, xid, &inqReq->inqReqData.transIDs.xid);

    /* create the cache to be associated with this transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(request, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process InqReq */
    /* ... */

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int cardholder_writes_InqReq PROTO_LIST((asn1set__LocalID*));
int
cardholder_writes_InqReq(lidc)
    asn1set__LocalID       *lidc;        /* IN */
{
    set_msg                 request  = 0;
    InqReq                  inqReq = 0;
    int                     msg_type;
    char                   *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"cardholder_writes_InqReq\n");

    status = startup(ROLE_CARDHOLDER, CARDHOLDER_SW, CARDHOLDER_LOG);
    assert(status == NO_ERROR);

    /* the cache to be associated with this LocalID is created */
    cache_name = lid2filename(lidc);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build InqReq message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__inqReq;
    status = make_set_msg(request, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    inqReq = safe_cast_InqReq(request);
    assert(inqReq != 0);

    /* insert user-layer data */
    if (USE_SIGNATURE) {
        SET_SELF_ID_car_sig(&inqReq->cardholderSignatureCID);
        SET_PRESENT(*inqReq, InqReq__cardholderSignatureCID);
    }

    /* send InqReq */
    status = send_set_msg(request, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int cardholder_reads_PRes PROTO_LIST((asn1set__LocalID*));
int
cardholder_reads_PRes(lidc)
    asn1set__LocalID         *lidc;        /* OUT */
{
    set_msg                   msg = 0;
    PRes                      pRes = 0;
    int                       msg_type;
    char                     *cache_name;
    struct AcqCardMsgSeq_    *seq;


    if (SHOW_ACTION) fprintf(stderr,"cardholder_reads_PRes\n");

    status = startup(ROLE_CARDHOLDER, CARDHOLDER_SW, CARDHOLDER_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&msg, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get PRes */
    status = receive_set_msg(msg, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__pRes);

    status = decode_set_msg(msg, app_ctx);
    assert(status == NO_ERROR);

    pRes = safe_cast_PRes(msg);
    assert(pRes != 0);

    /* copy the LocalID for use later */
    COPY_OCTET_STRING(heap, lidc, &pRes->pResData.transIDs.lid_C);

    /* open the cache associated with the LocalID in the message */
    cache_name = lid2filename(lidc);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(msg, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    if (SHOW_ACTION) {
        for (seq = pRes->acqCardMsgSeq; seq != 0; seq = seq->next) {
            if (IS_PRESENT(*seq, AcqCardMsgSeq__acqCardMsg)) {
                fprintf(stderr, "AcqCardCodeMsg: '%s'\n",
                        seq->acqCardMsg.acqCardCodeMsg.acqCardMsgData.
                        acqCardText.u.visibleString);
            }
        }
    }

    /* process PRes */
    /* ... */

    status = delete_set_msg(&msg);
    assert(status == NO_ERROR);

    if (SHOW_ACTION) fprintf(stderr, "Done....\n");

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}



int merchant_writes_PRes PROTO_LIST((asn1set__XID*));
int
merchant_writes_PRes(xid)
    asn1set__XID             *xid;        /* IN */
{
    set_msg                   msg = 0;
    PRes                      pRes = 0;
    int                       msg_type;
    char                     *cache_name;
    asn1set__PResPayloadSeq  *seq;

    if (SHOW_ACTION) fprintf(stderr,"merchant_writes_PRes\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build PRes message */
    status = create_set_msg(&msg, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__pRes;
    status = make_set_msg(msg, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    pRes = safe_cast_PRes(msg);
    assert(pRes != 0);

    /* insert user-layer data */

    seq = pRes->pResData.pResPayloadSeq;
    seq->pResPayload.completionCode = asn1set__CompletionCode__capturePerformed;

    SET_PRESENT(seq->pResPayload, asn1set__PResPayload__results);
    seq->pResPayload.results.authStatus.authDate = time(0); /* made up */
    seq->pResPayload.results.authStatus.authCode = asn1set__AuthCode__approved;
    seq->pResPayload.results.authStatus.authRatio = AUTHAMT/PURCHAMT;
    SET_PRESENT(seq->pResPayload.results, asn1set__Results__authStatus);

    seq->pResPayload.results.capStatus.capDate = time(0); /* made up */
    seq->pResPayload.results.capStatus.capCode = asn1set__CapCode__success;
    seq->pResPayload.results.capStatus.capRatio = CAPAMT/PURCHAMT;
    SET_PRESENT(seq->pResPayload.results, asn1set__Results__capStatus);


    /* send PRes */
    status = send_set_msg(msg, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&msg);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}





int merchant_reads_CapRes PROTO_LIST((asn1set__XID*));
int
merchant_reads_CapRes(xid)
    asn1set__XID             *xid;         /* OUT */
{
    set_msg                   msg = 0;
    CapRes                    capRes = 0;
    int                       msg_type;
    char                     *cache_name;
    int                       i;
    asn1set__CapResItemSeq   *capSeq;

    if (SHOW_ACTION) fprintf(stderr,"merchant_reads_CapRes\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&msg, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get CapRes */
    status = receive_set_msg(msg, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__capRes);

    status = decode_set_msg(msg, app_ctx);
    assert(status == NO_ERROR);

    capRes = safe_cast_CapRes(msg);
    assert(capRes != 0);

    /* open the cache(s) associated with the XID(s) in the message */
    i = 0;
    for (capSeq = capRes->capResData.capResItemSeq; capSeq != 0; capSeq=capSeq->next)
    {

        /* copy the XID for use later */
        COPY_OCTET_STRING(heap, &xid[i], &capSeq->capResItem.transIDs.xid);

        cache_name = xid2filename(&xid[i]);
        status = create_cache_ctx_db(&msg_cache[i], cache_name);
        assert(status == NO_ERROR);
        ++i;
    }
    msg_cache[i] = 0; /* zero after the last cache */
    xid[i].length = 0; /* zero after the last xid */

    status = verify_set_msg(msg, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    i = 0;
    for (capSeq = capRes->capResData.capResItemSeq; capSeq != 0; capSeq = capSeq->next)
    {
        CACHE_delete(&msg_cache[i]);
        assert(status == NO_ERROR);
        ++i;
    }

    /* process CapRes */
    /* ... */

    status = delete_set_msg(&msg);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}


int gateway_writes_CapRes PROTO_LIST((asn1set__XID*));
int
gateway_writes_CapRes(xid)
    asn1set__XID             *xid;         /* IN */
{
    set_msg                   response = 0;
    CapRes                    capRes = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"gateway_writes_CapRes\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__capRes;
    /* build CapRes message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    status = make_set_msg(response, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    capRes = safe_cast_CapRes(response);
    assert(capRes != 0);

    /* insert user-layer data */
    capRes->capResData.capResItemSeq->capResItem.capResPayload.capCode =
                                                   asn1set__CapCode__success;

    /*  this pseudo-cache state is used just in this demo app, since for
     *  demo purposes I don't want to maintain a seperate application-level
     *  database  */
    status = CACHE_get(*msg_cache, DCAPAMT, &capRes->capResData.capResItemSeq->
                       capResItem.capResPayload.capAmt, heap);
    assert(status == NO_ERROR);

    /* send CapRes */
    status = send_set_msg(response, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}


int gateway_reads_CapReq PROTO_LIST((asn1set__XID*));
int
gateway_reads_CapReq(xid)
    asn1set__XID             *xid;         /* OUT */
{
    set_msg                   request = 0;
    CapReq                    capReq = 0;
    int                       msg_type;
    char                     *cache_name;
    int                       i;
    asn1set__CapItemSeq      *capItemSeq;
    struct CapTokenSeq_      *capTokenSeq;

    if (SHOW_ACTION) fprintf(stderr,"gateway_reads_CapReq\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get CapReq */
    status = receive_set_msg(request, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__capReq);

    status = decode_set_msg(request, app_ctx);
    assert(status == NO_ERROR);

    capReq = safe_cast_CapReq(request);
    assert(capReq != 0);

    /* open the cache(s) associated with the XID(s) in the message */
    i = 0;
    capTokenSeq = capReq->capTokenSeq;
    for (capItemSeq = capReq->capReqData.capItemSeq; capItemSeq != 0; capItemSeq = capItemSeq->next)
    {

        /* copy the XID for use later */
        COPY_OCTET_STRING(heap, &xid[i], &capItemSeq->capItem.transIDs.xid);

        cache_name = xid2filename(&xid[i]);
        status = create_cache_ctx_db(&msg_cache[i], cache_name);
        assert(status == NO_ERROR);

        /*  this pseudo-cache state is used just in this demo app, since for
         *  demo purposes I don't want to maintain a seperate application-level
         *  database  */
        status = CACHE_put(msg_cache[i], DCAPAMT, 
                           &capItemSeq->capItem.capPayload.capReqAmt);
        assert(status == NO_ERROR);

        ++i;
        capTokenSeq = capTokenSeq->next;
    }
    msg_cache[i] = 0; /* zero after the last cache */
    xid[i].length = 0; /* zero after the last xid */

    status = verify_set_msg(request, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    i = 0;
    for (capItemSeq = capReq->capReqData.capItemSeq; capItemSeq != 0; capItemSeq = capItemSeq->next)
    {
        status = CACHE_delete(&msg_cache[i]);
        assert(status == NO_ERROR);
        ++i;
    }

    /* process CapReq */
    /* ... */

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int merchant_writes_CapReq PROTO_LIST((asn1set__XID*, cert_identity, cert_identity));
int
merchant_writes_CapReq(xid, merchantSignatureCID, gatewayKeyExchangeCID)
    asn1set__XID             *xid;                     /* IN */
    cert_identity             merchantSignatureCID;    /* IN */
    cert_identity             gatewayKeyExchangeCID;   /* IN */
{
    set_msg                   request = 0;
    CapReq                    capReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_writes_CapReq\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build CapReq message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__capReq;
    status = make_set_msg(request, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    capReq = safe_cast_CapReq(request);
    assert(capReq != 0);

    /* insert user-layer data */

    capReq->capReqData.capRRTags.merTermIDs.merchantID.u.visibleString =
                                        "MerchantID";
    capReq->capReqData.capRRTags.merTermIDs.merchantID._choice =
                                        asn1set__MerchantID__visibleString;

    capReq->capReqData.capRRTags.currentDate = time(0);
    capReq->capReqData.capItemSeq->capItem.capPayload.capDate = time(0);

    capReq->capReqData.capItemSeq->capItem.capPayload.capReqAmt.currency = US_DOLLAR;
    capReq->capReqData.capItemSeq->capItem.capPayload.capReqAmt.amount = CAPAMT;
    capReq->capReqData.capItemSeq->capItem.capPayload.capReqAmt.amtExp10 = EXPONENT;

    capReq->merchantSignatureCID = merchantSignatureCID;
    capReq->gatewayKeyExchangeCID = gatewayKeyExchangeCID;

    /* send CapReq */
    status = send_set_msg(request, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}





int merchant_reads_AuthRes PROTO_LIST((asn1set__XID*, cert_identity*, cert_identity*));
int
merchant_reads_AuthRes(xid, merchantSignatureCID, gatewayKeyExchangeCID)
    asn1set__XID             *xid;                     /* OUT */
    cert_identity            *merchantSignatureCID;    /* OUT */
    cert_identity            *gatewayKeyExchangeCID;   /* OUT */
{
    set_msg                   response = 0;
    AuthRes                   authRes = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_reads_AuthRes\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get AuthRes */
    status = receive_set_msg(response, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__authRes);

    status = decode_set_msg(response, app_ctx);
    assert(status == NO_ERROR);

    authRes = safe_cast_AuthRes(response);
    assert(authRes != 0);


    /* copy the XID for use later */
    COPY_OCTET_STRING(heap, xid, &authRes->authResData.authTags.transIDs.xid);

    status = make_key_exchange_cert_identity_using_signature_cert_identity(
                 app_ctx, heap,
                 gatewayKeyExchangeCID, authRes->gatewaySignatureCID);
    assert(status == NO_ERROR);
    status = make_signature_cert_identity_using_key_exchange_cert_identity(
                 app_ctx, heap,
                 merchantSignatureCID, authRes->merchantKeyExchangeCID);
    assert(status == NO_ERROR);

    /* default capture amount is the same amount as authorization */
    CAPAMT = authRes->authResData.authResPayload.authHeader.authAmt.amount;

    /* open the cache associated with the XID in the message */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(response, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process AuthRes */
    /* ... */

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int gateway_writes_AuthRes PROTO_LIST((asn1set__XID*));
int
gateway_writes_AuthRes(xid)
    asn1set__XID             *xid;         /* IN */
{
    set_msg                   response = 0;
    AuthRes                   authRes = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"gateway_writes_AuthRes\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build AuthRes message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__authRes;
    status = make_set_msg(response, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    authRes = safe_cast_AuthRes(response);
    assert(authRes != 0);

    /* insert user-layer data */

    authRes->authResData.authResPayload.authHeader.authAmt.currency = US_DOLLAR;
    authRes->authResData.authResPayload.authHeader.authAmt.amount   = AUTHAMT;
    authRes->authResData.authResPayload.authHeader.authAmt.amtExp10 = EXPONENT;

    authRes->authResData.authResPayload.authHeader.authCode =
                                               asn1set__AuthCode__approved;

    if (ACQ_MSG != 0) {
        /* send message back to cardholder */
        if (SHOW_ACTION) fprintf(stderr, "sending AcqCardCodeMsg\n");
        authRes->acqCardMsg.acqCardCodeMsg.acqCardCode = asn1set__AcqCardCode__messageOfDay;
        authRes->acqCardMsg.acqCardCodeMsg.acqCardMsgData.acqCardURL =
                                              "http://www.terisa.com";
        SET_PRESENT(authRes->acqCardMsg.acqCardCodeMsg.acqCardMsgData,
                    asn1set__AcqCardMsgData__acqCardURL);
        authRes->acqCardMsg.acqCardCodeMsg.acqCardMsgData.acqCardText.u.visibleString =
                                        ACQ_MSG;
        authRes->acqCardMsg.acqCardCodeMsg.acqCardMsgData.acqCardText._choice =
                              asn1set__SETStringub_acqCardText__visibleString;
        SET_PRESENT(authRes->acqCardMsg.acqCardCodeMsg.acqCardMsgData,
                    asn1set__AcqCardMsgData__acqCardText);
        SET_PRESENT(*authRes, AuthRes__acqCardMsg);
    }

    if (USE_PANTOKEN) {
        /* send PANToken back to merchant */
        if (SHOW_ACTION) fprintf(stderr, "Sending PANToken to merchant\n");
        SET_PRESENT(*authRes, AuthRes__panToken);
    }


    if (USE_CAPTOKEN) {
      /* send CapToken to merchant (to be included in CapReq) */
      if (SHOW_ACTION) fprintf(stderr,"using CapToken\n");
      SET_PRESENT(*authRes, AuthRes__capToken);
      authRes->capToken.capTokenData.authAmt.currency  = US_DOLLAR;
      authRes->capToken.capTokenData.authAmt.amount    = AUTHAMT;
      authRes->capToken.capTokenData.authAmt.amtExp10  = EXPONENT;
      authRes->capToken.capTokenData.tokenOpaque.object  = (UCHAR*)"\005\000";
      authRes->capToken.capTokenData.tokenOpaque.length  = 2;
      SET_PRESENT(authRes->capToken, CapToken__capTokenData);

      /* include PANToken with CapToken? */
      SET_PRESENT(authRes->capToken, CapToken__panToken);
    }

    if (USE_RECURRING) {
    }

    /* send AuthRes */
    status = send_set_msg(response, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}


int gateway_reads_AuthReq PROTO_LIST((asn1set__XID*));
int
gateway_reads_AuthReq(xid)
    asn1set__XID             *xid;         /* OUT */
{
    set_msg                   request = 0;
    AuthReq                   authReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"gateway_reads_AuthReq\n");

    status = startup(ROLE_GATEWAY, GATEWAY_SW, GATEWAY_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get AuthReq */
    status = receive_set_msg(request, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__authReq);

    status = decode_set_msg(request, app_ctx);
    assert(status == NO_ERROR);

    authReq = safe_cast_AuthReq(request);
    assert(authReq != 0);

    /* default auth amount returned in AuthRes */
    AUTHAMT = authReq->authReqData.authReqItem.authReqPayload.authReqAmt.amount;

    /* copy the XID for use later */
    COPY_OCTET_STRING(heap, xid, &authReq->authReqData.authReqItem.authTags.transIDs.xid);

    /* open the cache associated with the XID in the message */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(request, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);


    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process AuthReq */
    /* ... */

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}






int merchant_writes_AuthReq PROTO_LIST((asn1set__XID*));
int
merchant_writes_AuthReq(xid)
    asn1set__XID             *xid;         /* IN */
{
    set_msg                   msg = 0;
    AuthReq                   authReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_writes_AuthReq\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* build AuthReq message */
    status = create_set_msg(&msg, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__authReq;
    status = make_set_msg(msg, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    authReq = safe_cast_AuthReq(msg);
    assert(authReq != 0);

    /* insert user-layer data */

    authReq->authReqData.authReqItem.authTags.authRRTags.merTermIDs.merchantID.u.visibleString = "MerchantID";
    authReq->authReqData.authReqItem.authTags.authRRTags.merTermIDs.merchantID._choice =
                                         asn1set__MerchantID__visibleString;

    authReq->authReqData.authReqItem.authTags.authRRTags.currentDate = time(0);

    authReq->authReqData.authReqItem.authReqPayload.authReqAmt.currency  = US_DOLLAR;
    authReq->authReqData.authReqItem.authReqPayload.authReqAmt.amount    = AUTHAMT;
    authReq->authReqData.authReqItem.authReqPayload.authReqAmt.amtExp10  = EXPONENT;
    authReq->authReqData.captureNow = 0; /* FALSE */

    /* this information isn't used for AuthTokens */
    authReq->hodInput.purchAmt.currency  = US_DOLLAR;
    authReq->hodInput.purchAmt.amount    = PURCHAMT;
    authReq->hodInput.purchAmt.amtExp10  = EXPONENT;
    authReq->hodInput.od.length  = strlen(ORDER_DESC);
    authReq->hodInput.od.value   = (UCHAR*)ORDER_DESC;

    /* send AuthReq */
    status = send_set_msg(msg, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);


    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&msg);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}



int merchant_reads_PReq PROTO_LIST((asn1set__XID*));
int
merchant_reads_PReq(xid)
    asn1set__XID             *xid;         /* OUT */
{
    set_msg                   msg = 0;
    PReq                      pReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_reads_PReq\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&msg, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get PReq */
    status = receive_set_msg(msg, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__pReq);

    status = decode_set_msg(msg, app_ctx);
    assert(status == NO_ERROR);

    pReq = safe_cast_PReq(msg);
    assert(pReq != 0);

    pReq->order_description.length = strlen(ORDER_DESC);
    pReq->order_description.value  = (UCHAR*)ORDER_DESC;
    pReq->purchAmt.currency  = US_DOLLAR;
    pReq->purchAmt.amount    = PURCHAMT;
    pReq->purchAmt.amtExp10  = EXPONENT;

    if (USE_RECURRING) {
        if (SHOW_ACTION) fprintf(stderr, "recurring total: %d\n", TOTAL_TRANS);
        pReq->installRecurData.installRecurInd.u.installTotalTrans = TOTAL_TRANS;
        pReq->installRecurData.installRecurInd._choice =
                                 asn1set__InstallRecurInd__installTotalTrans;
        SET_PRESENT(*pReq, PReq__installRecurData);
    }


    /* copy the XID for use later */
    COPY_OCTET_STRING(heap, xid, &pReq->oiData.transIDs.xid);

    /* open the cache associated with the XID in the message */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(msg, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process PReq */
    /* ... */

    status = delete_set_msg(&msg);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int cardholder_writes_PReq PROTO_LIST((asn1set__LocalID*));
int
cardholder_writes_PReq(lidc)
    asn1set__LocalID         *lidc;        /* IN */
{
    set_msg                   request = 0;
    PReq                      pReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"cardholder_writes_PReq\n");

    status = startup(ROLE_CARDHOLDER, CARDHOLDER_SW, CARDHOLDER_LOG);
    assert(status == NO_ERROR);

    /* build PReq */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = lid2filename(lidc);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__pReq;
    status = make_set_msg(request, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    pReq = safe_cast_PReq(request);
    assert(pReq != 0);

    /* insert user-layer data */

    pReq->piData.panData.pan = PAN;
    pReq->piData.panData.cardExpiry = CARDEXPIRY;
    pReq->piData.panData.panSecret.length = strlen(PANSECRET);
    pReq->piData.panData.panSecret.value  = (UCHAR*)PANSECRET;

    pReq->piData.piHead.inputs.purchAmt.currency  = US_DOLLAR;
    pReq->piData.piHead.inputs.purchAmt.amount    = PURCHAMT;
    pReq->piData.piHead.inputs.purchAmt.amtExp10  = EXPONENT;

    pReq->order_description.length = strlen(ORDER_DESC);
    pReq->order_description.value  = (UCHAR*)ORDER_DESC;

    pReq->oiData.bin = BIN;

    pReq->oiData.brandID.u.visibleString = BRANDID;
    pReq->oiData.brandID._choice = asn1set__BrandID__visibleString;

    if (USE_RECURRING) {
        if (SHOW_ACTION) fprintf(stderr, "recurring total: %d\n", TOTAL_TRANS);
        pReq->piData.piHead.installRecurData.installRecurInd.u.installTotalTrans = TOTAL_TRANS;
        pReq->piData.piHead.installRecurData.installRecurInd._choice =
                                  asn1set__InstallRecurInd__installTotalTrans;
        SET_PRESENT(pReq->piData.piHead, asn1set__PIHead__installRecurData);
    }

    SET_SELF_ID_car_sig(&pReq->cardholderSignatureCID);

    /* send PReq */
    status = send_set_msg(request, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}



int cardholder_reads_PInitRes PROTO_LIST((asn1set__LocalID*));
int
cardholder_reads_PInitRes(lidc)
    asn1set__LocalID         *lidc;        /* OUT */
{
    set_msg                   response = 0;
    PInitRes                  pInitRes = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"cardholder_reads_PInitRes\n");

    status = startup(ROLE_CARDHOLDER, CARDHOLDER_SW, CARDHOLDER_LOG);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get PInitRes */
    status = receive_set_msg(response, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__pInitRes);

    status = decode_set_msg(response, app_ctx);
    assert(status == NO_ERROR);

    pInitRes = safe_cast_PInitRes(response);
    assert(pInitRes != 0);

    /* copy the LocalID for use later */
    COPY_OCTET_STRING(heap, lidc, &pInitRes->pInitResData.transIDs.lid_C);

    /* open the cache associated with the LocalID in the message */
    cache_name = lid2filename(lidc);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    status = verify_set_msg(response, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    /* process PInitRes */
    /* ... */

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int merchant_writes_PInitRes PROTO_LIST((asn1set__XID*));
int
merchant_writes_PInitRes(xid)
    asn1set__XID             *xid;         /* IN */
{
    set_msg                   response = 0;
    PInitRes                  pInitRes = 0;
    int                       msg_type;
    asn1set__LocalID          lidm;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_writes_PInitRes\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* open the cache associated with the appropriate transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    if (USE_LID_M) {
        if (SHOW_ACTION) fprintf(stderr,"using LocalID-M\n");
        /* a LocalID may be assigned (but it's not necessary) */
        lidm.length = strlen(LOCALID_M);
        lidm.value = (UCHAR*)LOCALID_M;

        /* put the LocalID into the cache (so that setref uses this info) */
        status = CACHE_put(msg_cache[0], STATE_LOCALID_M, &lidm);
        assert(status == NO_ERROR);
    }

    /* insert GATEWAY_DN into cache, based on BrandID in PInitReq */
    {
        cert_identity     payid;
        SET_PEER_ID_pay_key(&payid);
        status = CACHE_put(msg_cache[0], STATE_GATEWAY_KEYID, payid);
        assert(status == NO_ERROR);
    }

    /* build PInitRes message */
    status = create_set_msg(&response, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__pInitRes;
    status = make_set_msg(response, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    pInitRes = safe_cast_PInitRes(response);
    assert(pInitRes != 0);

    /* insert user-layer data */
    pInitRes->pInitResData.transIDs.pReqDate = time(0);
    SET_SELF_ID_mer_sig(&pInitRes->merchantSignatureCID);

    /* send PInitRes */
    status = send_set_msg(response, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&response);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}


int merchant_reads_PInitReq PROTO_LIST((asn1set__XID*));
int
merchant_reads_PInitReq(xid)
    asn1set__XID             *xid;         /* OUT */
{
    set_msg                   request = 0;
    PInitReq                  pInitReq = 0;
    int                       msg_type;
    char                     *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"merchant_reads_PInitReq\n");

    status = startup(ROLE_MERCHANT, MERCHANT_SW, MERCHANT_LOG);
    assert(status == NO_ERROR);

    /* first, an XID must be assigned */
    xid->length = strlen(XID);
    xid->value = AC_malloc(heap, xid->length);
                       /* this should be random */
    memcpy(xid->value, XID, xid->length);

    /* create the cache to be associated with this transaction */
    cache_name = xid2filename(xid);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* place the xid into the cache */
    status = CACHE_put(msg_cache[0], STATE_XID, xid);
    assert(status == NO_ERROR);

    /* create a blank SET message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    /* get PInitReq */
    status = receive_set_msg(request, app_ctx, instream, &msg_type);
    assert(status == NO_ERROR);
    assert(msg_type == asn1set__Message__pInitReq);

    status = decode_set_msg(request, app_ctx);
    assert(status == NO_ERROR);

    status = verify_set_msg(request, app_ctx, &msg_cache[0]);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    pInitReq = safe_cast_PInitReq(request);
    assert(pInitReq != 0);

    /* process PInitReq */
    /* ... */

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




int cardholder_writes_PInitReq PROTO_LIST((asn1set__LocalID*));
int
cardholder_writes_PInitReq(lidc)
    asn1set__LocalID       *lidc;        /* OUT */
{
    set_msg                 request  = 0;
    PInitReq                pInitReq = 0;
    int                     msg_type;
    char                   *cache_name;

    if (SHOW_ACTION) fprintf(stderr,"cardholder_writes_PInitReq\n");

    status = startup(ROLE_CARDHOLDER, CARDHOLDER_SW, CARDHOLDER_LOG);
    assert(status == NO_ERROR);

    /* first, a LocalID must be assigned */
    lidc->length = strlen(LOCALID_C);
    lidc->value = AC_malloc(heap, lidc->length);
    memcpy(lidc->value, LOCALID_C, lidc->length);

    /* second, the cache to be associated with this LocalID is created */
    cache_name = lid2filename(lidc);
    status = create_cache_ctx_db(&msg_cache[0], cache_name);
    assert(status == NO_ERROR);

    /* third, the LocalID is placed in the cache */
    status = CACHE_put(msg_cache[0], STATE_LOCALID_C, lidc);
    assert(status == NO_ERROR);

    /* build PInitReq message */
    status = create_set_msg(&request, app_ctx, heap);
    assert(status == NO_ERROR);

    msg_type = asn1set__Message__pInitReq;
    status = make_set_msg(request, app_ctx, &msg_cache[0], msg_type);
    assert(status == NO_ERROR);

    pInitReq = safe_cast_PInitReq(request);
    assert(pInitReq != 0);

    /* insert user-layer data */

    pInitReq->messageWrapper.message.u.pInitReq.language = ENGLISH;

    pInitReq->messageWrapper.message.u.pInitReq.brandID.u.visibleString =
                                                    BRANDID;
    pInitReq->messageWrapper.message.u.pInitReq.brandID._choice =
                                           asn1set__BrandID__visibleString;
    
    pInitReq->messageWrapper.message.u.pInitReq.bin = BIN;

    if (USE_LID_M) {
        if (SHOW_ACTION) fprintf(stderr,"using LocalID-M\n");
        pInitReq->messageWrapper.messageHeader.messageIDs.lid_M.length = strlen(LOCALID_M);
        pInitReq->messageWrapper.messageHeader.messageIDs.lid_M.value = (UCHAR*)LOCALID_M;
        SET_PRESENT(pInitReq->messageWrapper.messageHeader.messageIDs, asn1set__MessageIDs__lid_M);
    }

    /* send PInitReq */
    status = send_set_msg(request, app_ctx, &msg_cache[0], outstream);
    assert(status == NO_ERROR);

    status = CACHE_delete(&msg_cache[0]);
    assert(status == NO_ERROR);

    status = delete_set_msg(&request);
    assert(status == NO_ERROR);

    status = finish();
    assert(status == NO_ERROR);

    return NO_ERROR;
}




VOID usage PROTO_LIST((char *prog));
VOID
usage(prog)
    char *prog;
{
    fprintf(stderr, "usage: %s [-Dopt(s)] -{wr}msg ...\n", prog);
#define USAGE(msg)  fprintf(stderr, "        %s\n", (msg))
    USAGE("msg={PInitReq, PInitRes, PReq, PRes, AuthReq,");
    USAGE("     AuthRes, CapReq, CapRes, InqReq, InqRes,");
    USAGE("     AuthRevReq, AuthRevRes, CapRevReq, CapRevRes,");
    USAGE("     CredReq, CredRes, CredRevReq, CredRevRes,");
    USAGE("     BatchAdminReq, BatchAdminRes,");
    USAGE("     CardCInitReq, CardCInitRes, Me_AqCInitReq, Me_AqCInitRes,");
    USAGE("     RegFormReq, RegFormReq, CertReq, CertRes, ");
    USAGE("     CertInqReq, CertInqRes, ");
    USAGE("     Error}");
    USAGE("-wAuthReq:        xid=hex AuthAmt=amt OrderDesc=txt PurchAmt=amt");
    USAGE("-wAuthRes:        xid=hex AcqMsg=txt AuthAmt=amt CapToken PANToken");
    USAGE("                  recurring");
    USAGE("-wAuthRevRes:     xid=hex AuthAmt=amt AuthToken CapToken PANToken");
    USAGE("-wCapReq:         xid=hex CapAmt=amt");
    USAGE("-wCapRes:         xid=hex");
    USAGE("-wPInitReq:       BrandID=txt lidm");
    USAGE("-rPInitReq:       BrandID=txt");
    USAGE("-wPInitRes:       xid=hex lidm");
    USAGE("-wPReq:           lidc=hex BrandID=txt OrderDesc=txt PurchAmt=amt");
    USAGE("                  recurring TotalRecur");
    USAGE("-rPReq:           BrandID=txt OrderDesc=txt PurchAmt=amt");
    USAGE("                  recurring TotalRecur=val");
    USAGE("-wPRes:           xid=hex");
    USAGE("-wInqReq:         sign");
    USAGE("-wInqRes:         xid=hex");
    USAGE("-wCardCInitReq:   BrandID=txt");
    USAGE("-wCardCInitRes:   lidca=hex");
    USAGE("-wRegFormReq:     lidee=hex CertReqType=enum");
    USAGE("-wRegFormRes:     lidca=hex referral");
    USAGE("-wMe_AqCInitReq:  BrandID=txt CertReqType=enum MerchantBIN=val");
    USAGE("                  MerchantID=txt");
    USAGE("-wMe_AqCInitRes:  lidca=hex");
    USAGE("-wCertReq         lidee=hex");
    USAGE("-wCertRes         lidca=hex");
    USAGE("-wError:          error=val");
    USAGE("enum={cardInitialSig,");
    USAGE("      merInitialSig, merInitialEnc, merInitialBoth,");
    USAGE("      pgwyInitialSig, pgwyInitialEnc, pgwyInitialBoth,");
    USAGE("      cardRenewalSig,"); 
    USAGE("      merRenewalSig, merRenewalEnc, merRenewalBoth,");
    USAGE("      pgwyRenewalSig, pgwyRenewalEnc, pgwyRenewalBoth}");
#undef USAGE
    exit(1);
}

int certreqtype PROTO_LIST((char *argv0, char *txt));
int
certreqtype(argv0, txt)
   char *argv0;
   char *txt;
{

#define IF(msg)        if(strcasecmp((msg),txt)==0)
    IF      ("cardInitialSig")   return asn1set__RequestType__cardInitialSig;
    else IF ("merInitialSig")    return asn1set__RequestType__merInitialSig;
    else IF ("merInitialEnc")    return asn1set__RequestType__merInitialEnc;
    else IF ("merInitialBoth")   return asn1set__RequestType__merInitialBoth;
    else IF ("pgwyInitialSig")   return asn1set__RequestType__pgwyInitialSig;
    else IF ("pgwyInitialEnc")   return asn1set__RequestType__pgwyInitialEnc;
    else IF ("pgwyInitialBoth")  return asn1set__RequestType__pgwyInitialBoth;
    else IF ("cardRenewalSig")   return asn1set__RequestType__cardRenewalSig;
    else IF ("merRenewalSig")    return asn1set__RequestType__merRenewalSig;
    else IF ("merRenewalEnc")    return asn1set__RequestType__merRenewalEnc;
    else IF ("merRenewalBoth")   return asn1set__RequestType__merRenewalBoth;
    else IF ("pgwyRenewalSig")   return asn1set__RequestType__pgwyRenewalSig;
    else IF ("pgwyRenewalEnc")   return asn1set__RequestType__pgwyRenewalEnc;
    else IF ("pgwyRenewalBoth")  return asn1set__RequestType__pgwyRenewalBoth;
    else usage(argv0);
#undef IF

        return 0;
}

int set_id PROTO_LIST((char *id, char **str, void *asn));
int
set_id(id, str, asn)
  char  *id;
  char **str;
  void  *asn;
{
    int     len;
    UCHAR  *val;
    int     i, j;
    unsigned int num;

    len = strlen(id);
    val = (UCHAR*)malloc(len);
   
    for (i = 0, j = 0; j < len; ++i, j += 2) {
       sscanf(&id[j], "%02x", &num); 
       val[i] = num;
    }
    *str = (char*)val;
    ((asn__octet_string*)asn)->length = len;
    ((asn__octet_string*)asn)->value = val;

    return 0;    
}



int main PROTO_LIST((int,char**));
int
main(argc, argv)
  int argc;
  char **argv;
{
    int                i = 1;
    int                nread;
    char               buffer[8192];
    asn1set__LocalID   lidca;
    asn1set__LocalID   lidee;
    asn1set__LocalID   lidc;
    asn1set__RRPID     rrpid;
    asn1set__XID       xid[16];
    cert_identity      merchantSignatureCID;
    cert_identity      gatewayKeyExchangeCID;

    if (argc < 2) usage(argv[0]);

    memset(xid, 0, sizeof(xid));

#ifdef WIN32
    /* set stdin and stdout to binary mode */
    _setmode( _fileno(stdin), _O_BINARY);
    _setmode( _fileno(stdout), _O_BINARY);
#endif /* _WINDOWS */

    status = create_pool_ctx(&heap);
    assert(status == NO_ERROR);

    /* defaults */

    for (i = 1; i < argc; ++i) {

        if (argv[i][0] != '-') usage(argv[0]);

        /* the output from the previous write becomes the next input */
        if (outstream != 0) {
            if (instream != 0) (VOID)IO_delete(&instream);
            instream = outstream;
            outstream = 0;
            (VOID)IO_fseek(instream, 0, FSEEK_BOF);
        }
    
#define IF(msg)        if(strcasecmp((msg),&argv[i][2])==0)
#define IFE(msg)       if(strncasecmp((msg),&argv[i][2],strlen(msg))==0)
#define GET_VALUE      (strchr(argv[i],'=')+1)

        switch (argv[i][1]) {
        case 'r':   /* READ */
    
            if (instream == 0) {
                status = create_stdio_ctx(&instream, stdin);
                assert(status == NO_ERROR);
            }
    
    IF      ("pinitreq")       status = merchant_reads_PInitReq(&xid[0]);
    else IF ("pinitres")       status = cardholder_reads_PInitRes(&lidc);
    else IF ("preq")           status = merchant_reads_PReq(&xid[0]); 
    else IF ("pres")           status = cardholder_reads_PRes(&lidc);
    else IF ("inqreq")         status = merchant_reads_InqReq(&xid[0]);
    else IF ("inqres")         status = cardholder_reads_InqRes(&lidc);
    else IF ("authreq")        status = gateway_reads_AuthReq(&xid[0]);
    else IF ("authres")        status = merchant_reads_AuthRes(&xid[0],
                                            &merchantSignatureCID,
                                            &gatewayKeyExchangeCID);
    else IF ("authrevreq")     status = gateway_reads_AuthRevReq(&xid[0]);
    else IF ("authrevres")     status = merchant_reads_AuthRevRes(&xid[0],
                                            &merchantSignatureCID,
                                            &gatewayKeyExchangeCID);
    else IF ("capreq")         status = gateway_reads_CapReq(&xid[0]);
    else IF ("capres")         status = merchant_reads_CapRes(&xid[0]);
    else IF ("caprevreq")      status = gateway_reads_CapRevReq(&xid[0]);
    else IF ("caprevres")      status = merchant_reads_CapRevRes(&xid[0]);
    else IF ("credreq")        status = gateway_reads_CredReq(&xid[0]);
    else IF ("credres")        status = merchant_reads_CredRes(&xid[0]);
    else IF ("credrevreq")     status = gateway_reads_CredRevReq(&xid[0]);
    else IF ("credrevres")     status = merchant_reads_CredRevRes(&xid[0]);
    else IF ("pcertreq")       status = gateway_reads_PCertReq(&rrpid);
    else IF ("pcertres")       status = merchant_reads_PCertRes(&rrpid);
    else IF ("batchadminreq")  status = gateway_reads_BatchAdminReq(&rrpid);
    else IF ("batchadminres")  status = merchant_reads_BatchAdminRes(&xid[0]);
    else IF ("cardcinitreq")   status = cca_reads_CardCInitReq(&lidca);
    else IF ("cardcinitres")   status = cardholder_reads_CardCInitRes(&lidee);
    else IF ("regformreq")     status = cca_reads_RegFormReq(&lidca);
    else IF ("regformres")     status = cardholder_reads_RegFormRes(&lidee);
    else IF ("certreq")        status = ca_reads_CertReq(&lidca);
    else IF ("certres")        status = ee_reads_CertRes(&lidee);
    else IF ("certinqreq")     status = ca_reads_CertInqReq(&lidca);
    else IF ("certinqres")     status = ee_reads_CertInqRes(&lidee);
    else IF ("me_aqcinitreq")  status = ca_reads_Me_AqCInitReq(&lidca);
    else IF ("me_aqcinitres")  status = ee_reads_Me_AqCInitRes(&lidee);
    else IF ("error")          status = read_Error();
    else usage(argv[0]);
            assert(status == NO_ERROR);
    
            break;
    
        case 'w':   /* WRITE */
    
            status = create_memio_ctx(&outstream, 0);
            assert(status == NO_ERROR);
    
    IF      ("pinitreq")       status = cardholder_writes_PInitReq(&lidc);
    else IF ("pinitres")       status = merchant_writes_PInitRes(&xid[0]);
    else IF ("preq")           status = cardholder_writes_PReq(&lidc);
    else IF ("pres")           status = merchant_writes_PRes(&xid[0]);
    else IF ("inqreq")         status = cardholder_writes_InqReq(&lidc);
    else IF ("inqres")         status = merchant_writes_InqRes(&xid[0]);
    else IF ("authreq")        status = merchant_writes_AuthReq(&xid[0]);
    else IF ("authres")        status = gateway_writes_AuthRes(&xid[0]);
    else IF ("authrevreq")     status = merchant_writes_AuthRevReq(&xid[0]);
    else IF ("authrevres")     status = gateway_writes_AuthRevRes(&xid[0]);
    else IF ("capreq")         status = merchant_writes_CapReq(&xid[0],
                                               merchantSignatureCID,
                                               gatewayKeyExchangeCID);
    else IF ("capres")         status = gateway_writes_CapRes(&xid[0]);
    else IF ("caprevreq")      status = merchant_writes_CapRevReq(&xid[0],
                                               merchantSignatureCID,
                                               gatewayKeyExchangeCID);
    else IF ("caprevres")      status = gateway_writes_CapRevRes(&xid[0]);
    else IF ("credreq")        status = merchant_writes_CredReq(&xid[0],
                                               merchantSignatureCID,
                                               gatewayKeyExchangeCID);
    else IF ("credres")        status = gateway_writes_CredRes(&xid[0]);
    else IF ("credrevreq")     status = merchant_writes_CredRevReq(&xid[0],
                                               merchantSignatureCID,
                                               gatewayKeyExchangeCID);
    else IF ("credrevres")     status = gateway_writes_CredRevRes(&xid[0]);
    else IF ("pcertreq")       status = merchant_writes_PCertReq(&rrpid);
    else IF ("pcertres")       status = gateway_writes_PCertRes(&rrpid);
    else IF ("batchadminreq")  status = merchant_writes_BatchAdminReq(&xid[0]);
    else IF ("batchadminres")  status = gateway_writes_BatchAdminRes(&rrpid);
    else IF ("cardcinitreq")   status = cardholder_writes_CardCInitReq(&lidee);
    else IF ("cardcinitres")   status = cca_writes_CardCInitRes(&lidca);
    else IF ("regformreq")     status = cardholder_writes_RegFormReq(&lidee);
    else IF ("regformres")     status = cca_writes_RegFormRes(&lidca);
    else IF ("certreq")        status = ee_writes_CertReq(&lidee);
    else IF ("certres")        status = ca_writes_CertRes(&lidca);
    else IF ("certinqreq")     status = ee_writes_CertInqReq(&lidee);
    else IF ("certinqres")     status = ca_writes_CertInqRes(&lidca);
    else IF ("me_Aqcinitreq")  status = ee_writes_Me_AqCInitReq(&lidee);
    else IF ("me_Aqcinitres")  status = ca_writes_Me_AqCInitRes(&lidca);
    else IF ("error")          status = write_Error();
    else usage(argv[0]);
            assert(status == NO_ERROR);
    
            break;

        case 'D':   /* DEFINE */ 

  IFE      ("acqmsg=")        ACQ_MSG        = GET_VALUE;
  else IFE ("authamt=")       AUTHAMT        = (int)(atof(GET_VALUE)*100);
  else IF  ("authtoken")      USE_AUTHTOKEN  = 1;
  else IFE ("brandid=")       BRANDID        = GET_VALUE;
  else IFE ("capamt=")        CAPAMT         = (int)(atof(GET_VALUE)*100);
  else IF  ("captoken")       USE_CAPTOKEN   = 1;
  else IFE ("certreqtype=")   CERT_REQ_TYPE  = certreqtype(argv[0], GET_VALUE);
  else IFE ("error=")         ERROR          = atoi(GET_VALUE);
  else IFE ("lidc=")          set_id(GET_VALUE, &LOCALID_C, &lidc);
  else IFE ("lidca=")         set_id(GET_VALUE, &LOCALID_CA, &lidca);
  else IFE ("lidee=")         set_id(GET_VALUE, &LOCALID_EE, &lidee);
  else IF  ("lidm")           USE_LID_M      = 1;
  else IFE ("orderdesc=")     ORDER_DESC     = GET_VALUE;
  else IFE ("merchantbin=")   MERCHANTBIN    = GET_VALUE;
  else IFE ("merchantid=")    MERCHANTID     = GET_VALUE;
  else IF  ("pantoken")       USE_PANTOKEN   = 1;
  else IFE ("purchamt=")      PURCHAMT       = (int)(atof(GET_VALUE)*100);
  else IF  ("recurring")      USE_RECURRING  = 1;
  else IF  ("referral")       USE_REFERRAL   = 1;
  else IF  ("sign")           USE_SIGNATURE  = 1;
  else IFE ("totalrecur=")    TOTAL_TRANS    = atoi(GET_VALUE);
  else IFE ("xid=")           set_id(GET_VALUE, &XID, &xid);
  else usage(argv[0]);

            break;

        default:
            usage(argv[0]);
            break;
        }
    }

#undef GET_VALUE
#undef IFE
#undef IF

    if (outstream != 0) {
        IO_fseek(outstream, 0, FSEEK_BOF);
        while ((nread = IO_fread(buffer, 1, sizeof(buffer), outstream)) != 0) {
            fwrite(buffer, 1, nread, stdout);
        }
        (VOID)IO_delete(&outstream);
    }


    status = IO_delete(&instream);
    assert(status == NO_ERROR);

    status = AC_delete(&heap);
    assert(status == NO_ERROR);

    
    return NO_ERROR;
}


